@@ -6,7 +6,6 @@
 #include "cache.h"
 #include "colormanager.h"
 #include "configcontainer.h"
-#include "configpaths.h"
 #include "feedcontainer.h"
 #include "filtercontainer.h"
 #include "fslock.h"
@@ -20,13 +19,15 @@
 
 namespace newsboat {
 
+class CliArgsParser;
+class ConfigPaths;
 class View;
 
 class CurlHandle;
 
 class Controller {
 public:
-	Controller();
+	Controller(ConfigPaths& configpaths);
 	~Controller();
 	void set_view(View* vv);
 	View* get_view()
@@ -95,8 +96,18 @@ public:
 		return api;
 	}
 
+	RegexManager& get_regexmanager()
+	{
+		return rxman;
+	}
+
+	FilterContainer& get_filtercontainer()
+	{
+		return filters;
+	}
+
 private:
-	void import_opml(const std::string& filename);
+	void import_opml(const std::string& opmlFile, const std::string& urlFile);
 	void export_opml();
 	void rec_find_rss_outlines(xmlNode* node, std::string tag);
 	int execute_commands(const std::vector<std::string>& cmds);
@@ -121,7 +132,7 @@ private:
 
 	std::unique_ptr<FsLock> fslock;
 
-	ConfigPaths configpaths;
+	ConfigPaths& configpaths;
 
 	std::unique_ptr<Reloader> reloader;
 
