@@ -5,6 +5,8 @@
 #include <grp.h>
 
 #include "configcontainer.h"
+#include "listformatter.h"
+#include "listwidget.h"
 #include "formaction.h"
 
 namespace newsboat {
@@ -29,11 +31,12 @@ public:
 	std::string title() override;
 
 private:
-	void process_operation(Operation op,
+	bool process_operation(Operation op,
 		bool automatic = false,
 		std::vector<std::string>* args = nullptr) override;
+	void update_title(const std::string& working_directory);
 
-	std::string add_directory(std::string dirname);
+	void add_directory(ListFormatter& listfmt, std::string dirname);
 	std::string get_rwx(unsigned short val);
 
 	std::string get_owner(uid_t uid);
@@ -43,6 +46,7 @@ private:
 	std::string cwd;
 	std::string dir;
 
+	ListWidget files_list;
 };
 
 } // namespace newsboat
