@@ -1,8 +1,12 @@
 #ifndef NEWSBOAT_FEEDLISTFORMACTION_H_
 #define NEWSBOAT_FEEDLISTFORMACTION_H_
 
+#include "3rd-party/optional.hpp"
+
+#include "configcontainer.h"
 #include "history.h"
 #include "listformaction.h"
+#include "listwidget.h"
 #include "matcher.h"
 #include "regexmanager.h"
 #include "view.h"
@@ -16,14 +20,14 @@ public:
 	FeedListFormAction(View*,
 		std::string formstr,
 		Cache* cc,
-		FilterContainer* f,
-		ConfigContainer* cfg);
+		FilterContainer& f,
+		ConfigContainer* cfg,
+		RegexManager& r);
 	~FeedListFormAction() override;
 	void prepare() override;
 	void init() override;
 	void set_feedlist(std::vector<std::shared_ptr<RssFeed>>& feeds);
 	void update_visible_feeds(std::vector<std::shared_ptr<RssFeed>>& feeds);
-	void set_tags(const std::vector<std::string>& t);
 	KeyMapHintEntry* get_keymap_hint() override;
 	std::shared_ptr<RssFeed> get_feed();
 
@@ -45,11 +49,15 @@ public:
 
 	void mark_pos_if_visible(unsigned int pos);
 
-	void set_regexmanager(RegexManager* r);
-
 private:
+	void register_format_styles();
+
+	void update_form_title(unsigned int width);
+
+	unsigned int count_unread_feeds();
+
 	int get_pos(unsigned int realidx);
-	void process_operation(Operation op,
+	bool process_operation(Operation op,
 		bool automatic = false,
 		std::vector<std::string>* args = nullptr) override;
 
@@ -72,13 +80,10 @@ private:
 		unsigned int width);
 
 	bool zero_feedpos;
-	unsigned int feeds_shown;
-	bool quit;
 	std::vector<FeedPtrPosPair> visible_feeds;
 	std::string tag;
-	std::vector<std::string> tags;
 
-	Matcher m;
+	Matcher matcher;
 	bool apply_filter;
 
 	History filterhistory;
@@ -88,16 +93,13 @@ private:
 	unsigned int filterpos;
 	bool set_filterpos;
 
-	RegexManager* rxman;
+	RegexManager& rxman;
 
-	unsigned int old_width;
+	FilterContainer& filters;
 
-	unsigned int unread_feeds;
-	unsigned int total_feeds;
+	nonstd::optional<FeedSortStrategy> old_sort_strategy;
 
-	FilterContainer* filters;
-
-	std::string old_sort_order;
+	ListWidget feeds_list;
 };
 
 } // namespace newsboat
