@@ -5,6 +5,8 @@
 #include <grp.h>
 
 #include "configcontainer.h"
+#include "listformatter.h"
+#include "listwidget.h"
 #include "formaction.h"
 
 namespace newsboat {
@@ -33,11 +35,12 @@ public:
 	std::string title() override;
 
 private:
-	void process_operation(Operation op,
+	bool process_operation(Operation op,
 		bool automatic = false,
 		std::vector<std::string>* args = nullptr) override;
+	void update_title(const std::string& working_directory);
 
-	std::string add_file(std::string filename);
+	void add_file(ListFormatter& listfmt, std::string filename);
 	std::string get_filename_suggestion(const std::string& s);
 	std::string get_rwx(unsigned short val);
 
@@ -52,6 +55,7 @@ private:
 	std::string dir;
 	std::string default_filename;
 
+	ListWidget files_list;
 };
 
 } // namespace newsboat
