@@ -3,9 +3,12 @@
 
 #include <assert.h>
 
+#include "3rd-party/optional.hpp"
+
 #include "history.h"
 #include "listformaction.h"
 #include "listformatter.h"
+#include "listwidget.h"
 #include "regexmanager.h"
 #include "view.h"
 
@@ -13,15 +16,16 @@ namespace newsboat {
 
 typedef std::pair<std::shared_ptr<RssItem>, unsigned int> ItemPtrPosPair;
 
-enum class InvalidationMode { PARTIAL, COMPLETE };
+enum class InvalidationMode { NONE, PARTIAL, COMPLETE };
 
 class ItemListFormAction : public ListFormAction {
 public:
 	ItemListFormAction(View*,
 		std::string formstr,
 		Cache* cc,
-		FilterContainer* f,
-		ConfigContainer* cfg);
+		FilterContainer& f,
+		ConfigContainer* cfg,
+		RegexManager& r);
 	~ItemListFormAction() override;
 	void prepare() override;
 	void init() override;
@@ -74,10 +78,10 @@ public:
 
 	void recalculate_form() override;
 
-	void set_regexmanager(RegexManager* r);
-
 private:
-	void process_operation(Operation op,
+	void register_format_styles();
+
+	bool process_operation(Operation op,
 		bool automatic = false,
 		std::vector<std::string>* args = nullptr) override;
 	void set_head(const std::string& s,
@@ -103,18 +107,15 @@ private:
 
 	void invalidate_everything()
 	{
-		invalidated = true;
 		invalidation_mode = InvalidationMode::COMPLETE;
 	}
 
 	void invalidate(const unsigned int invalidated_pos)
 	{
-		if (invalidated == true &&
-			invalidation_mode == InvalidationMode::COMPLETE) {
+		if (invalidation_mode == InvalidationMode::COMPLETE) {
 			return;
 		}
 
-		invalidated = true;
 		invalidation_mode = InvalidationMode::PARTIAL;
 		invalidated_itempos.push_back(invalidated_pos);
 	}
@@ -141,21 +142,22 @@ private:
 	bool set_filterpos;
 	unsigned int filterpos;
 
-	RegexManager* rxman;
+	RegexManager& rxman;
 
 	unsigned int old_width;
 	int old_itempos;
-	ArticleSortStrategy old_sort_strategy;
+	nonstd::optional<ArticleSortStrategy> old_sort_strategy;
 
-	bool invalidated;
 	InvalidationMode invalidation_mode;
 	std::vector<unsigned int> invalidated_itempos;
 
 	ListFormatter listfmt;
 	Cache* rsscache;
-	FilterContainer* filters;
+	FilterContainer& filters;
 
 	void handle_op_saveall();
+
+	ListWidget items_list;
 };
 
 } // namespace newsboat
