@@ -19,10 +19,10 @@ enum { KM_FEEDLIST = 1 << 0,
 	KM_FILTERSELECT = 1 << 6,
 	KM_URLVIEW = 1 << 7,
 	KM_PODBOAT = 1 << 8,
-	KM_SYSKEYS = 1 << 9,
-	KM_INTERNAL = 1 << 10,
-	KM_DIALOGS = 1 << 11,
-	KM_DIRBROWSER = 1 << 12,
+	KM_DIALOGS = 1 << 9,
+	KM_DIRBROWSER = 1 << 10,
+	KM_SYSKEYS = 1 << 11,
+	KM_INTERNAL = 1 << 12,
 	KM_NEWSBOAT = KM_FEEDLIST | KM_FILEBROWSER | KM_HELP | KM_ARTICLELIST |
 		KM_ARTICLE | KM_TAGSELECT | KM_FILTERSELECT | KM_URLVIEW |
 		KM_DIALOGS | KM_DIRBROWSER,
@@ -43,6 +43,7 @@ enum Operation {
 	OP_MARKALLFEEDSREAD,
 	OP_MARKALLABOVEASREAD,
 	OP_OPEN,
+	OP_SWITCH_FOCUS,
 	OP_SAVE,
 	OP_SAVEALL,
 	OP_NEXTUNREAD,
@@ -133,16 +134,26 @@ enum Operation {
 	OP_INT_SET,
 
 	OP_INT_MAX,
-	OP_1 = 3001,
-	OP_2,
-	OP_3,
-	OP_4,
-	OP_5,
-	OP_6,
-	OP_7,
-	OP_8,
-	OP_9,
-	OP_0
+	OP_OPEN_URL_1 = 3001,
+	OP_OPEN_URL_2,
+	OP_OPEN_URL_3,
+	OP_OPEN_URL_4,
+	OP_OPEN_URL_5,
+	OP_OPEN_URL_6,
+	OP_OPEN_URL_7,
+	OP_OPEN_URL_8,
+	OP_OPEN_URL_9,
+	OP_OPEN_URL_10,
+
+	OP_CMD_START_1,
+	OP_CMD_START_2,
+	OP_CMD_START_3,
+	OP_CMD_START_4,
+	OP_CMD_START_5,
+	OP_CMD_START_6,
+	OP_CMD_START_7,
+	OP_CMD_START_8,
+	OP_CMD_START_9,
 };
 
 struct KeyMapDesc {
@@ -172,16 +183,15 @@ public:
 		const std::string& context);
 	std::vector<MacroCmd> get_macro(const std::string& key);
 	char get_key(const std::string& keycode);
-	std::string getkey(Operation op, const std::string& context);
+	std::vector<std::string> get_keys(Operation op, const std::string& context);
 	void handle_action(const std::string& action,
 		const std::vector<std::string>& params) override;
 	void dump_config(std::vector<std::string>& config_output) override;
-	void get_keymap_descriptions(std::vector<KeyMapDesc>& descs,
-		unsigned short flags);
-	unsigned short get_flag_from_context(const std::string& context);
+	std::vector<KeyMapDesc> get_keymap_descriptions(std::string context);
 
 private:
 	bool is_valid_context(const std::string& context);
+	unsigned short get_flag_from_context(const std::string& context);
 	std::map<std::string, Operation> get_internal_operations() const;
 	std::string getopname(Operation op);
 	std::map<std::string, std::map<std::string, Operation>> keymap_;
