@@ -3,6 +3,8 @@
 
 #include "colormanager.h"
 #include "keymap.h"
+#include "listwidget.h"
+#include "textviewwidget.h"
 #include "stflpp.h"
 
 using namespace newsboat;
@@ -18,11 +20,10 @@ class PbView {
 public:
 	explicit PbView(PbController* c = 0);
 	~PbView();
-	void run(bool auto_download);
+	void run(bool auto_download, bool wrap_scroll);
 	void set_keymap(newsboat::KeyMap* k)
 	{
 		keys = k;
-		set_bindings();
 	}
 
 private:
@@ -36,7 +37,7 @@ private:
 	void run_help();
 	void set_dllist_keymap_hint();
 	void set_help_keymap_hint();
-	void set_bindings();
+	std::pair<double, std::string> get_speed_human_readable(double kbps);
 
 	std::string prepare_keymaphint(KeyMapHintEntry* hints);
 	std::string format_line(const std::string& podlist_format,
@@ -47,6 +48,9 @@ private:
 	newsboat::Stfl::Form dllist_form;
 	newsboat::Stfl::Form help_form;
 	newsboat::KeyMap* keys;
+
+	ListWidget downloads_list;
+	TextviewWidget help_textview;
 };
 
 } // namespace podboat
