@@ -5,6 +5,11 @@
 extern "C" {
 #endif
 
+struct FilterUrl {
+	char* filter;
+	char* url;
+};
+
 char* rs_replace_all(
 	const char* str,
 	const char* from,
@@ -50,12 +55,12 @@ char* rs_unescape_url(const char* str);
 
 char* rs_make_title(const char* str);
 
+int32_t rs_run_interactively(const char* command, const char* caller);
+
 char* rs_getcwd();
 
 int rs_strnaturalcmp(const char* a, const char* b);
 
-void rs_cstring_free(char* str);
-
 char* rs_get_default_browser();
 
 bool rs_is_valid_color(const char* str);
@@ -66,17 +71,23 @@ size_t rs_strwidth(const char* str);
 
 size_t rs_strwidth_stfl(const char* str);
 
+char* rs_substr_with_width(const char* str, size_t max_width);
+
+char* rs_substr_with_width_stfl(const char* str, size_t max_width);
+
 char* rs_remove_soft_hyphens(const char* str);
 
 bool rs_is_valid_podcast_type(const char* mimetype);
 
+std::int64_t rs_podcast_mime_to_link_type(const char* mimetype, bool* success);
+
 char* rs_get_command_output(const char* str);
 
 char* rs_get_basename(const char* str);
 
 void rs_run_command(const char* command, const char* param);
 
-char* rs_run_program(char* argv[], const char* input);
+char* rs_run_program(const char* argv[], const char* input);
 
 char* rs_program_version();
 
@@ -88,47 +99,7 @@ int rs_mkdir_parents(const char* path, const std::uint32_t mode);
 
 char* rs_strip_comments(const char* line);
 
-class RustString {
-private:
-	char* str;
-
-public:
-	RustString() = delete;
-	RustString(const RustString&) = delete;
-
-	RustString(RustString&& rs)
-		: str(std::move(rs.str))
-	{
-		rs.str = nullptr;
-	}
-
-	RustString& operator=(RustString&& rs) noexcept
-	{
-		if (&rs != this) {
-			str = std::move(rs.str);
-		}
-		return *this;
-	}
-
-	explicit RustString(char* ptr)
-	{
-		str = ptr;
-	}
-
-	operator std::string()
-	{
-		if (str != nullptr) {
-			return std::string(str);
-		}
-		return std::string();
-	}
-
-	~RustString()
-	{
-		// This pointer is checked for nullptr on the rust side.
-		rs_cstring_free(str);
-	}
-};
+FilterUrl rs_extract_filter(const char* line);
 
 #ifdef __cplusplus
 }
