@@ -1,7 +1,7 @@
 #ifndef NEWSBOAT_SCOPEMEASURE_H_
 #define NEWSBOAT_SCOPEMEASURE_H_
 
-#include <sys/time.h>
+#include <chrono>
 #include <string>
 
 #include "logger.h"
@@ -15,9 +15,7 @@ public:
 	void stopover(const std::string& son = "");
 
 private:
-	// Can't use zero initialization (= {}) here because it fails with GCC 4.9:
-	// https://travis-ci.org/newsboat/newsboat/jobs/557484737#L760
-	struct timeval tv1 = {0, 0}, tv2 = {0, 0};
+	std::chrono::time_point<std::chrono::steady_clock> start_time;
 	std::string funcname;
 	Level lvl = Level::DEBUG;
 };
