@@ -8,11 +8,16 @@
 #include <string>
 #include <vector>
 
+#include "3rd-party/optional.hpp"
+
 #include "configcontainer.h"
 #include "logger.h"
 
 namespace newsboat {
 
+// Forward declaration for a type from htmlrenderer.h
+enum class LinkType;
+
 namespace utils {
 std::string strip_comments(const std::string& line);
 std::vector<std::string> tokenize(const std::string& str,
@@ -35,6 +40,10 @@ std::string convert_text(const std::string& text,
 	const std::string& tocode,
 	const std::string& fromcode);
 
+/// Converts input string from UTF-8 to the locale's encoding (as detected by
+/// nl_langinfo(CODESET)).
+std::string utf8_to_locale(const std::string& text);
+
 std::string get_command_output(const std::string& cmd);
 void extract_filter(const std::string& line,
 	std::string& filter,
@@ -46,7 +55,7 @@ std::string retrieve_url(const std::string& url,
 	CURL* easyhandle = nullptr);
 void run_command(const std::string& cmd,
 	const std::string& param); // used for notifications only
-std::string run_program(char* argv[], const std::string& input);
+std::string run_program(const char* argv[], const std::string& input);
 
 std::string resolve_tilde(const std::string&);
 std::string resolve_relative(const std::string&, const std::string&);
@@ -71,6 +80,9 @@ size_t wcswidth_stfl(const std::wstring& str, size_t size);
 std::string substr_with_width(const std::string& str,
 	const size_t max_width);
 
+std::string substr_with_width_stfl(const std::string& str,
+	const size_t max_width);
+
 unsigned int to_u(const std::string& str,
 	const unsigned int default_value = 0);
 
@@ -123,7 +135,7 @@ int mkdir_parents(const std::string& pathname,
 
 std::string make_title(const std::string& url);
 
-int run_interactively(const std::string& command,
+int32_t run_interactively(const std::string& command,
 	const std::string& caller);
 
 std::string getcwd();
@@ -134,6 +146,9 @@ void remove_soft_hyphens(std::string& text);
 
 bool is_valid_podcast_type(const std::string& mimetype);
 
+nonstd::optional<LinkType> podcast_mime_to_link_type(const std::string&
+	mimetype);
+
 std::string get_default_browser();
 
 /// The tag and Git commit ID the program was built from, or a pre-defined
