@@ -35,7 +35,6 @@ public:
 	void set_config_container(ConfigContainer* cfgcontainer);
 	void show_error(const std::string& msg);
 	void set_status(const std::string& msg);
-	void set_status_unlocked(const std::string& msg);
 	Controller* get_ctrl()
 	{
 		return ctrl;
@@ -44,7 +43,7 @@ public:
 	{
 		return cfg;
 	}
-	KeyMap* get_keys()
+	KeyMap* get_keymap()
 	{
 		return keys;
 	}
@@ -111,9 +110,7 @@ public:
 
 	void feedlist_mark_pos_if_visible(unsigned int pos);
 
-	void set_regexmanager(RegexManager* r);
 	void set_cache(Cache* c);
-	void set_filters(FilterContainer* f);
 
 	std::vector<std::pair<unsigned int, std::string>> get_formaction_names();
 
@@ -129,8 +126,6 @@ public:
 
 	void apply_colors_to_all_formactions();
 
-	void update_bindings();
-
 	void inside_qna(bool f);
 	void inside_cmdline(bool f);
 
@@ -139,7 +134,6 @@ public:
 	static void ctrl_c_action(int sig);
 
 protected:
-	void set_bindings(std::shared_ptr<FormAction> fa);
 	void apply_colors(std::shared_ptr<FormAction> fa);
 
 	void handle_cmdline_completion(std::shared_ptr<FormAction> fa);
@@ -161,7 +155,7 @@ protected:
 
 	std::vector<std::string> tags;
 
-	RegexManager* rxman;
+	RegexManager& rxman;
 
 	std::map<std::string, std::string> fg_colors;
 	std::map<std::string, std::string> bg_colors;
@@ -173,7 +167,7 @@ protected:
 	std::string last_fragment;
 	unsigned int tab_count;
 	Cache* rsscache;
-	FilterContainer* filters;
+	FilterContainer& filters;
 	std::vector<std::string> suggestions;
 };
 
