@@ -1,7 +1,3 @@
-extern crate libc;
-#[macro_use]
-extern crate libnewsboat;
-
 use libc::c_char;
 use std::ffi::CString;
 use std::panic::{catch_unwind, UnwindSafe};
@@ -13,6 +9,7 @@ pub mod fmtstrformatter;
 pub mod history;
 pub mod human_panic;
 pub mod logger;
+pub mod matchererror;
 pub mod utils;
 
 /// Runs a Rust function, and if it panics, calls abort(); otherwise returns what function
