@@ -1,11 +1,20 @@
-use abort_on_panic;
+use crate::abort_on_panic;
 use libc::{c_char, c_ulong};
-use libnewsboat::logger::{self, Level};
 use libnewsboat::utils;
+use libnewsboat::{
+    log,
+    logger::{self, Level},
+};
 use std::ffi::{CStr, CString};
 use std::path;
 use std::ptr;
 
+#[repr(C)]
+pub struct FilterUrl {
+    filter: *mut char,
+    url: *mut char,
+}
+
 #[no_mangle]
 pub unsafe extern "C" fn rs_replace_all(
     input: *const c_char,
@@ -17,9 +26,11 @@ pub unsafe extern "C" fn rs_replace_all(
         let rs_input = rs_input.to_string_lossy().into_owned();
 
         let rs_from = CStr::from_ptr(from);
+        // This won't panic because all strings in Newsboat are in UTF-8
         let rs_from = rs_from.to_str().expect("rs_from contained invalid UTF-8");
 
         let rs_to = CStr::from_ptr(to);
+        // This won't panic because all strings in Newsboat are in UTF-8
         let rs_to = rs_to.to_str().expect("rs_to contained invalid UTF-8");
 
         let result = utils::replace_all(rs_input, rs_from, rs_to);
@@ -54,10 +65,21 @@ pub unsafe extern "C" fn rs_consolidate_whitespace(input: *const c_char) -> *mut
 pub unsafe extern "C" fn rs_resolve_tilde(path: *const c_char) -> *mut c_char {
     abort_on_panic(|| {
         let rs_path = CStr::from_ptr(path);
+        // We simply assume that all the paths are in UTF-8 -- hence to_string_lossy().
         let rs_path = rs_path.to_string_lossy().into_owned();
 
-        let result = utils::resolve_tilde(rs_path);
+        let result = utils::resolve_tilde(path::PathBuf::from(rs_path));
+        // We simply assume that all the paths are in UTF-8 -- hence to_string_lossy().
+        let result = result.to_string_lossy().into_owned();
 
+        // `result` consists of:
+        // 1. a path to the home dir, which can't contain NUL bytes, since it has to be handled by
+        //    C APIs;
+        // 2. a path delimiter, which is a slash and not a NUL byte;
+        // 3. the original input, `path`, which came here as a C string, so doesn't contain NUL
+        //    bytes.
+        //
+        // Thus, `unwrap` won't panic.
         let result = CString::new(result).unwrap();
         result.into_raw()
     })
@@ -112,6 +134,9 @@ pub unsafe extern "C" fn rs_absolute_url(
         let rs_link = rs_link.to_string_lossy();
 
         let ret = utils::absolute_url(&rs_base_url, &rs_link);
+        // `ret` is either the same as `rs_link`, or a combination of `rs_base_url`, `rs_link`, and
+        // a slash. `rs_base_url` and `rs_link` came here as C strings, so they are guaranteed not
+        // to contain NUL. The slash is obviously not a NUL. Thus, `unwrap` won't panic.
         CString::new(ret).unwrap().into_raw()
     })
 }
@@ -166,6 +191,9 @@ pub unsafe extern "C" fn rs_censor_url(url: *const c_char) -> *mut c_char {
     abort_on_panic(|| {
         let rs_url = CStr::from_ptr(url);
         let rs_url = rs_url.to_string_lossy();
+        // `censor_url` replaces part of `rs_url` with `*`. Since `rs_url` came here as a C string,
+        // it doesn't contain NUL bytes. Thus, the result of `censor_url` won't contain NUL either,
+        // and `unwrap` won't panic.
         CString::new(utils::censor_url(&rs_url)).unwrap().into_raw()
     })
 }
@@ -193,6 +221,7 @@ pub unsafe extern "C" fn rs_trim_end(input: *const c_char) -> *mut c_char {
 
         let result = utils::trim_end(rs_input);
 
+        // `result` is a subset of `rs_input`, which is a C string; thus, `unwrap` won't panic.
         let result = CString::new(result).unwrap();
         result.into_raw()
     })
@@ -206,6 +235,7 @@ pub unsafe extern "C" fn rs_trim(input: *const c_char) -> *mut c_char {
 
         let result = utils::trim(rs_input);
 
+        // `result` is a subset of `rs_input`, which is a C string; thus, `unwrap` won't panic.
         let result = CString::new(result).unwrap();
         result.into_raw()
     })
@@ -265,16 +295,16 @@ pub unsafe extern "C" fn rs_unescape_url(input: *const c_char) -> *mut c_char {
         let rs_input = rs_input.to_string_lossy().into_owned();
 
         let result = utils::unescape_url(rs_input);
-        // Panic here can't happen because:
-        // 1. It would have already occured within unescape_url
-        // 2. panic can only happen if `result` contains null bytes;
-        // 3. `result` contains what `input` contained, and input is a
-        // null-terminated string from C.
-        if result.is_some() {
-            let result = CString::new(result.unwrap()).unwrap();
+        if let Some(result) = result {
+            // Panic here can't happen because:
+            // 1. It would have already occured within unescape_url
+            // 2. panic can only happen if `result` contains null bytes;
+            // 3. `result` contains what `input` contained, and input is a
+            // null-terminated string from C.
+            let result = CString::new(result).unwrap();
             return result.into_raw();
         }
-        return ptr::null_mut();
+        ptr::null_mut()
     })
 }
 
@@ -296,6 +326,24 @@ pub unsafe extern "C" fn rs_make_title(input: *const c_char) -> *mut c_char {
     })
 }
 
+#[no_mangle]
+pub unsafe extern "C" fn rs_run_interactively(
+    command: *const c_char,
+    caller: *const c_char,
+) -> i32 {
+    abort_on_panic(|| {
+        let command = CStr::from_ptr(command);
+        // This won't panic because all strings in Newsboat are in UTF-8
+        let command = command.to_str().expect("command contained invalid UTF-8");
+
+        let caller = CStr::from_ptr(caller);
+        // This won't panic because all strings in Newsboat are in UTF-8
+        let caller = caller.to_str().expect("caller contained invalid UTF-8");
+
+        utils::run_interactively(&command, &caller)
+    })
+}
+
 #[no_mangle]
 /// Gets the current working directory or an empty string on error.
 pub extern "C" fn rs_getcwd() -> *mut c_char {
@@ -356,6 +404,8 @@ pub unsafe extern "C" fn rs_get_basename(input: *const c_char) -> *mut c_char {
         let rs_input = CStr::from_ptr(input);
         let rs_input = rs_input.to_string_lossy();
         let output = utils::get_basename(&rs_input);
+        // `output` is a subset of `rs_input`, which is a C string. `output` can also be an empty
+        // string, which obviously doesn't contain NUL. Thus, `unwrap` here won't panic.
         let result = CString::new(output).unwrap();
         result.into_raw()
     })
@@ -409,6 +459,38 @@ pub unsafe extern "C" fn rs_strwidth_stfl(input: *const c_char) -> usize {
     })
 }
 
+#[no_mangle]
+pub unsafe extern "C" fn rs_substr_with_width(
+    string: *const c_char,
+    max_width: usize,
+) -> *mut c_char {
+    abort_on_panic(|| {
+        let rs_str = CStr::from_ptr(string);
+        let rs_str = rs_str.to_string_lossy().into_owned();
+        let output = utils::substr_with_width(&rs_str, max_width);
+        // `output` contains a subset of `string`, which is a C string. Thus, we conclude that
+        // `output` doesn't contain null bytes. Therefore, `CString::new` always returns `Some`.
+        let result = CString::new(output).unwrap();
+        result.into_raw()
+    })
+}
+
+#[no_mangle]
+pub unsafe extern "C" fn rs_substr_with_width_stfl(
+    string: *const c_char,
+    max_width: usize,
+) -> *mut c_char {
+    abort_on_panic(|| {
+        let rs_str = CStr::from_ptr(string);
+        let rs_str = rs_str.to_string_lossy().into_owned();
+        let output = utils::substr_with_width_stfl(&rs_str, max_width);
+        // `output` contains a subset of `string`, which is a C string. Thus, we conclude that
+        // `output` doesn't contain null bytes. Therefore, `CString::new` always returns `Some`.
+        let result = CString::new(output).unwrap();
+        result.into_raw()
+    })
+}
+
 #[no_mangle]
 pub unsafe extern "C" fn rs_remove_soft_hyphens(text: *const c_char) -> *mut c_char {
     abort_on_panic(|| {
@@ -432,6 +514,28 @@ pub unsafe extern "C" fn rs_is_valid_podcast_type(mimetype: *const c_char) -> bo
     })
 }
 
+#[no_mangle]
+pub unsafe extern "C" fn rs_podcast_mime_to_link_type(
+    mimetype: *const c_char,
+    success: *mut bool,
+) -> i64 {
+    abort_on_panic(|| {
+        let rs_mimetype = CStr::from_ptr(mimetype);
+        let rs_mimetype = rs_mimetype.to_string_lossy();
+        match utils::podcast_mime_to_link_type(&rs_mimetype) {
+            Some(link_type) => {
+                *success = true;
+                link_type as i64
+            }
+
+            None => {
+                *success = false;
+                0 // arbitrary value -- it won't be used on the other side
+            }
+        }
+    })
+}
+
 #[no_mangle]
 pub unsafe extern "C" fn rs_get_command_output(input: *const c_char) -> *mut c_char {
     abort_on_panic(|| {
@@ -449,9 +553,11 @@ pub unsafe extern "C" fn rs_get_command_output(input: *const c_char) -> *mut c_c
 pub unsafe extern "C" fn rs_run_command(command: *const c_char, param: *const c_char) {
     abort_on_panic(|| {
         let command = CStr::from_ptr(command);
+        // This won't panic because all strings in Newsboat are in UTF-8
         let command = command.to_str().expect("command contained invalid UTF-8");
 
         let param = CStr::from_ptr(param);
+        // This won't panic because all strings in Newsboat are in UTF-8
         let param = param.to_str().expect("param contained invalid UTF-8");
 
         utils::run_command(command, param);
@@ -460,7 +566,7 @@ pub unsafe extern "C" fn rs_run_command(command: *const c_char, param: *const c_
 
 #[no_mangle]
 pub unsafe extern "C" fn rs_run_program(
-    argv: *mut *mut c_char,
+    argv: *const *mut c_char,
     input: *const c_char,
 ) -> *mut c_char {
     abort_on_panic(|| {
@@ -485,6 +591,7 @@ pub unsafe extern "C" fn rs_run_program(
         };
 
         let input = CStr::from_ptr(input);
+        // This won't panic because all strings in Newsboat are in UTF-8
         let input = input.to_str().expect("input contained invalid UTF-8");
 
         let output = utils::run_program(&argv, input);
@@ -511,13 +618,14 @@ pub extern "C" fn rs_program_version() -> *mut c_char {
 
 #[no_mangle]
 pub extern "C" fn rs_newsboat_version_major() -> u32 {
-    abort_on_panic(|| utils::newsboat_major_version())
+    abort_on_panic(utils::newsboat_major_version)
 }
 
 #[no_mangle]
 pub unsafe extern "C" fn rs_strip_comments(line: *const c_char) -> *mut c_char {
     abort_on_panic(|| {
         let line = CStr::from_ptr(line);
+        // This won't panic because all strings in Newsboat are in UTF-8
         let line = line.to_str().expect("line contained invalid UTF-8");
 
         let result = utils::strip_comments(line);
@@ -528,3 +636,32 @@ pub unsafe extern "C" fn rs_strip_comments(line: *const c_char) -> *mut c_char {
         result.into_raw()
     })
 }
+
+#[no_mangle]
+pub unsafe extern "C" fn rs_extract_filter(line: *const c_char) -> FilterUrl {
+    abort_on_panic(|| {
+        let line = CStr::from_ptr(line);
+        // This won't panic because all strings in Newsboat are in UTF-8
+        let line = line.to_str().expect("line contained invalid UTF-8");
+
+        let (filter, url) = utils::extract_filter(line);
+        // `rfilter` contains a subset of `line`, which is a C string. Thus, we conclude that
+        // `rfilter` doesn't contain null bytes. Therefore, `CString::new` always returns `Some`.
+        let filter = CString::new(filter).unwrap();
+        // `rurl` contains a subset of `line`, which is a C string. Thus, we conclude that
+        // `rurl` doesn't contain null bytes. Therefore, `CString::new` always returns `Some`.
+        let url = CString::new(url).unwrap();
+
+        FilterUrl {
+            filter: filter.into_raw() as *mut char,
+            url: url.into_raw() as *mut char,
+        }
+    })
+}
+
+#[no_mangle]
+pub unsafe extern "C" fn rs_get_string(line: *const c_char) -> *mut c_char {
+    let line = CStr::from_ptr(line);
+    let result = CString::from(line);
+    result.into_raw()
+}
