@@ -3,11 +3,10 @@
 mod limited_string;
 mod parser;
 
-use self::limited_string::LimitedString;
-use self::parser::{parse, Padding, Specifier};
-use std::cmp::min;
+use crate::utils;
+use limited_string::LimitedString;
+use parser::{parse, Padding, Specifier};
 use std::collections::BTreeMap;
-use utils;
 
 /// Produces strings of values in a specified format, strftime(3)-like.
 ///
@@ -53,6 +52,7 @@ use utils;
 /// Doe` etc. would be "values". The term "format specifiers" will be reserved to things like `%a`,
 /// and "format strings" would mean a collection of format specifiers, with optional text in
 /// between.
+#[derive(Default)]
 pub struct FmtStrFormatter {
     /// Stores keys and their values.
     fmts: BTreeMap<char, String>,
@@ -83,7 +83,7 @@ impl FmtStrFormatter {
             result.push(c);
         } else {
             let padding_width = {
-                let content_width = utils::graphemes_count(&rest) + result.length();
+                let content_width = utils::strwidth(&rest) + result.length();
                 if content_width > width as usize {
                     0
                 } else {
@@ -91,7 +91,8 @@ impl FmtStrFormatter {
                 }
             };
 
-            let padding = format!("{}", c).repeat(padding_width);
+            let padding_value = &format!("{}", c);
+            let padding = padding_value.repeat(padding_width);
             result.push_str(&padding);
         };
         result.push_str(&rest);
@@ -100,22 +101,22 @@ impl FmtStrFormatter {
     fn format_format(&self, c: char, padding: &Padding, _width: u32, result: &mut LimitedString) {
         let empty_string = String::new();
         let value = self.fmts.get(&c).unwrap_or_else(|| &empty_string);
-        match padding {
-            &Padding::None => result.push_str(value),
+        match *padding {
+            Padding::None => result.push_str(value),
 
-            &Padding::Left(total_width) => {
-                let padding_width = total_width - min(total_width, utils::graphemes_count(value));
-                let stripping_width = total_width - padding_width;
+            Padding::Left(total_width) => {
+                let text = &utils::substr_with_width(value, total_width);
+                let padding_width = total_width - utils::strwidth(text);
                 let padding = String::from(" ").repeat(padding_width);
                 result.push_str(&padding);
-                result.push_str(&utils::take_graphemes(value, stripping_width));
+                result.push_str(text);
             }
 
-            &Padding::Right(total_width) => {
-                let padding_width = total_width - min(total_width, utils::graphemes_count(value));
-                let stripping_width = total_width - padding_width;
+            Padding::Right(total_width) => {
+                let text = &utils::substr_with_width(value, total_width);
+                let padding_width = total_width - utils::strwidth(text);
                 let padding = String::from(" ").repeat(padding_width);
-                result.push_str(&utils::take_graphemes(value, stripping_width));
+                result.push_str(text);
                 result.push_str(&padding);
             }
         }
@@ -130,11 +131,11 @@ impl FmtStrFormatter {
         result: &mut LimitedString,
     ) {
         match self.fmts.get(&cond) {
-            Some(value) if !value.is_empty() => {
+            Some(value) if !value.trim().is_empty() => {
                 result.push_str(&self.formatting_helper(then, width))
             }
             _ => {
-                if let &Some(ref els) = els {
+                if let Some(ref els) = *els {
                     result.push_str(&self.formatting_helper(&els, width))
                 }
             }
@@ -149,33 +150,33 @@ impl FmtStrFormatter {
         });
 
         for (i, specifier) in format_ast.iter().enumerate() {
-            match specifier {
-                &Specifier::Spacing(c) => {
+            match *specifier {
+                Specifier::Spacing(c) => {
                     let rest = &format_ast[i + 1..];
                     self.format_spacing(c, rest, width, &mut result);
                     // format_spacing will also format the rest of the string, so quit the loop
                     break;
                 }
 
-                &Specifier::Format(c, ref padding) => {
+                Specifier::Format(c, ref padding) => {
                     self.format_format(c, &padding, width, &mut result);
                 }
 
-                &Specifier::Text(s) => {
+                Specifier::Text(s) => {
                     if width == 0 {
                         result.push_str(s);
                     } else {
                         let remaining = width as usize - result.length();
-                        let count = utils::graphemes_count(&s);
+                        let count = utils::strwidth(&s);
                         if remaining >= count {
                             result.push_str(&s);
                         } else {
-                            result.push_str(&utils::take_graphemes(s, remaining));
+                            result.push_str(&utils::substr_with_width(s, remaining));
                         }
                     }
                 }
 
-                &Specifier::Conditional(cond, ref then, ref els) => {
+                Specifier::Conditional(cond, ref then, ref els) => {
                     self.format_conditional(cond, &then, &els, width, &mut result)
                 }
             }
@@ -352,6 +353,62 @@ mod tests {
         assert_eq!(fmt.do_format("%?c?asdf?", 0), "asdf");
     }
 
+    #[test]
+    fn t_do_format_keeps_wide_characters_within_specified_width() {
+        let mut fmt = FmtStrFormatter::new();
+
+        fmt.register_fmt('a', "ＡＢＣ".to_string());
+        fmt.register_fmt('b', "def".to_string());
+
+        assert_eq!(fmt.do_format("%a %b", 0), "ＡＢＣ def");
+        assert_eq!(fmt.do_format("%a %b", 10), "ＡＢＣ def");
+        assert_eq!(fmt.do_format("%a %b", 9), "ＡＢＣ de");
+        assert_eq!(fmt.do_format("%a %b", 7), "ＡＢＣ ");
+        assert_eq!(fmt.do_format("%a %b", 6), "ＡＢＣ");
+        assert_eq!(fmt.do_format("%a %b", 4), "ＡＢ");
+    }
+
+    #[test]
+    fn t_do_format_does_not_include_wide_character_if_only_1_column_left() {
+        let mut fmt = FmtStrFormatter::new();
+
+        fmt.register_fmt('a', "ＡＢＣ".to_string());
+
+        assert_eq!(fmt.do_format("%a", 6), "ＡＢＣ");
+        assert_eq!(fmt.do_format("%a", 5), "ＡＢ");
+        assert_eq!(fmt.do_format("%a", 4), "ＡＢ");
+
+        assert_eq!(fmt.do_format("%-6a", 0), "ＡＢＣ");
+        assert_eq!(fmt.do_format("%-5a", 0), "ＡＢ ");
+        assert_eq!(fmt.do_format("%-4a", 0), "ＡＢ");
+
+        assert_eq!(fmt.do_format("%6a", 0), "ＡＢＣ");
+        assert_eq!(fmt.do_format("%5a", 0), " ＡＢ");
+        assert_eq!(fmt.do_format("%4a", 0), "ＡＢ");
+    }
+
+    #[test]
+    fn t_do_format_does_not_escape_less_than_signs_in_regular_text() {
+        let mut fmt = FmtStrFormatter::new();
+
+        fmt.register_fmt('a', "AAA".to_string());
+        fmt.register_fmt('b', "BBB".to_string());
+
+        assert_eq!(fmt.do_format("%a <%b>", 0), "AAA <BBB>");
+    }
+
+    #[test]
+    fn t_do_format_does_not_escape_less_than_signs_in_filling_format() {
+        let mut fmt = FmtStrFormatter::new();
+
+        fmt.register_fmt('a', "AAA".to_string());
+        fmt.register_fmt('b', "BBB".to_string());
+
+        assert_eq!(fmt.do_format("%a%>.%b", 10), "AAA....BBB");
+        assert_eq!(fmt.do_format("%a%><%b", 10), "AAA<<<<BBB");
+        assert_eq!(fmt.do_format("%a%>>%b", 10), "AAA>>>>BBB");
+    }
+
     #[test]
     fn t_do_format_ignores_start_of_conditional_at_the_end_of_format_string() {
         let fmt = FmtStrFormatter::new();
@@ -489,6 +546,17 @@ mod tests {
         assert_eq!(fmt.do_format("%?m?непустое?", 0), "");
     }
 
+    #[test]
+    fn t_conditional_whitespace_is_handled_as_empty() {
+        let mut fmt = FmtStrFormatter::new();
+
+        fmt.register_fmt('a', " \t ".to_string());
+        fmt.register_fmt('b', "  some whitespace  ".to_string());
+
+        assert_eq!(fmt.do_format("%?a?non-empty&empty?", 0), "empty");
+        assert_eq!(fmt.do_format("%?b?non-empty&empty?", 0), "non-empty");
+    }
+
     #[test]
     fn t_do_format_replaces_double_percent_sign_with_a_percent_sign() {
         let fmt = FmtStrFormatter::new();
@@ -518,7 +586,7 @@ mod tests {
         assert_eq!(fmt.do_format("%x? %y", 0), "What's the ultimate answer? 42");
     }
 
-    proptest! {
+    proptest::proptest! {
         #[test]
         fn does_not_crash_when_formatting_with_no_formats_registered(ref input in "\\PC*") {
             let fmt = FmtStrFormatter::new();
@@ -543,7 +611,7 @@ mod tests {
         fn result_is_never_longer_than_specified_width(length in 1u32..10000, ref input in "\\PC*") {
             let fmt = FmtStrFormatter::new();
             let result = fmt.do_format(&input, length);
-            assert!(utils::graphemes_count(&result) <= length as usize);
+            assert!(utils::strwidth(&result) <= length as usize);
         }
     }
 }
