@@ -1,8 +1,6 @@
 //! Keeps a record of what the program did.
 
-extern crate chrono;
-
-use self::chrono::{offset::Local, Datelike, Timelike};
+use chrono::{offset::Local, Datelike, Timelike};
 use once_cell::sync::OnceCell;
 use std::fmt;
 use std::fs::{File, OpenOptions};
@@ -244,7 +242,7 @@ impl Logger {
                 let _ = logfile.write_all(timestamp.as_bytes());
                 let _ = logfile.write_all(level.as_bytes());
                 let _ = logfile.write_all(data);
-                let _ = logfile.write_all("\n".as_bytes());
+                let _ = logfile.write_all(b"\n");
             }
         }
 
@@ -253,7 +251,7 @@ impl Logger {
                 // Ignoring the error since checking every log() call will be too bothersome.
                 let _ = user_error_logfile.write_all(timestamp.as_bytes());
                 let _ = user_error_logfile.write_all(data);
-                let _ = user_error_logfile.write_all("\n".as_bytes());
+                let _ = user_error_logfile.write_all(b"\n");
             }
         }
     }
@@ -279,7 +277,13 @@ impl Logger {
     }
 }
 
-static GLOBAL_LOGGER: OnceCell<Logger> = OnceCell::INIT;
+impl Default for Logger {
+    fn default() -> Self {
+        Self::new()
+    }
+}
+
+static GLOBAL_LOGGER: OnceCell<Logger> = OnceCell::new();
 
 /// Returns a global logger instance.
 ///
@@ -293,8 +297,7 @@ pub fn get_instance() -> &'static Logger {
 ///
 /// Most of the time, you should just use this. For example:
 /// ```no_run
-/// # #[macro_use] extern crate libnewsboat;
-/// use libnewsboat::logger::{self, Level};
+/// use libnewsboat::{log, logger::{self, Level}};
 ///
 /// fn super_cool_function(value: u32) {
 ///     log!(Level::Debug, "super_cool_function(): value = {}", value);
@@ -312,14 +315,12 @@ macro_rules! log {
 
 #[cfg(test)]
 mod tests {
-    extern crate tempfile;
-
     use super::*;
 
-    use self::chrono::{Duration, TimeZone};
-    use self::tempfile::TempDir;
+    use chrono::{Duration, TimeZone};
     use std::io::{self, BufRead, BufReader};
     use std::path;
+    use tempfile::TempDir;
 
     fn setup_logger() -> io::Result<(TempDir, path::PathBuf, path::PathBuf, Logger)> {
         let tmp = TempDir::new()?;
