@@ -1,26 +1,14 @@
-extern crate curl_sys;
-extern crate dirs;
-extern crate libc;
-extern crate natord;
-extern crate rand;
-extern crate regex;
-extern crate std;
-extern crate unicode_segmentation;
-extern crate unicode_width;
-extern crate url;
-
-use self::regex::Regex;
-use self::unicode_segmentation::UnicodeSegmentation;
-use self::unicode_width::UnicodeWidthStr;
-use self::url::percent_encoding::*;
-use self::url::Url;
+use crate::htmlrenderer;
+use crate::logger::{self, Level};
 use libc::c_ulong;
-use logger::{self, Level};
+use percent_encoding::*;
 use std::fs::DirBuilder;
 use std::io::{self, Write};
 use std::os::unix::fs::DirBuilderExt;
 use std::path::{Path, PathBuf};
 use std::process::{Command, Stdio};
+use unicode_width::{UnicodeWidthChar, UnicodeWidthStr};
+use url::Url;
 
 pub fn replace_all(input: String, from: &str, to: &str) -> String {
     input.replace(from, to)
@@ -86,28 +74,15 @@ pub fn absolute_url(base_url: &str, link: &str) -> String {
         .to_owned()
 }
 
-pub fn resolve_tilde(path: String) -> String {
-    let mut file_path: String = path;
-    let home_path = dirs::home_dir();
-
-    if let Some(home_path) = home_path {
-        let home_path_string = home_path.to_string_lossy().into_owned();
-
-        if file_path == "~" {
-            file_path = home_path_string;
-        } else {
-            let tmp_file_path = file_path.clone();
-
-            if tmp_file_path.len() > 1 {
-                let (tilde, remaining) = tmp_file_path.split_at(2);
-
-                if tilde == "~/" {
-                    file_path = home_path_string + "/" + remaining;
-                }
-            }
-        }
+/// Replaces tilde (`~`) at the beginning of the path with the path to user's home directory.
+pub fn resolve_tilde(path: PathBuf) -> PathBuf {
+    if let (Some(home), Ok(suffix)) = (dirs::home_dir(), path.strip_prefix("~")) {
+        return home.join(suffix);
     }
-    file_path
+
+    // Either the `path` doesn't start with tilde, or we couldn't figure out the path to the
+    // home directory -- either way, it's no big deal. Let's return the original string.
+    path
 }
 
 pub fn resolve_relative(reference: &Path, path: &Path) -> PathBuf {
@@ -211,7 +186,7 @@ pub fn censor_url(url: &str) -> String {
 /// assert_eq!(&quote_for_stfl("test"), "test");
 /// ```
 pub fn quote_for_stfl(string: &str) -> String {
-    return string.replace("<", "<>");
+    string.replace("<", "<>")
 }
 
 /// Get basename from a URL if available else return an empty string
@@ -244,7 +219,7 @@ pub fn trim(rs_str: String) -> String {
 
 pub fn trim_end(rs_str: String) -> String {
     let x: &[_] = &['\n', '\r'];
-    rs_str.trim_right_matches(x).to_string()
+    rs_str.trim_end_matches(x).to_string()
 }
 
 pub fn quote(input: String) -> String {
@@ -301,19 +276,123 @@ pub fn strwidth(rs_str: &str) -> usize {
     UnicodeWidthStr::width(rs_str)
 }
 
+/// Returns the width of `rs_str` when displayed on screen.
+///
+/// STFL tags (e.g. `<b>`, `<foobar>`, `</>`) are counted as having 0 width.
+/// Escaped less-than sign (`<` escaped as `<>`) is counted as having a width of 1 character.
+/// ```
+/// use libnewsboat::utils::strwidth_stfl;
+/// assert_eq!(strwidth_stfl("a"), 1);
+/// assert_eq!(strwidth_stfl("abc<tag>def"), 6);
+/// assert_eq!(strwidth_stfl("less-than: <>"), 12);
+/// assert_eq!(strwidth_stfl("ＡＢＣＤＥＦ"), 12);
+///```
 pub fn strwidth_stfl(rs_str: &str) -> usize {
-    let reduce = 3 * rs_str
-        .chars()
-        .zip(rs_str.chars().skip(1))
-        .filter(|&(c, next_c)| c == '<' && next_c != '>')
-        .count();
-
-    let width = strwidth(rs_str);
-    if width < reduce {
-        0
-    } else {
-        width - reduce
+    let mut s = &rs_str[..];
+    let mut width = 0;
+    loop {
+        if let Some(pos) = s.find('<') {
+            width += strwidth(&s[..pos]);
+            s = &s[pos..];
+            if let Some(endpos) = s.find('>') {
+                if endpos == 1 {
+                    // Found "<>" which stfl uses to encode a literal '<'
+                    width += strwidth("<");
+                }
+                s = &s[endpos + 1..];
+            } else {
+                // '<' without closing '>' so ignore rest of string
+                break;
+            }
+        } else {
+            width += strwidth(s);
+            break;
+        }
     }
+
+    width
+}
+
+/// Returns a longest substring fits to the given width.
+/// Returns an empty string if `str` is an empty string or `max_width` is zero.
+///
+/// Each chararacter width is calculated with UnicodeWidthChar::width. If UnicodeWidthChar::width()
+/// returns None, the character width is treated as 0.
+/// ```
+/// use libnewsboat::utils::substr_with_width;
+/// assert_eq!(substr_with_width("a", 1), "a");
+/// assert_eq!(substr_with_width("a", 2), "a");
+/// assert_eq!(substr_with_width("ab", 1), "a");
+/// assert_eq!(substr_with_width("abc", 1), "a");
+/// assert_eq!(substr_with_width("A\u{3042}B\u{3044}C\u{3046}", 5), "A\u{3042}B")
+///```
+pub fn substr_with_width(string: &str, max_width: usize) -> String {
+    let mut result = String::new();
+    let mut width = 0;
+    for c in string.chars() {
+        // Control chars count as width 0
+        let w = UnicodeWidthChar::width(c).unwrap_or(0);
+        if width + w > max_width {
+            break;
+        }
+        width += w;
+        result.push(c);
+    }
+    result
+}
+
+/// Returns a longest substring fits to the given width.
+/// Returns an empty string if `str` is an empty string or `max_width` is zero.
+///
+/// Each chararacter width is calculated with UnicodeWidthChar::width. If UnicodeWidthChar::width()
+/// returns None, the character width is treated as 0. A STFL tag (e.g. `<b>`, `<foobar>`, `</>`)
+/// width is treated as 0, but escaped less-than (`<>`) width is treated as 1.
+/// ```
+/// use libnewsboat::utils::substr_with_width_stfl;
+/// assert_eq!(substr_with_width_stfl("a", 1), "a");
+/// assert_eq!(substr_with_width_stfl("a", 2), "a");
+/// assert_eq!(substr_with_width_stfl("ab", 1), "a");
+/// assert_eq!(substr_with_width_stfl("abc", 1), "a");
+/// assert_eq!(substr_with_width_stfl("A\u{3042}B\u{3044}C\u{3046}", 5), "A\u{3042}B")
+///```
+pub fn substr_with_width_stfl(string: &str, max_width: usize) -> String {
+    let mut result = String::new();
+    let mut in_bracket = false;
+    let mut tagbuf = Vec::<char>::new();
+    let mut width = 0;
+    for c in string.chars() {
+        if in_bracket {
+            tagbuf.push(c);
+            if c == '>' {
+                in_bracket = false;
+                if tagbuf == ['<', '>'] {
+                    if width + 1 > max_width {
+                        break;
+                    }
+                    result += "<>"; // escaped less-than
+                    tagbuf.clear();
+                    width += 1;
+                } else {
+                    result += &tagbuf.iter().collect::<String>();
+                    tagbuf.clear();
+                }
+            }
+        } else {
+            if c == '<' {
+                in_bracket = true;
+                tagbuf.push(c);
+            } else {
+                // Control chars count as width 0
+                let w = UnicodeWidthChar::width(c).unwrap_or(0);
+                if width + w > max_width {
+                    break;
+                }
+                width += w;
+                result.push(c);
+            }
+        }
+    }
+    result
 }
 
 /// Remove all soft-hyphens as they can behave unpredictably (see
@@ -323,14 +402,51 @@ pub fn remove_soft_hyphens(text: &mut String) {
     text.retain(|c| c != '\u{00AD}')
 }
 
+/// An array of "MIME matchers" and their associated LinkTypes
+///
+/// This is used for two tasks:
+///
+/// 1. checking if a MIME type is a podcast type (`utils::is_valid_podcast_type`). That involves
+///    running all matching functions on given input and checking if any of them returned `true`;
+///
+/// 2. figuring out the `LinkType` for a particular enclosure, given its MIME type
+///    (`utils::podcast_mime_to_link_type`).
+const PODCAST_MIME_TO_LINKTYPE: [(fn(&str) -> bool, htmlrenderer::LinkType); 2] = [
+    (
+        |mime| {
+            // RFC 5334, section 10.1 says "historically, some implementations expect .ogg files to be
+            // solely Vorbis-encoded audio", so let's assume it's audio, not video.
+            // https://tools.ietf.org/html/rfc5334#section-10.1
+            mime.starts_with("audio/") || mime == "application/ogg"
+        },
+        htmlrenderer::LinkType::Audio,
+    ),
+    (
+        |mime| mime.starts_with("video/"),
+        htmlrenderer::LinkType::Video,
+    ),
+];
+
+/// Returns `true` if given MIME type is considered to be a podcast by Newsboat.
 pub fn is_valid_podcast_type(mimetype: &str) -> bool {
-    let re = Regex::new(r"(audio|video)/.*").unwrap();
-    let matches = re.is_match(mimetype);
+    PODCAST_MIME_TO_LINKTYPE
+        .iter()
+        .any(|(matcher, _)| matcher(mimetype))
+}
 
-    let acceptable = ["application/ogg"];
-    let found = acceptable.contains(&mimetype);
-
-    matches || found
+/// Converts podcast's MIME type into an HtmlRenderer's "link type"
+///
+/// Returns None if given MIME type is not a podcast type. See `is_valid_podcast_type()`.
+pub fn podcast_mime_to_link_type(mime_type: &str) -> Option<htmlrenderer::LinkType> {
+    PODCAST_MIME_TO_LINKTYPE
+        .iter()
+        .find_map(|(matcher, link_type)| {
+            if matcher(mime_type) {
+                Some(*link_type)
+            } else {
+                None
+            }
+        })
 }
 
 pub fn get_auth_method(method: &str) -> c_ulong {
@@ -448,7 +564,7 @@ pub fn make_title(rs_str: String) -> String {
      * http://domain.com/story/yy/mm/dd/title-with-dashes?a=b
      */
     // Strip out trailing slashes
-    let mut result = rs_str.trim_right_matches('/');
+    let mut result = rs_str.trim_end_matches('/');
 
     // get to the final part of the URI's path and
     // extract just the juicy part 'title-with-dashes?a=b'
@@ -462,10 +578,10 @@ pub fn make_title(rs_str: String) -> String {
 
     // Throw away common webpage suffixes: .html, .php, .aspx, .htm
     result = result
-        .trim_right_matches(".html")
-        .trim_right_matches(".php")
-        .trim_right_matches(".aspx")
-        .trim_right_matches(".htm");
+        .trim_end_matches(".html")
+        .trim_end_matches(".php")
+        .trim_end_matches(".aspx")
+        .trim_end_matches(".htm");
 
     // 'title with dashes'
     let result = result.replace('-', " ").replace('_', " ");
@@ -486,6 +602,34 @@ pub fn make_title(rs_str: String) -> String {
     }
 }
 
+/// Run the given command interactively with inherited stdin and stdout/stderr
+/// ```
+/// use libnewsboat::utils::run_interactively;
+///
+/// let result = run_interactively("echo true", "test");
+/// assert_eq!(result, 0);
+///
+/// let result = run_interactively("exit 1", "test");
+/// assert_eq!(result, 1);
+/// ```
+pub fn run_interactively(command: &str, caller: &str) -> i32 {
+    log!(Level::Debug, &format!("{}: running `{}'", caller, command));
+    Command::new("sh")
+        .arg("-c")
+        .arg(command)
+        .status()
+        .map_err(|err| {
+            log!(
+                Level::Warn,
+                &format!("{}: Couldn't create child process: {}", caller, err)
+            )
+        })
+        .ok()
+        .and_then(|exit_status| exit_status.code())
+        // return -1 if command got killed by signal
+        .unwrap_or(-1)
+}
+
 /// Get the current working directory.
 pub fn getcwd() -> Result<PathBuf, io::Error> {
     use std::env;
@@ -543,44 +687,6 @@ pub fn mkdir_parents<R: AsRef<Path>>(p: &R, mode: u32) -> io::Result<()> {
         .create(p.as_ref())
 }
 
-/// Counts graphemes in a given string.
-///
-/// ```
-/// use libnewsboat::utils::graphemes_count;
-///
-/// assert_eq!(graphemes_count("D"), 1);
-/// // len() counts bytes, not characters, but all ASCII symbols are represented by one byte in
-/// // UTF-8, so len() returns 1 in this case
-/// assert_eq!("D".len(), 1);
-///
-/// // Here's a situation where a single grapheme is represented by multiple bytes
-/// assert_eq!(graphemes_count("Ж"), 1);
-/// assert_eq!("Ж".len(), 2);
-///
-/// assert_eq!(graphemes_count("📰"), 1);
-/// assert_eq!("📰".len(), 4);
-/// ```
-pub fn graphemes_count(input: &str) -> usize {
-    UnicodeSegmentation::graphemes(input, true).count()
-}
-
-/// Extracts up to `n` first graphemes from the given string.
-///
-/// ```
-/// use libnewsboat::utils::take_graphemes;
-///
-/// let input = "Привет!";
-/// assert_eq!(take_graphemes(input, 1), "П");
-/// assert_eq!(take_graphemes(input, 4), "Прив");
-/// assert_eq!(take_graphemes(input, 6), "Привет");
-/// assert_eq!(take_graphemes(input, 20), input);
-/// ```
-pub fn take_graphemes(input: &str, n: usize) -> String {
-    UnicodeSegmentation::graphemes(input, true)
-        .take(n)
-        .collect::<String>()
-}
-
 /// The tag and Git commit ID the program was built from, or a pre-defined value from config.h if
 /// there is no Git directory.
 pub fn program_version() -> String {
@@ -633,9 +739,26 @@ pub fn strip_comments(line: &str) -> &str {
     &line[0..first_pound_chr_idx]
 }
 
+/// Extract filter and url from line separated by ':'.
+pub fn extract_filter(line: &str) -> (&str, &str) {
+    debug_assert!(line.starts_with("filter:"));
+    // line must start with "filter:"
+    let line = line.get("filter:".len()..).unwrap();
+    let (filter, url) = line.split_at(line.find(':').unwrap_or(0));
+    let url = url.get(1..).unwrap_or("");
+    log!(
+        Level::Debug,
+        "utils::extract_filter: {} -> filter: {} url: {}",
+        line,
+        filter,
+        url
+    );
+    (filter, url)
+}
+
 #[cfg(test)]
 mod tests {
-    extern crate tempfile;
+    use tempfile;
 
     use super::*;
 
@@ -832,19 +955,112 @@ mod tests {
     #[test]
     fn t_strwidth_stfl() {
         assert_eq!(strwidth_stfl(""), 0);
-        assert_eq!(strwidth_stfl("x<hi>x"), 3);
-        assert_eq!(strwidth_stfl("x<>x"), 4);
+        assert_eq!(strwidth_stfl("x<hi>x"), 2);
+        assert_eq!(strwidth_stfl("x<longtag>x</>"), 2);
+        assert_eq!(strwidth_stfl("x<>x"), 3);
+        assert_eq!(strwidth_stfl("x<>y<>z"), 5);
+        assert_eq!(strwidth_stfl("x<>hi>x"), 6);
         assert_eq!(strwidth_stfl("\u{F91F}"), 2);
         assert_eq!(strwidth_stfl("\u{0007}"), 0);
         assert_eq!(strwidth_stfl("<a"), 0); // #415
     }
 
+    #[test]
+    fn t_substr_with_width_given_string_empty() {
+        assert_eq!(substr_with_width("", 0), "");
+        assert_eq!(substr_with_width("", 1), "");
+    }
+
+    #[test]
+    fn t_substr_with_width_max_width_zero() {
+        assert_eq!(substr_with_width("world", 0), "");
+        assert_eq!(substr_with_width("", 0), "");
+    }
+
+    #[test]
+    fn t_substr_with_width_max_width_dont_split_codepoints() {
+        assert_eq!(substr_with_width("ＡＢＣＤＥＦ", 9), "ＡＢＣＤ");
+        assert_eq!(substr_with_width("ＡＢＣ", 4), "ＡＢ");
+        assert_eq!(substr_with_width("a>bcd", 3), "a>b");
+        assert_eq!(substr_with_width("ＡＢＣＤＥ", 10), "ＡＢＣＤＥ");
+        assert_eq!(substr_with_width("abc", 2), "ab");
+    }
+
+    #[test]
+    fn t_substr_with_width_max_width_does_count_stfl_tag() {
+        assert_eq!(substr_with_width("ＡＢＣ<b>ＤＥ</b>Ｆ", 9), "ＡＢＣ<b>");
+        assert_eq!(substr_with_width("<foobar>ＡＢＣ", 4), "<foo");
+        assert_eq!(substr_with_width("a<<xyz>>bcd", 3), "a<<");
+        assert_eq!(substr_with_width("ＡＢＣ<b>ＤＥ", 10), "ＡＢＣ<b>");
+        assert_eq!(substr_with_width("a</>b</>c</>", 2), "a<");
+    }
+
+    #[test]
+    fn t_substr_with_width_max_width_count_marks_as_regular_characters() {
+        assert_eq!(substr_with_width("<><><>", 2), "<>");
+        assert_eq!(substr_with_width("a<>b<>c", 3), "a<>");
+    }
+
+    #[test]
+    fn t_substr_with_width_max_width_non_printable() {
+        assert_eq!(substr_with_width("\x01\x02abc", 1), "\x01\x02a");
+    }
+
+    #[test]
+    fn t_substr_with_width_stfl_given_string_empty() {
+        assert_eq!(substr_with_width_stfl("", 0), "");
+        assert_eq!(substr_with_width_stfl("", 1), "");
+    }
+
+    #[test]
+    fn t_substr_with_width_stfl_max_width_zero() {
+        assert_eq!(substr_with_width_stfl("world", 0), "");
+        assert_eq!(substr_with_width_stfl("", 0), "");
+    }
+
+    #[test]
+    fn t_substr_with_width_stfl_max_width_dont_split_codepoints() {
+        assert_eq!(
+            substr_with_width_stfl("ＡＢＣ<b>ＤＥ</b>Ｆ", 9),
+            "ＡＢＣ<b>Ｄ"
+        );
+        assert_eq!(substr_with_width_stfl("<foobar>ＡＢＣ", 4), "<foobar>ＡＢ");
+        assert_eq!(substr_with_width_stfl("a<<xyz>>bcd", 3), "a<<xyz>>b"); // tag: "<<xyz>"
+        assert_eq!(substr_with_width_stfl("ＡＢＣ<b>ＤＥ", 10), "ＡＢＣ<b>ＤＥ");
+        assert_eq!(substr_with_width_stfl("a</>b</>c</>", 2), "a</>b</>");
+    }
+
+    #[test]
+    fn t_substr_with_width_stfl_max_width_do_not_count_stfl_tag() {
+        assert_eq!(
+            substr_with_width_stfl("ＡＢＣ<b>ＤＥ</b>Ｆ", 9),
+            "ＡＢＣ<b>Ｄ"
+        );
+        assert_eq!(substr_with_width_stfl("<foobar>ＡＢＣ", 4), "<foobar>ＡＢ");
+        assert_eq!(substr_with_width_stfl("a<<xyz>>bcd", 3), "a<<xyz>>b"); // tag: "<<xyz>"
+        assert_eq!(substr_with_width_stfl("ＡＢＣ<b>ＤＥ", 10), "ＡＢＣ<b>ＤＥ");
+        assert_eq!(substr_with_width_stfl("a</>b</>c</>", 2), "a</>b</>");
+    }
+
+    #[test]
+    fn t_substr_with_width_stfl_max_width_count_escaped_less_than_mark() {
+        assert_eq!(substr_with_width_stfl("<><><>", 2), "<><>");
+        assert_eq!(substr_with_width_stfl("a<>b<>c", 3), "a<>b");
+    }
+
+    #[test]
+    fn t_substr_with_width_stfl_max_width_non_printable() {
+        assert_eq!(substr_with_width_stfl("\x01\x02abc", 1), "\x01\x02a");
+    }
+
     #[test]
     fn t_is_valid_podcast_type() {
         assert!(is_valid_podcast_type("audio/mpeg"));
         assert!(is_valid_podcast_type("audio/mp3"));
         assert!(is_valid_podcast_type("audio/x-mp3"));
         assert!(is_valid_podcast_type("audio/ogg"));
+        assert!(is_valid_podcast_type("video/x-matroska"));
+        assert!(is_valid_podcast_type("video/webm"));
         assert!(is_valid_podcast_type("application/ogg"));
 
         assert!(!is_valid_podcast_type("image/jpeg"));
@@ -853,6 +1069,24 @@ mod tests {
         assert!(!is_valid_podcast_type("application/zip"));
     }
 
+    #[test]
+    fn t_podcast_mime_to_link_type() {
+        use crate::htmlrenderer::LinkType::*;
+
+        assert_eq!(podcast_mime_to_link_type("audio/mpeg"), Some(Audio));
+        assert_eq!(podcast_mime_to_link_type("audio/mp3"), Some(Audio));
+        assert_eq!(podcast_mime_to_link_type("audio/x-mp3"), Some(Audio));
+        assert_eq!(podcast_mime_to_link_type("audio/ogg"), Some(Audio));
+        assert_eq!(podcast_mime_to_link_type("video/x-matroska"), Some(Video));
+        assert_eq!(podcast_mime_to_link_type("video/webm"), Some(Video));
+        assert_eq!(podcast_mime_to_link_type("application/ogg"), Some(Audio));
+
+        assert_eq!(podcast_mime_to_link_type("image/jpeg"), None);
+        assert_eq!(podcast_mime_to_link_type("image/png"), None);
+        assert_eq!(podcast_mime_to_link_type("text/plain"), None);
+        assert_eq!(podcast_mime_to_link_type("application/zip"), None);
+    }
+
     #[test]
     fn t_is_valid_attribte() {
         let invalid = ["foo", "bar", "baz", "quux"];
@@ -920,8 +1154,8 @@ mod tests {
 
     #[test]
     fn t_run_command_executes_given_command_with_given_argument() {
-        use self::tempfile::TempDir;
         use std::{thread, time};
+        use tempfile::TempDir;
 
         let tmp = TempDir::new().unwrap();
         let filepath = {
@@ -933,11 +1167,11 @@ mod tests {
 
         run_command("touch", filepath.to_str().unwrap());
 
-        // Busy-wait for 10 tries of 10 milliseconds each, waiting for `touch` to
+        // Busy-wait for 100 tries of 10 milliseconds each, waiting for `touch` to
         // create the file. Usually it happens quickly, and the loop exists on the
         // first try; but sometimes on CI it takes longer for `touch` to finish, so
         // we need a slightly longer wait.
-        for _ in 0..10 {
+        for _ in 0..100 {
             thread::sleep(time::Duration::from_millis(10));
 
             if filepath.exists() {
@@ -1090,9 +1324,9 @@ mod tests {
 
     #[test]
     fn t_mkdir_parents() {
-        use self::tempfile::TempDir;
         use std::fs;
         use std::os::unix::fs::PermissionsExt;
+        use tempfile::TempDir;
 
         let mode: u32 = 0o700;
         let tmp_dir = TempDir::new().unwrap();
@@ -1192,4 +1426,27 @@ mod tests {
         let input = expected.to_owned() + "# and a comment";
         assert_eq!(strip_comments(&input), expected);
     }
+
+    #[test]
+    fn t_extract_filter() {
+        let expected = ("~/bin/script.sh", "https://newsboat.org");
+        let input = "filter:~/bin/script.sh:https://newsboat.org";
+        assert_eq!(extract_filter(input), expected);
+
+        let expected = ("", "https://newsboat.org");
+        let input = "filter::https://newsboat.org";
+        assert_eq!(extract_filter(input), expected);
+
+        let expected = ("https", "//newsboat.org");
+        let input = "filter:https://newsboat.org";
+        assert_eq!(extract_filter(input), expected);
+
+        let expected = ("foo", "");
+        let input = "filter:foo:";
+        assert_eq!(extract_filter(input), expected);
+
+        let expected = ("", "");
+        let input = "filter:";
+        assert_eq!(extract_filter(input), expected);
+    }
 }
