@@ -769,12 +769,12 @@ void Cache::cleanup_cache(std::vector<std::shared_ptr<RssFeed>>& feeds)
 		std::string cleanup_rss_feeds_statement(
 			"DELETE FROM rss_feed WHERE rssurl NOT IN ");
 		cleanup_rss_feeds_statement.append(list);
-		cleanup_rss_feeds_statement.append(1, ';');
+		cleanup_rss_feeds_statement.push_back(';');
 
 		std::string cleanup_rss_items_statement(
 			"DELETE FROM rss_item WHERE feedurl NOT IN ");
 		cleanup_rss_items_statement.append(list);
-		cleanup_rss_items_statement.append(1, ';');
+		cleanup_rss_items_statement.push_back(';');
 
 		std::string cleanup_read_items_statement(
 			"UPDATE rss_item SET deleted = 1 WHERE unread = 0");
@@ -812,13 +812,13 @@ void Cache::update_rssitem_unlocked(std::shared_ptr<RssItem> item,
 					"'%q';",
 					item->guid());
 			run_sql(query, single_string_callback, &content);
-			if (content != item->description_raw()) {
+			if (content != item->description()) {
 				LOG(Level::DEBUG,
 					"Cache::update_rssitem_unlocked: '%s' "
 					"is "
 					"different from '%s'",
 					content,
-					item->description_raw());
+					item->description());
 				query = prepare_query(
 						"UPDATE rss_item SET unread = 1 WHERE "
 						"guid = '%q';",
@@ -836,11 +836,11 @@ void Cache::update_rssitem_unlocked(std::shared_ptr<RssItem> item,
 					"enclosure_type = '%q', base = '%q', unread = "
 					"'%d' "
 					"WHERE guid = '%q'",
-					item->title_raw(),
-					item->author_raw(),
+					item->title(),
+					item->author(),
 					item->link(),
 					feedurl,
-					item->description_raw(),
+					item->description(),
 					item->enclosure_url(),
 					item->enclosure_type(),
 					item->get_base(),
@@ -854,11 +854,11 @@ void Cache::update_rssitem_unlocked(std::shared_ptr<RssItem> item,
 					"content = '%q', enclosure_url = '%q', "
 					"enclosure_type = '%q', base = '%q' "
 					"WHERE guid = '%q'",
-					item->title_raw(),
-					item->author_raw(),
+					item->title(),
+					item->author(),
 					item->link(),
 					feedurl,
-					item->description_raw(),
+					item->description(),
 					item->enclosure_url(),
 					item->enclosure_type(),
 					item->get_base(),
@@ -876,12 +876,12 @@ void Cache::update_rssitem_unlocked(std::shared_ptr<RssItem> item,
 				" "
 				"'%q')",
 				item->guid(),
-				item->title_raw(),
-				item->author_raw(),
+				item->title(),
+				item->author(),
 				item->link(),
 				feedurl,
 				item->pubDate_timestamp(),
-				item->description_raw(),
+				item->description(),
 				(item->unread() ? 1 : 0),
 				item->enclosure_url(),
 				item->enclosure_type(),
@@ -1030,19 +1030,6 @@ void Cache::remove_old_deleted_items(RssFeed* feed)
 	run_sql(query);
 }
 
-unsigned int Cache::get_unread_count()
-{
-	std::lock_guard<std::mutex> lock(mtx);
-
-	std::string countquery =
-		"SELECT count(id) FROM rss_item WHERE unread = 1;";
-	CbHandler count_cbh;
-	run_sql(countquery, count_callback, &count_cbh);
-	unsigned int count = static_cast<unsigned int>(count_cbh.count());
-	LOG(Level::DEBUG, "Cache::get_unread_count: count = %u", count);
-	return count;
-}
-
 void Cache::mark_items_read_by_guid(const std::vector<std::string>& guids)
 {
 	ScopeMeasure m1("Cache::mark_items_read_by_guid");
