@@ -9,6 +9,7 @@
 
 #include "config.h"
 #include "configparser.h"
+#include "configdata.h"
 #include "confighandlerexception.h"
 #include "logger.h"
 #include "strprintf.h"
@@ -49,6 +50,7 @@ ConfigContainer::ConfigContainer()
 	{
 		"delete-read-articles-on-quit",
 		ConfigData("false", ConfigDataType::BOOL)},
+	{"delete-played-files", ConfigData("false", ConfigDataType::BOOL)},
 	{
 		"display-article-progress",
 		ConfigData("yes", ConfigDataType::BOOL)},
@@ -102,6 +104,8 @@ ConfigContainer::ConfigContainer()
 		ConfigData("download",
 			std::unordered_set<std::string>(
 		{"download", "display"}))},
+	{"inoreader-app-id", ConfigData("", ConfigDataType::STR)},
+	{"inoreader-app-key", ConfigData("", ConfigDataType::STR)},
 	{"inoreader-login", ConfigData("", ConfigDataType::STR)},
 	{"inoreader-password", ConfigData("", ConfigDataType::STR)},
 	{
@@ -167,7 +171,7 @@ ConfigContainer::ConfigContainer()
 		ConfigData("no", ConfigDataType::BOOL)},
 	{
 		"podlist-format",
-		ConfigData( "%4i [%6dMB/%6tMB] [%5p %%] [%7k kb/s] %-20S %u -> %F", ConfigDataType::STR)},
+		ConfigData( "%4i [%6dMB/%6tMB] [%5p %%] [%12K] %-20S %u -> %F", ConfigDataType::STR)},
 	{
 		"prepopulate-query-feeds",
 		ConfigData("false", ConfigDataType::BOOL)},
@@ -257,18 +261,18 @@ ConfigContainer::ConfigContainer()
 	/* title formats: */
 	{
 		"articlelist-title-format",
-		ConfigData(_("%N %V - Articles in feed '%T' (%u "
-				"unread, %t "
-				"total) - %U"),
+		ConfigData(_("%N %V - Articles in feed '%T' (%u unread, %t total)"
+				"%?F? matching filter `%F'&?"
+				" - %U"),
 			ConfigDataType::STR)},
 	{
 		"dialogs-title-format",
 		ConfigData(_("%N %V - Dialogs"), ConfigDataType::STR)},
 	{
 		"feedlist-title-format",
-		ConfigData(_("%N %V - Your feeds (%u unread, %t "
-				"total)%?T? - "
-				"tag `%T'&?"),
+		ConfigData(_("%N %V - %?F?Feeds&Your feeds? (%u unread, %t total)"
+				"%?F? matching filter `%F'&?"
+				"%?T? - tag `%T'&?"),
 			ConfigDataType::STR)},
 	{
 		"filebrowser-title-format",
@@ -288,8 +292,8 @@ ConfigContainer::ConfigContainer()
 			ConfigDataType::STR)},
 	{
 		"searchresult-title-format",
-		ConfigData(_("%N %V - Search result (%u unread, %t "
-				"total)"),
+		ConfigData(_("%N %V - Search results (%u unread, %t total)"
+				"%?F? matching filter `%F'&?"),
 			ConfigDataType::STR)},
 	{
 		"selectfilter-title-format",
@@ -301,7 +305,10 @@ ConfigContainer::ConfigContainer()
 			ConfigDataType::STR)},
 	{
 		"urlview-title-format",
-		ConfigData(_("%N %V - URLs"), ConfigDataType::STR)}}
+		ConfigData(_("%N %V - URLs"), ConfigDataType::STR)},
+	{
+		"wrap-scroll",
+		ConfigData("no", ConfigDataType::BOOL)}}
 {
 }
 
@@ -330,54 +337,39 @@ void ConfigContainer::handle_action(const std::string& action,
 
 	// ConfigDataType::INVALID indicates that the action didn't exist, and
 	// that the returned object was created ad-hoc.
-	if (cfgdata.type == ConfigDataType::INVALID) {
+	if (cfgdata.type() == ConfigDataType::INVALID) {
 		LOG(Level::WARN,
 			"ConfigContainer::handle_action: unknown action %s",
 			action);
-		throw ConfigHandlerException(
-			ActionHandlerStatus::INVALID_COMMAND);
+		throw ConfigHandlerException(ActionHandlerStatus::INVALID_COMMAND);
 	}
 
 	LOG(Level::DEBUG,
 		"ConfigContainer::handle_action: action = %s, type = %u",
 		action,
-		static_cast<unsigned int>(cfgdata.type));
+		static_cast<unsigned int>(cfgdata.type()));
 
 	if (params.size() < 1) {
-		throw ConfigHandlerException(
-			ActionHandlerStatus::TOO_FEW_PARAMS);
+		throw ConfigHandlerException(ActionHandlerStatus::TOO_FEW_PARAMS);
 	}
 
-	switch (cfgdata.type) {
+	switch (cfgdata.type()) {
 	case ConfigDataType::BOOL:
-		if (!is_bool(params[0]))
-			throw ConfigHandlerException(strprintf::fmt(
-					_("expected boolean value, found `%s' instead"),
-					params[0]));
-		cfgdata.value = params[0];
-		break;
-
 	case ConfigDataType::INT:
-		if (!is_int(params[0]))
-			throw ConfigHandlerException(strprintf::fmt(
-					_("expected integer value, found `%s' instead"),
-					params[0]));
-		cfgdata.value = params[0];
-		break;
+	case ConfigDataType::ENUM: {
+		const auto result = cfgdata.set_value(params[0]);
+		if (!result) {
+			throw ConfigHandlerException(result.error());
+		}
+	}
+	break;
 
-	case ConfigDataType::ENUM:
-		if (cfgdata.enum_values.find(params[0]) ==
-			cfgdata.enum_values.end())
-			throw ConfigHandlerException(strprintf::fmt(
-					_("invalid configuration value `%s'"),
-					params[0]));
-	// fall-through
 	case ConfigDataType::STR:
 	case ConfigDataType::PATH:
-		if (cfgdata.multi_option) {
-			cfgdata.value = utils::join(params, " ");
+		if (cfgdata.multi_option()) {
+			cfgdata.set_value(utils::join(params, " "));
 		} else {
-			cfgdata.value = params[0];
+			cfgdata.set_value(params[0]);
 		}
 		break;
 
@@ -387,43 +379,47 @@ void ConfigContainer::handle_action(const std::string& action,
 	}
 }
 
-bool ConfigContainer::is_bool(const std::string& s)
-{
-	const auto bool_values = std::vector<std::string>(
-	{"yes", "no", "true", "false"});
-	return (std::find(bool_values.begin(), bool_values.end(), s) !=
-			bool_values.end());
-}
-
-bool ConfigContainer::is_int(const std::string& s)
-{
-	return std::all_of(s.begin(), s.end(), ::isdigit);
-}
-
-std::string ConfigContainer::get_configvalue(const std::string& key)
+std::string ConfigContainer::get_configvalue(const std::string& key) const
 {
 	std::lock_guard<std::recursive_mutex> guard(config_data_mtx);
-	std::string retval = config_data[key].value;
-	if (config_data[key].type == ConfigDataType::PATH) {
-		retval = utils::resolve_tilde(retval);
+	auto it = config_data.find(key);
+	if (it != config_data.cend()) {
+		const auto& entry = it->second;
+		std::string value = entry.value();
+		if (entry.type() == ConfigDataType::PATH) {
+			value = utils::resolve_tilde(value);
+		}
+		return value;
 	}
-	return retval;
+
+	return {};
 }
 
-int ConfigContainer::get_configvalue_as_int(const std::string& key)
+int ConfigContainer::get_configvalue_as_int(const std::string& key) const
 {
 	std::lock_guard<std::recursive_mutex> guard(config_data_mtx);
-	std::istringstream is(config_data[key].value);
-	int i;
-	is >> i;
-	return i;
+	auto it = config_data.find(key);
+	if (it != config_data.cend()) {
+		const auto& value = it->second.value();
+		std::istringstream is(value);
+		int i;
+		is >> i;
+		return i;
+	}
+
+	return 0;
 }
 
-bool ConfigContainer::get_configvalue_as_bool(const std::string& key)
+bool ConfigContainer::get_configvalue_as_bool(const std::string& key) const
 {
 	std::lock_guard<std::recursive_mutex> guard(config_data_mtx);
-	std::string value = config_data[key].value;
-	return (value == "true" || value == "yes");
+	auto it = config_data.find(key);
+	if (it != config_data.cend()) {
+		const auto& value = it->second.value();
+		return (value == "true" || value == "yes");
+	}
+
+	return false;
 }
 
 void ConfigContainer::set_configvalue(const std::string& key,
@@ -434,19 +430,19 @@ void ConfigContainer::set_configvalue(const std::string& key,
 		key,
 		value);
 	std::lock_guard<std::recursive_mutex> guard(config_data_mtx);
-	config_data[key].value = value;
+	config_data[key].set_value(value);
 }
 
 void ConfigContainer::reset_to_default(const std::string& key)
 {
 	std::lock_guard<std::recursive_mutex> guard(config_data_mtx);
-	config_data[key].value = config_data[key].default_value;
+	config_data[key].reset_to_default();
 }
 
 void ConfigContainer::toggle(const std::string& key)
 {
 	std::lock_guard<std::recursive_mutex> guard(config_data_mtx);
-	if (config_data[key].type == ConfigDataType::BOOL) {
+	if (config_data[key].type() == ConfigDataType::BOOL) {
 		set_configvalue(key,
 			std::string(get_configvalue_as_bool(key) ? "false"
 				: "true"));
@@ -458,34 +454,33 @@ void ConfigContainer::dump_config(std::vector<std::string>& config_output)
 	std::lock_guard<std::recursive_mutex> guard(config_data_mtx);
 	for (const auto& cfg : config_data) {
 		std::string configline = cfg.first + " ";
-		assert(cfg.second.type != ConfigDataType::INVALID);
-		switch (cfg.second.type) {
+		assert(cfg.second.type() != ConfigDataType::INVALID);
+		switch (cfg.second.type()) {
 		case ConfigDataType::BOOL:
 		case ConfigDataType::INT:
-			configline.append(cfg.second.value);
-			if (cfg.second.value != cfg.second.default_value)
+			configline.append(cfg.second.value());
+			if (cfg.second.value() != cfg.second.default_value()) {
 				configline.append(
-					strprintf::fmt(" # default: %s",
-						cfg.second.default_value));
+					strprintf::fmt(
+						" # default: %s",
+						cfg.second.default_value()));
+			}
 			break;
 		case ConfigDataType::ENUM:
 		case ConfigDataType::STR:
 		case ConfigDataType::PATH:
-			if (cfg.second.multi_option) {
-				std::vector<std::string> tokens =
-					utils::tokenize(cfg.second.value, " ");
+			if (cfg.second.multi_option()) {
+				const std::vector<std::string> tokens = utils::tokenize(cfg.second.value(),
+						" ");
 				for (const auto& token : tokens) {
-					configline.append(
-						utils::quote(token) + " ");
+					configline.append(utils::quote(token) + " ");
 				}
 			} else {
-				configline.append(
-					utils::quote(cfg.second.value));
-				if (cfg.second.value !=
-					cfg.second.default_value) {
+				configline.append(utils::quote(cfg.second.value()));
+				if (cfg.second.value() != cfg.second.default_value()) {
 					configline.append(strprintf::fmt(
 							" # default: %s",
-							cfg.second.default_value));
+							cfg.second.default_value()));
 				}
 			}
 			break;
@@ -499,7 +494,7 @@ void ConfigContainer::dump_config(std::vector<std::string>& config_output)
 }
 
 std::vector<std::string> ConfigContainer::get_suggestions(
-	const std::string& fragment)
+	const std::string& fragment) const
 {
 	std::vector<std::string> result;
 	std::lock_guard<std::recursive_mutex> guard(config_data_mtx);
@@ -512,18 +507,18 @@ std::vector<std::string> ConfigContainer::get_suggestions(
 	return result;
 }
 
-FeedSortStrategy ConfigContainer::get_feed_sort_strategy()
+FeedSortStrategy ConfigContainer::get_feed_sort_strategy() const
 {
 	FeedSortStrategy ss;
-	const auto sortmethod_info =
-		utils::tokenize(get_configvalue("feed-sort-order"), "-");
-	const std::string sortmethod = sortmethod_info[0];
 
-	std::string direction = "desc";
-	if (sortmethod_info.size() > 1) {
-		direction = sortmethod_info[1];
+	const auto setting = get_configvalue("feed-sort-order");
+	if (setting.empty()) {
+		return ss;
 	}
 
+	const auto sortmethod_info = utils::tokenize(setting, "-");
+	const std::string sortmethod = sortmethod_info[0];
+
 	if (sortmethod == "none") {
 		ss.sm = FeedSortMethod::NONE;
 	} else if (sortmethod == "firsttag") {
@@ -538,6 +533,11 @@ FeedSortStrategy ConfigContainer::get_feed_sort_strategy()
 		ss.sm = FeedSortMethod::LAST_UPDATED;
 	}
 
+	std::string direction = "desc";
+	if (sortmethod_info.size() > 1) {
+		direction = sortmethod_info[1];
+	}
+
 	if (direction == "asc") {
 		ss.sd = SortDirection::ASC;
 	} else if (direction == "desc") {
@@ -547,7 +547,7 @@ FeedSortStrategy ConfigContainer::get_feed_sort_strategy()
 	return ss;
 }
 
-ArticleSortStrategy ConfigContainer::get_article_sort_strategy()
+ArticleSortStrategy ConfigContainer::get_article_sort_strategy() const
 {
 	ArticleSortStrategy ss;
 	const auto methods =
