@@ -66,8 +66,9 @@ bool ConfigParser::parse(const std::string& tmp_filename)
 
 	// It would be nice if this function was only give absolute paths, but the
 	// tests are easier as relative paths
-	const std::string filename = (tmp_filename.front() == '/') ? tmp_filename :
-		utils::getcwd() + '/' + tmp_filename;
+	const std::string filename = (tmp_filename.front() == NEWSBEUTER_PATH_SEP) ?
+		tmp_filename :
+		utils::getcwd() + NEWSBEUTER_PATH_SEP + tmp_filename;
 
 	if (std::find(included_files.begin(), included_files.end(),
 			filename) != included_files.end()) {
