@@ -29,6 +29,7 @@
 #include "configcontainer.h"
 #include "configexception.h"
 #include "configparser.h"
+#include "configpaths.h"
 #include "dbexception.h"
 #include "downloadthread.h"
 #include "exception.h"
@@ -73,12 +74,13 @@ void ignore_signal(int sig)
 	LOG(Level::WARN, "caught signal %d but ignored it", sig);
 }
 
-Controller::Controller()
+Controller::Controller(ConfigPaths& configpaths)
 	: v(0)
 	, urlcfg(0)
 	, rsscache(0)
 	, refresh_on_start(false)
 	, api(0)
+	, configpaths(configpaths)
 	, queueManager(&cfg, &configpaths)
 {
 }
@@ -104,31 +106,24 @@ int Controller::run(const CliArgsParser& args)
 	::signal(SIGPIPE, ignore_signal);
 	::signal(SIGHUP, sighup_action);
 
-	if (!configpaths.initialized()) {
-		std::cerr << configpaths.error_message() << std::endl;
-		return EXIT_FAILURE;
-	}
-
 	refresh_on_start = args.refresh_on_start();
 
-	if (args.set_log_file()) {
-		Logger::set_logfile(args.log_file());
+	if (args.log_file().has_value()) {
+		Logger::set_logfile(args.log_file().value());
 	}
 
-	if (args.set_log_level()) {
-		Logger::set_loglevel(args.log_level());
+	if (args.log_level().has_value()) {
+		Logger::set_loglevel(args.log_level().value());
 	}
 
 	if (!args.display_msg().empty()) {
 		std::cerr << args.display_msg() << std::endl;
 	}
 
-	if (args.should_return()) {
-		return args.return_code();
+	if (args.return_code().has_value()) {
+		return args.return_code().value();
 	}
 
-	configpaths.process_args(args);
-
 	const auto migrated = configpaths.try_migrate_from_newsbeuter();
 	if (migrated) {
 		std::cerr << "\nPlease check the results and press Enter to "
@@ -141,12 +136,8 @@ int Controller::run(const CliArgsParser& args)
 	}
 
 	if (args.do_import()) {
-		LOG(Level::INFO,
-			"Importing OPML file from %s",
-			args.importfile());
-		urlcfg = new FileUrlReader(configpaths.url_file());
-		urlcfg->reload();
-		import_opml(args.importfile());
+		LOG(Level::INFO, "Importing OPML file from %s", args.importfile());
+		import_opml(args.importfile(), configpaths.url_file());
 		return EXIT_SUCCESS;
 	}
 
@@ -162,11 +153,9 @@ int Controller::run(const CliArgsParser& args)
 		fslock = std::unique_ptr<FsLock>(new FsLock());
 		pid_t pid;
 		if (!fslock->try_lock(configpaths.lock_file(), pid)) {
-			if (!args.execute_cmds()) {
+			if (!args.cmds_to_execute().has_value()) {
 				std::cout << strprintf::fmt(
-						_("Error: an instance of "
-							"%s is already running "
-							"(PID: %u)"),
+						_("Error: an instance of %s is already running (PID: %u)"),
 						PROGRAM_NAME,
 						pid)
 					<< std::endl;
@@ -216,7 +205,7 @@ int Controller::run(const CliArgsParser& args)
 
 	// create cache object
 	std::string cachefilepath = cfg.get_configvalue("cache-file");
-	if (cachefilepath.length() > 0 && !args.set_cache_file()) {
+	if (cachefilepath.length() > 0 && !args.cache_file().has_value()) {
 		configpaths.set_cache_file(cachefilepath);
 		fslock = std::unique_ptr<FsLock>(new FsLock());
 		pid_t pid;
@@ -300,13 +289,20 @@ int Controller::run(const CliArgsParser& args)
 		api = new OcNewsApi(&cfg);
 		urlcfg = new OcNewsUrlReader(configpaths.url_file(), api);
 	} else if (type == "inoreader") {
+		const auto all_set = !cfg.get_configvalue("inoreader-app-id").empty()
+			&& !cfg.get_configvalue("inoreader-app-key").empty();
+		if (!all_set) {
+			std::cerr <<
+				_("ERROR: You must set *both* `inoreader-app-id` and `inoreader-app-key` to use Inoreader.\n");
+			return EXIT_FAILURE;
+		}
+
 		api = new InoreaderApi(&cfg);
-		urlcfg = new InoreaderUrlReader(
-			&cfg, configpaths.url_file(), api);
+		urlcfg = new InoreaderUrlReader(&cfg, configpaths.url_file(), api);
 	} else {
-		LOG(Level::ERROR,
-			"unknown urls-source `%s'",
-			urlcfg->get_source());
+		std::cerr << strprintf::fmt(_("ERROR: Unknown urls-source `%s'"),
+				type) << std::endl;
+		return EXIT_FAILURE;
 	}
 
 	if (!args.do_export() && !args.silent()) {
@@ -440,24 +436,24 @@ int Controller::run(const CliArgsParser& args)
 		return EXIT_SUCCESS;
 	}
 
-	if (args.do_read_import()) {
+	if (args.readinfo_import_file().has_value()) {
 		LOG(Level::INFO,
 			"Importing read information file from %s",
-			args.readinfo_import_file());
+			args.readinfo_import_file().value());
 		std::cout << _("Importing list of read articles...");
 		std::cout.flush();
-		import_read_information(args.readinfo_import_file());
+		import_read_information(args.readinfo_import_file().value());
 		std::cout << _("done.") << std::endl;
 		return EXIT_SUCCESS;
 	}
 
-	if (args.do_read_export()) {
+	if (args.readinfo_export_file().has_value()) {
 		LOG(Level::INFO,
 			"Exporting read information file to %s",
-			args.readinfo_export_file());
+			args.readinfo_export_file().value());
 		std::cout << _("Exporting list of read articles...");
 		std::cout.flush();
-		export_read_information(args.readinfo_export_file());
+		export_read_information(args.readinfo_export_file().value());
 		std::cout << _("done.") << std::endl;
 		return EXIT_SUCCESS;
 	}
@@ -467,10 +463,9 @@ int Controller::run(const CliArgsParser& args)
 	v->set_keymap(&keys);
 	v->set_tags(tags);
 	v->set_cache(rsscache);
-	v->set_filters(&filters);
 
-	if (args.execute_cmds()) {
-		execute_commands(args.cmds_to_execute());
+	if (args.cmds_to_execute().has_value()) {
+		execute_commands(args.cmds_to_execute().value());
 		return EXIT_SUCCESS;
 	}
 
@@ -597,7 +592,7 @@ void Controller::replace_feed(std::shared_ptr<RssFeed> oldfeed,
 {
 	std::lock_guard<std::mutex> feedslock(feeds_mutex);
 
-	LOG(Level::DEBUG, "Controller::replace_feed: feed is nonempty, saving");
+	LOG(Level::DEBUG, "Controller::replace_feed: saving");
 	rsscache->externalize_rssfeed(
 		newfeed, ign.matches_resetunread(newfeed->rssurl()));
 	LOG(Level::DEBUG,
@@ -625,17 +620,20 @@ void Controller::replace_feed(std::shared_ptr<RssFeed> oldfeed,
 	}
 }
 
-void Controller::import_opml(const std::string& filename)
+void Controller::import_opml(const std::string& opmlFile,
+	const std::string& urlFile)
 {
-	if (!opml::import(filename, urlcfg)) {
+	auto urlReader = FileUrlReader(urlFile);
+	urlReader.reload(); // Load existing URLs
+
+	if (!opml::import(opmlFile, urlReader)) {
 		std::cout << strprintf::fmt(
-				_("An error occurred while parsing %s."),
-				filename)
+				_("An error occurred while parsing %s."), opmlFile)
 			<< std::endl;
 		return;
 	} else {
 		std::cout << strprintf::fmt(
-				_("Import of %s finished."), filename)
+				_("Import of %s finished."), opmlFile)
 			<< std::endl;
 	}
 }
@@ -766,7 +764,7 @@ int Controller::execute_commands(const std::vector<std::string>& cmds)
 			reloader->reload_all(true);
 		} else if (cmd == "print-unread") {
 			std::cout << strprintf::fmt(_("%u unread articles"),
-					rsscache->get_unread_count())
+					feedcontainer.unread_item_count())
 				<< std::endl;
 		} else {
 			std::cerr
@@ -853,9 +851,6 @@ void Controller::export_read_information(const std::string& readinfofile)
 
 void Controller::update_config()
 {
-	v->set_regexmanager(&rxman);
-	v->update_bindings();
-
 	if (colorman.colors_loaded()) {
 		v->set_colors(colorman.get_fgcolors(),
 			colorman.get_bgcolors(),
@@ -893,7 +888,7 @@ void Controller::dump_config(const std::string& filename)
 	std::vector<std::string> configlines;
 	cfg.dump_config(configlines);
 	if (v) {
-		v->get_keys()->dump_config(configlines);
+		v->get_keymap()->dump_config(configlines);
 	}
 	ign.dump_config(configlines);
 	filters.dump_config(configlines);
