@@ -1,6 +1,7 @@
 #include "dialogsformaction.h"
 
 #include <cstdio>
+#include <string>
 
 #include "config.h"
 #include "fmtstrformatter.h"
@@ -16,6 +17,7 @@ DialogsFormAction::DialogsFormAction(View* vv,
 	ConfigContainer* cfg)
 	: FormAction(vv, formstr, cfg)
 	, update_list(true)
+	, dialogs_list("dialogs", FormAction::f)
 {
 }
 
@@ -25,12 +27,14 @@ void DialogsFormAction::init()
 {
 	set_keymap_hints();
 
-	unsigned int width = utils::to_u(f->get("dialogs:w"));
+	f.run(-3); // compute all widget dimensions
+
+	const unsigned int width = dialogs_list.get_width();
 	std::string title_format = cfg->get_configvalue("dialogs-title-format");
 	FmtStrFormatter fmt;
 	fmt.register_fmt('N', PROGRAM_NAME);
 	fmt.register_fmt('V', utils::program_version());
-	f->set("head", fmt.do_format(title_format, width));
+	f.set("head", fmt.do_format(title_format, width));
 }
 
 void DialogsFormAction::prepare()
@@ -45,18 +49,19 @@ void DialogsFormAction::prepare()
 				v->get_formaction(fa.first).get(),
 				get_parent_formaction().get());
 			listfmt.add_line(
-				strprintf::fmt("%4u %s %s",
-					i,
-					(v->get_formaction(fa.first).get() ==
-						get_parent_formaction().get())
-					? "*"
-					: " ",
-					fa.second),
-				fa.first);
+				utils::quote_for_stfl(
+					strprintf::fmt("%4u %s %s",
+						i,
+						(v->get_formaction(fa.first).get() ==
+							get_parent_formaction().get())
+						? "*"
+						: " ",
+						fa.second)),
+				std::to_string(fa.first));
 			i++;
 		}
 
-		f->modify("dialogs", "replace_inner", listfmt.format_list());
+		dialogs_list.stfl_replace_lines(listfmt);
 
 		update_list = false;
 	}
@@ -72,43 +77,52 @@ KeyMapHintEntry* DialogsFormAction::get_keymap_hint()
 	return hints;
 }
 
-void DialogsFormAction::process_operation(Operation op,
+bool DialogsFormAction::process_operation(Operation op,
 	bool /* automatic */,
 	std::vector<std::string>* /* args */)
 {
 	switch (op) {
 	case OP_OPEN: {
-		std::string dialogposname = f->get("dialogpos");
-		if (dialogposname.length() > 0) {
-			v->set_current_formaction(utils::to_u(dialogposname));
-		} else {
-			v->show_error(_("No item selected!"));
-		}
+		const unsigned int pos = dialogs_list.get_position();
+		v->set_current_formaction(pos);
 	}
 	break;
 	case OP_CLOSEDIALOG: {
-		std::string dialogposname = f->get("dialogpos");
-		if (dialogposname.length() > 0) {
-			unsigned int dialogpos = utils::to_u(dialogposname);
-			if (dialogpos != 0) {
-				v->remove_formaction(dialogpos);
-				update_list = true;
-			} else {
-				v->show_error(
-					_("Error: you can't remove the feed "
-						"list!"));
-			}
+		const unsigned int pos = dialogs_list.get_position();
+		if (pos != 0) {
+			v->remove_formaction(pos);
+			update_list = true;
 		} else {
-			v->show_error(_("No item selected!"));
+			v->show_error(
+				_("Error: you can't remove the feed list!"));
 		}
 	}
 	break;
+	case OP_SK_UP:
+		dialogs_list.move_up(cfg->get_configvalue_as_bool("wrap-scroll"));
+		break;
+	case OP_SK_DOWN:
+		dialogs_list.move_down(cfg->get_configvalue_as_bool("wrap-scroll"));
+		break;
+	case OP_SK_HOME:
+		dialogs_list.move_to_first();
+		break;
+	case OP_SK_END:
+		dialogs_list.move_to_last();
+		break;
+	case OP_SK_PGUP:
+		dialogs_list.move_page_up(cfg->get_configvalue_as_bool("wrap-scroll"));
+		break;
+	case OP_SK_PGDOWN:
+		dialogs_list.move_page_down(cfg->get_configvalue_as_bool("wrap-scroll"));
+		break;
 	case OP_QUIT:
 		v->pop_current_formaction();
 		break;
 	default:
 		break;
 	}
+	return true;
 }
 
 std::string DialogsFormAction::title()
@@ -121,7 +135,7 @@ void DialogsFormAction::handle_cmdline(const std::string& cmd)
 	unsigned int idx = 0;
 	if (1 == sscanf(cmd.c_str(), "%u", &idx)) {
 		if (idx <= v->formaction_stack_size()) {
-			f->set("dialogpos", std::to_string(idx - 1));
+			dialogs_list.set_position(idx - 1);
 		} else {
 			v->show_error(_("Invalid position!"));
 		}
