@@ -25,34 +25,31 @@ namespace newsboat {
 FeedListFormAction::FeedListFormAction(View* vv,
 	std::string formstr,
 	Cache* cc,
-	FilterContainer* f,
-	ConfigContainer* cfg)
+	FilterContainer& f,
+	ConfigContainer* cfg,
+	RegexManager& r)
 	: ListFormAction(vv, formstr, cfg)
 	, zero_feedpos(false)
-	, feeds_shown(0)
-	, quit(false)
 	, apply_filter(false)
 	, search_dummy_feed(new RssFeed(cc))
 	, filterpos(0)
 	, set_filterpos(false)
-	, rxman(0)
-	, old_width(0)
-	, unread_feeds(0)
-	, total_feeds(0)
+	, rxman(r)
 	, filters(f)
+	, feeds_list("feeds", FormAction::f)
 {
 	valid_cmds.push_back("tag");
 	valid_cmds.push_back("goto");
 	std::sort(valid_cmds.begin(), valid_cmds.end());
-	old_sort_order = cfg->get_configvalue("feed-sort-order");
 	search_dummy_feed->set_search_feed(true);
+	register_format_styles();
 }
 
 void FeedListFormAction::init()
 {
 	set_keymap_hints();
 
-	f->run(-3); // compute all widget dimensions
+	f.run(-3); // compute all widget dimensions
 
 	if (v->get_ctrl()->get_refresh_on_start()) {
 		v->get_ctrl()->get_reloader()->start_reload_all_thread();
@@ -72,20 +69,10 @@ FeedListFormAction::~FeedListFormAction() {}
 
 void FeedListFormAction::prepare()
 {
-	unsigned int width = utils::to_u(f->get("items:w"));
-
-	if (old_width != width) {
-		do_redraw = true;
-		old_width = width;
-		LOG(Level::DEBUG,
-			"FeedListFormAction::prepare: apparent resize");
-	}
-
-	std::string sort_order = cfg->get_configvalue("feed-sort-order");
-	if (sort_order != old_sort_order) {
-		v->get_ctrl()->get_feedcontainer()->sort_feeds(
-			cfg->get_feed_sort_strategy());
-		old_sort_order = sort_order;
+	const auto sort_strategy = cfg->get_feed_sort_strategy();
+	if (!old_sort_strategy || sort_strategy != *old_sort_strategy) {
+		v->get_ctrl()->get_feedcontainer()->sort_feeds(sort_strategy);
+		old_sort_strategy = sort_strategy;
 		do_redraw = true;
 	}
 
@@ -97,16 +84,17 @@ void FeedListFormAction::prepare()
 	}
 }
 
-void FeedListFormAction::process_operation(Operation op,
+bool FeedListFormAction::process_operation(Operation op,
 	bool automatic,
 	std::vector<std::string>* args)
 {
-	std::string feedpos = f->get("feedposname");
+	std::string feedpos = f.get("feedposname");
 	unsigned int pos = utils::to_u(feedpos);
+	bool quit = false;
 REDO:
 	switch (op) {
 	case OP_OPEN: {
-		if (f->get_focus() == "feeds") {
+		if (f.get_focus() == "feeds") {
 			if (automatic && args->size() > 0) {
 				pos = utils::to_u((*args)[0]);
 			}
@@ -114,7 +102,7 @@ REDO:
 				"FeedListFormAction: opening feed at position "
 				"`%s'",
 				feedpos);
-			if (feeds_shown > 0 && feedpos.length() > 0) {
+			if (visible_feeds.size() > 0 && feedpos.length() > 0) {
 				v->push_itemlist(pos);
 			} else {
 				// should not happen
@@ -127,7 +115,7 @@ REDO:
 		LOG(Level::INFO,
 			"FeedListFormAction: reloading feed at position `%s'",
 			feedpos);
-		if (feeds_shown > 0 && feedpos.length() > 0) {
+		if (visible_feeds.size() > 0 && feedpos.length() > 0) {
 			v->get_ctrl()->get_reloader()->reload(pos);
 		} else {
 			v->show_error(
@@ -141,6 +129,24 @@ REDO:
 	case OP_RELOADURLS:
 		v->get_ctrl()->reload_urls_file();
 		break;
+	case OP_SK_UP:
+		feeds_list.move_up(cfg->get_configvalue_as_bool("wrap-scroll"));
+		break;
+	case OP_SK_DOWN:
+		feeds_list.move_down(cfg->get_configvalue_as_bool("wrap-scroll"));
+		break;
+	case OP_SK_HOME:
+		feeds_list.move_to_first();
+		break;
+	case OP_SK_END:
+		feeds_list.move_to_last();
+		break;
+	case OP_SK_PGUP:
+		feeds_list.move_page_up(cfg->get_configvalue_as_bool("wrap-scroll"));
+		break;
+	case OP_SK_PGDOWN:
+		feeds_list.move_page_down(cfg->get_configvalue_as_bool("wrap-scroll"));
+		break;
 	case OP_SORT: {
 		/// This string is related to the letters in parentheses in the
 		/// "Sort by (f)irsttag/..." and "Reverse Sort by
@@ -224,7 +230,7 @@ REDO:
 	}
 	break;
 	case OP_OPENINBROWSER:
-		if (feeds_shown > 0 && feedpos.length() > 0) {
+		if (visible_feeds.size() > 0 && feedpos.length() > 0) {
 			std::shared_ptr<RssFeed> feed =
 				v->get_ctrl()->get_feedcontainer()->get_feed(
 					pos);
@@ -237,9 +243,16 @@ REDO:
 						feedpos,
 						feed->link());
 					if (!feed->link().empty()) {
-						v->open_in_browser(feed->link());
+						if (int err = v->open_in_browser(feed->link())) {
+							v->show_error(strprintf::fmt(_("Browser returned error code %i"), err));
+							return false;
+						}
 					} else if (!feed->rssurl().empty()) {
-						v->open_in_browser(feed->rssurl());
+						if (int err = v->open_in_browser(feed->rssurl())) {
+							v->show_error(strprintf::fmt(_("Browser returned error code %i"), err));
+							return false;
+						}
+
 					} else {
 						// rssurl can't be empty, so if we got to this branch,
 						// something is clearly wrong with Newsboat internals.
@@ -261,7 +274,7 @@ REDO:
 		}
 		break;
 	case OP_OPENALLUNREADINBROWSER:
-		if (feeds_shown > 0 && feedpos.length() > 0) {
+		if (visible_feeds.size() > 0 && feedpos.length() > 0) {
 			std::shared_ptr<RssFeed> feed =
 				v->get_ctrl()->get_feedcontainer()->get_feed(
 					pos);
@@ -271,14 +284,18 @@ REDO:
 					"unread "
 					"items in feed at position `%s'",
 					feedpos.c_str());
-				open_unread_items_in_browser(feed, false);
+				if (int err = open_unread_items_in_browser(feed, false)) {
+					v->show_error(strprintf::fmt(_("Browser returned error code %i"), err));
+					return false;
+				}
+
 			}
 		} else {
 			v->show_error(_("No feed selected!"));
 		}
 		break;
 	case OP_OPENALLUNREADINBROWSER_AND_MARK:
-		if (feeds_shown > 0 && feedpos.length() > 0) {
+		if (visible_feeds.size() > 0 && feedpos.length() > 0) {
 			std::shared_ptr<RssFeed> feed =
 				v->get_ctrl()->get_feedcontainer()->get_feed(
 					pos);
@@ -289,7 +306,12 @@ REDO:
 					"items in feed at position `%s' and "
 					"marking read",
 					feedpos.c_str());
-				open_unread_items_in_browser(feed, true);
+
+				if (int err = open_unread_items_in_browser(feed, true)) {
+					v->show_error(strprintf::fmt(_("Browser returned error code %i"), err));
+					return false;
+				}
+
 				do_redraw = true;
 			}
 		}
@@ -301,11 +323,12 @@ REDO:
 				cfg->get_configvalue_as_bool(
 					"reload-only-visible-feeds");
 			std::vector<int> idxs;
-			for (const auto& feed : visible_feeds) {
-				idxs.push_back(feed.second);
+			if (reload_only_visible_feeds) {
+				for (const auto& feed : visible_feeds) {
+					idxs.push_back(feed.second);
+				}
 			}
-			v->get_ctrl()->get_reloader()->start_reload_all_thread(
-				reload_only_visible_feeds ? &idxs : nullptr);
+			v->get_ctrl()->get_reloader()->start_reload_all_thread(idxs);
 		}
 		break;
 	case OP_MARKFEEDREAD: {
@@ -313,16 +336,15 @@ REDO:
 			"FeedListFormAction: marking feed read at position "
 			"`%s'",
 			feedpos);
-		if (feeds_shown > 0 && feedpos.length() > 0) {
+		if (visible_feeds.size() > 0 && feedpos.length() > 0) {
 			v->set_status(_("Marking feed read..."));
 			try {
 				v->get_ctrl()->mark_all_read(pos);
 				do_redraw = true;
 				v->set_status("");
 				bool show_read = cfg->get_configvalue_as_bool("show-read-feeds");
-				if (feeds_shown > (pos + 1) && show_read) {
-					f->set("feedpos",
-						std::to_string(pos + 1));
+				if (visible_feeds.size() > (pos + 1) && show_read) {
+					feeds_list.set_position(pos + 1);
 				}
 			} catch (const DbException& e) {
 				v->show_error(strprintf::fmt(
@@ -427,24 +449,24 @@ REDO:
 	}
 	break;
 	case OP_SELECTFILTER:
-		if (filters->size() > 0) {
+		if (filters.size() > 0) {
 			std::string newfilter;
 			if (automatic && args->size() > 0) {
 				newfilter = (*args)[0];
 			} else {
 				newfilter = v->select_filter(
-						filters->get_filters());
+						filters.get_filters());
 			}
 			if (newfilter != "") {
 				filterhistory.add_line(newfilter);
 				if (newfilter.length() > 0) {
-					if (!m.parse(newfilter)) {
+					if (!matcher.parse(newfilter)) {
 						v->show_error(strprintf::fmt(
 								_("Error: couldn't "
 									"parse filter "
 									"command `%s': %s"),
 								newfilter,
-								m.get_parse_error()));
+								matcher.get_parse_error()));
 					} else {
 						save_filterpos();
 						apply_filter = true;
@@ -522,6 +544,7 @@ REDO:
 			v->pop_current_formaction();
 		}
 	}
+	return true;
 }
 
 void FeedListFormAction::update_visible_feeds(
@@ -538,14 +561,12 @@ void FeedListFormAction::update_visible_feeds(
 		feed->set_index(i + 1);
 		if ((tag == "" || feed->matches_tag(tag)) &&
 			(show_read || feed->unread_item_count() > 0) &&
-			(!apply_filter || m.matches(feed.get())) &&
+			(!apply_filter || matcher.matches(feed.get())) &&
 			!feed->hidden()) {
 			visible_feeds.push_back(FeedPtrPosPair(feed, i));
 		}
 		i++;
 	}
-
-	feeds_shown = visible_feeds.size();
 }
 
 void FeedListFormAction::set_feedlist(
@@ -553,52 +574,25 @@ void FeedListFormAction::set_feedlist(
 {
 	assert(cfg != nullptr); // must not happen
 
-	unsigned int width = utils::to_u(f->get("feeds:w"));
-
-	unsigned int i = 0;
-	unread_feeds = 0;
+	const unsigned int width = feeds_list.get_width();
 
 	std::string feedlist_format = cfg->get_configvalue("feedlist-format");
 
-	ListFormatter listfmt;
+	ListFormatter listfmt(&rxman, "feedlist");
 
 	update_visible_feeds(feeds);
 
 	for (const auto& feed : visible_feeds) {
-		if (feed.first->unread_item_count() > 0) {
-			++unread_feeds;
-		}
-
 		listfmt.add_line(format_line(feedlist_format,
 				feed.first,
 				feed.second,
 				width),
-			feed.second);
-		i++;
+			std::to_string(feed.second));
 	}
 
-	total_feeds = i;
+	feeds_list.stfl_replace_lines(listfmt);
 
-	f->modify("feeds",
-		"replace_inner",
-		listfmt.format_list(rxman, "feedlist"));
-
-	std::string title_format =
-		cfg->get_configvalue("feedlist-title-format");
-
-	FmtStrFormatter fmt;
-	fmt.register_fmt('T', tag);
-	fmt.register_fmt('N', PROGRAM_NAME);
-	fmt.register_fmt('V', utils::program_version());
-	fmt.register_fmt('u', std::to_string(unread_feeds));
-	fmt.register_fmt('t', std::to_string(i));
-
-	f->set("head", fmt.do_format(title_format, width));
-}
-
-void FeedListFormAction::set_tags(const std::vector<std::string>& t)
-{
-	tags = t;
+	update_form_title(width);
 }
 
 KeyMapHintEntry* FeedListFormAction::get_keymap_hint()
@@ -619,7 +613,7 @@ KeyMapHintEntry* FeedListFormAction::get_keymap_hint()
 
 bool FeedListFormAction::jump_to_previous_unread_feed(unsigned int& feedpos)
 {
-	unsigned int curpos = utils::to_u(f->get("feedpos"));
+	const unsigned int curpos = feeds_list.get_position();
 	LOG(Level::DEBUG,
 		"FeedListFormAction::jump_to_previous_unread_feed: searching "
 		"for "
@@ -636,7 +630,7 @@ bool FeedListFormAction::jump_to_previous_unread_feed(unsigned int& feedpos)
 				"FeedListFormAction::jump_to_previous_unread_"
 				"feed:"
 				" hit");
-			f->set("feedpos", std::to_string(i));
+			feeds_list.set_position(i);
 			feedpos = visible_feeds[i].second;
 			return true;
 		}
@@ -653,7 +647,7 @@ bool FeedListFormAction::jump_to_previous_unread_feed(unsigned int& feedpos)
 				"FeedListFormAction::jump_to_previous_unread_"
 				"feed:"
 				" hit");
-			f->set("feedpos", std::to_string(i));
+			feeds_list.set_position(i);
 			feedpos = visible_feeds[i].second;
 			return true;
 		}
@@ -663,7 +657,7 @@ bool FeedListFormAction::jump_to_previous_unread_feed(unsigned int& feedpos)
 
 void FeedListFormAction::goto_feed(const std::string& str)
 {
-	unsigned int curpos = utils::to_u(f->get("feedpos"));
+	const unsigned int curpos = feeds_list.get_position();
 	LOG(Level::DEBUG,
 		"FeedListFormAction::goto_feed: curpos = %u str = `%s'",
 		curpos,
@@ -671,14 +665,14 @@ void FeedListFormAction::goto_feed(const std::string& str)
 	for (unsigned int i = curpos + 1; i < visible_feeds.size(); ++i) {
 		if (strcasestr(visible_feeds[i].first->title().c_str(),
 				str.c_str()) != nullptr) {
-			f->set("feedpos", std::to_string(i));
+			feeds_list.set_position(i);
 			return;
 		}
 	}
 	for (unsigned int i = 0; i <= curpos; ++i) {
 		if (strcasestr(visible_feeds[i].first->title().c_str(),
 				str.c_str()) != nullptr) {
-			f->set("feedpos", std::to_string(i));
+			feeds_list.set_position(i);
 			return;
 		}
 	}
@@ -698,7 +692,7 @@ bool FeedListFormAction::jump_to_random_unread_feed(unsigned int& feedpos)
 			unsigned int pos =
 				utils::get_random_value(visible_feeds.size());
 			if (visible_feeds[pos].first->unread_item_count() > 0) {
-				f->set("feedpos", std::to_string(pos));
+				feeds_list.set_position(pos);
 				feedpos = visible_feeds[pos].second;
 				break;
 			}
@@ -709,7 +703,7 @@ bool FeedListFormAction::jump_to_random_unread_feed(unsigned int& feedpos)
 
 bool FeedListFormAction::jump_to_next_unread_feed(unsigned int& feedpos)
 {
-	unsigned int curpos = utils::to_u(f->get("feedpos"));
+	const unsigned int curpos = feeds_list.get_position();
 	LOG(Level::DEBUG,
 		"FeedListFormAction::jump_to_next_unread_feed: searching for "
 		"unread feed");
@@ -725,7 +719,7 @@ bool FeedListFormAction::jump_to_next_unread_feed(unsigned int& feedpos)
 				"FeedListFormAction::jump_to_next_unread_feed:"
 				" "
 				"hit");
-			f->set("feedpos", std::to_string(i));
+			feeds_list.set_position(i);
 			feedpos = visible_feeds[i].second;
 			return true;
 		}
@@ -741,7 +735,7 @@ bool FeedListFormAction::jump_to_next_unread_feed(unsigned int& feedpos)
 				"FeedListFormAction::jump_to_next_unread_feed:"
 				" "
 				"hit");
-			f->set("feedpos", std::to_string(i));
+			feeds_list.set_position(i);
 			feedpos = visible_feeds[i].second;
 			return true;
 		}
@@ -751,7 +745,7 @@ bool FeedListFormAction::jump_to_next_unread_feed(unsigned int& feedpos)
 
 bool FeedListFormAction::jump_to_previous_feed(unsigned int& feedpos)
 {
-	unsigned int curpos = utils::to_u(f->get("feedpos"));
+	const unsigned int curpos = feeds_list.get_position();
 
 	if (curpos > 0) {
 		unsigned int i = curpos - 1;
@@ -759,7 +753,7 @@ bool FeedListFormAction::jump_to_previous_feed(unsigned int& feedpos)
 			"FeedListFormAction::jump_to_previous_feed: "
 			"visible_feeds[%u]",
 			i);
-		f->set("feedpos", std::to_string(i));
+		feeds_list.set_position(i);
 		feedpos = visible_feeds[i].second;
 		return true;
 	}
@@ -768,7 +762,7 @@ bool FeedListFormAction::jump_to_previous_feed(unsigned int& feedpos)
 
 bool FeedListFormAction::jump_to_next_feed(unsigned int& feedpos)
 {
-	unsigned int curpos = utils::to_u(f->get("feedpos"));
+	const unsigned int curpos = feeds_list.get_position();
 
 	if ((curpos + 1) < visible_feeds.size()) {
 		unsigned int i = curpos + 1;
@@ -776,7 +770,7 @@ bool FeedListFormAction::jump_to_next_feed(unsigned int& feedpos)
 			"FeedListFormAction::jump_to_next_feed: "
 			"visible_feeds[%u]",
 			i);
-		f->set("feedpos", std::to_string(i));
+		feeds_list.set_position(i);
 		feedpos = visible_feeds[i].second;
 		return true;
 	}
@@ -785,7 +779,7 @@ bool FeedListFormAction::jump_to_next_feed(unsigned int& feedpos)
 
 std::shared_ptr<RssFeed> FeedListFormAction::get_feed()
 {
-	unsigned int curpos = utils::to_u(f->get("feedpos"));
+	const unsigned int curpos = feeds_list.get_position();
 	return visible_feeds[curpos].first;
 }
 
@@ -861,7 +855,7 @@ void FeedListFormAction::mark_pos_if_visible(unsigned int pos)
 				"match, "
 				"setting position to %u",
 				vpos);
-			f->set("feedpos", std::to_string(vpos));
+			feeds_list.set_position(vpos);
 			return;
 		}
 		vpos++;
@@ -875,7 +869,7 @@ void FeedListFormAction::mark_pos_if_visible(unsigned int pos)
 				"match "
 				"in 2nd try, setting position to %u",
 				vpos);
-			f->set("feedpos", std::to_string(vpos));
+			feeds_list.set_position(vpos);
 			return;
 		}
 		vpos++;
@@ -884,32 +878,48 @@ void FeedListFormAction::mark_pos_if_visible(unsigned int pos)
 
 void FeedListFormAction::save_filterpos()
 {
-	unsigned int i = utils::to_u(f->get("feedpos"));
+	const unsigned int i = feeds_list.get_position();
 	if (i < visible_feeds.size()) {
 		filterpos = visible_feeds[i].second;
 		set_filterpos = true;
 	}
 }
 
-void FeedListFormAction::set_regexmanager(RegexManager* r)
+void FeedListFormAction::register_format_styles()
 {
-	rxman = r;
-	std::vector<std::string>& attrs = r->get_attrs("feedlist");
-	unsigned int i = 0;
-	std::string attrstr;
-	for (const auto& attribute : attrs) {
-		attrstr.append(
-			strprintf::fmt("@style_%u_normal:%s ", i, attribute));
-		attrstr.append(
-			strprintf::fmt("@style_%u_focus:%s ", i, attribute));
-		i++;
-	}
-	std::string textview = strprintf::fmt(
+	const std::string attrstr = rxman.get_attrs_stfl_string("feedlist", true);
+	const std::string textview = strprintf::fmt(
 			"{!list[feeds] .expand:vh style_normal[listnormal]: "
 			"style_focus[listfocus]:fg=yellow,bg=blue,attr=bold "
-			"pos_name[feedposname]: pos[feedpos]:0 %s richtext:1}",
+			"pos_name[feedposname]: pos[feeds_pos]:0 %s richtext:1}",
 			attrstr);
-	f->modify("feeds", "replace", textview);
+	feeds_list.stfl_replace_list(0, textview);
+}
+
+void FeedListFormAction::update_form_title(unsigned int width)
+{
+	std::string title_format =
+		cfg->get_configvalue("feedlist-title-format");
+
+	FmtStrFormatter fmt;
+	fmt.register_fmt('T', tag);
+	fmt.register_fmt('N', PROGRAM_NAME);
+	fmt.register_fmt('V', utils::program_version());
+	fmt.register_fmt('u', std::to_string(count_unread_feeds()));
+	fmt.register_fmt('t', std::to_string(visible_feeds.size()));
+	fmt.register_fmt('F', apply_filter ? matcher.get_expression() : "");
+
+	f.set("head", fmt.do_format(title_format, width));
+}
+
+unsigned int FeedListFormAction::count_unread_feeds()
+{
+	return std::count_if(
+			visible_feeds.begin(),
+			visible_feeds.end(),
+	[](const FeedPtrPosPair& feed) {
+		return feed.first->unread_item_count() > 0;
+	});
 }
 
 void FeedListFormAction::op_end_setfilter()
@@ -917,7 +927,7 @@ void FeedListFormAction::op_end_setfilter()
 	std::string filtertext = qna_responses[0];
 	filterhistory.add_line(filtertext);
 	if (filtertext.length() > 0) {
-		if (!m.parse(filtertext)) {
+		if (!matcher.parse(filtertext)) {
 			v->show_error(
 				_("Error: couldn't parse filter command!"));
 		} else {
@@ -971,7 +981,7 @@ void FeedListFormAction::handle_cmdline_num(unsigned int idx)
 		if (i == -1) {
 			v->show_error(_("Position not visible!"));
 		} else {
-			f->set("feedpos", std::to_string(i));
+			feeds_list.set_position(i);
 		}
 	} else {
 		v->show_error(_("Invalid position!"));
@@ -985,14 +995,14 @@ void FeedListFormAction::set_pos()
 		unsigned int i = 0;
 		for (const auto& feed : visible_feeds) {
 			if (feed.second == filterpos) {
-				f->set("feedpos", std::to_string(i));
+				feeds_list.set_position(i);
 				return;
 			}
 			i++;
 		}
-		f->set("feedpos", "0");
+		feeds_list.set_position(0);
 	} else if (zero_feedpos) {
-		f->set("feedpos", "0");
+		feeds_list.set_position(0);
 		zero_feedpos = false;
 	}
 }
@@ -1031,9 +1041,10 @@ std::string FeedListFormAction::format_line(const std::string& feedlist_format,
 	fmt.register_fmt('T', feed->get_firsttag());
 	fmt.register_fmt('l', utils::censor_url(feed->link()));
 	fmt.register_fmt('L', utils::censor_url(feed->rssurl()));
-	fmt.register_fmt('d', feed->description());
+	fmt.register_fmt('d', utils::utf8_to_locale(feed->description()));
 
 	auto formattedLine = fmt.do_format(feedlist_format, width);
+	formattedLine = utils::quote_for_stfl(formattedLine);
 	if (unread_count > 0) {
 		formattedLine = strprintf::fmt("<unread>%s</>", formattedLine);
 	}
@@ -1044,8 +1055,8 @@ std::string FeedListFormAction::format_line(const std::string& feedlist_format,
 std::string FeedListFormAction::title()
 {
 	return strprintf::fmt(_("Feed List - %u unread, %u total"),
-			unread_feeds,
-			total_feeds);
+			count_unread_feeds(),
+			static_cast<unsigned int>(visible_feeds.size()));
 }
 
 } // namespace newsboat
