@@ -16,6 +16,7 @@
 
 #include "config.h"
 #include "fmtstrformatter.h"
+#include "listformatter.h"
 #include "logger.h"
 #include "strprintf.h"
 #include "utils.h"
@@ -28,17 +29,18 @@ FileBrowserFormAction::FileBrowserFormAction(View* vv,
 	ConfigContainer* cfg)
 	: FormAction(vv, formstr, cfg)
 	, quit(false)
+	, files_list("files", FormAction::f)
 {
 	// In filebrowser, keyboard focus is at the input field, so user can't
 	// possibly use 'q' key to exit the dialog
-	KeyMap* keys = vv->get_keys();
+	KeyMap* keys = vv->get_keymap();
 	keys->set_key(OP_QUIT, "ESC", id());
 	vv->set_keymap(keys);
 }
 
 FileBrowserFormAction::~FileBrowserFormAction() {}
 
-void FileBrowserFormAction::process_operation(Operation op,
+bool FileBrowserFormAction::process_operation(Operation op,
 	bool /* automatic */,
 	std::vector<std::string>* /* args */)
 {
@@ -53,65 +55,51 @@ void FileBrowserFormAction::process_operation(Operation op,
 		 * needs to be returned.
 		 */
 		LOG(Level::DEBUG, "FileBrowserFormAction: 'opening' item");
-		std::string focus = f->get_focus();
+		std::string focus = f.get_focus();
 		if (focus.length() > 0) {
 			if (focus == "files") {
-				std::string selection = f->get("listposname");
+				std::string selection = f.get("listposname");
 				char filetype = selection[0];
 				selection.erase(0, 1);
 				std::string filename(selection);
 				switch (filetype) {
 				case 'd': {
-					std::string fileswidth =
-						f->get("files:w");
-					unsigned int width =
-						utils::to_u(fileswidth);
-
-					FmtStrFormatter fmt;
-					fmt.register_fmt('N', PROGRAM_NAME);
-					fmt.register_fmt('V', utils::program_version());
-					fmt.register_fmt('f', filename);
-					f->set("head",
-						fmt.do_format(
-							cfg->get_configvalue(
-								"filebr"
-								"o"
-								"wser-"
-								"title-"
-								"forma"
-								"t"),
-							width));
 					int status = ::chdir(filename.c_str());
 					LOG(Level::DEBUG,
-						"FileBrowserFormAction:OP_"
-						"OPEN: "
-						"chdir(%s) = %i",
+						"FileBrowserFormAction:OP_OPEN: chdir(%s) = %i",
 						filename,
 						status);
-					f->set("listpos", "0");
+					files_list.set_position(0);
 					std::string fn = utils::getcwd();
-					fn.append(NEWSBEUTER_PATH_SEP);
+					update_title(fn);
+
+					if (fn.back() != NEWSBEUTER_PATH_SEP) {
+						fn.push_back(NEWSBEUTER_PATH_SEP);
+					}
+
 					std::string fnstr =
-						f->get("filenametext");
+						f.get("filenametext");
 					std::string::size_type base =
-						fnstr.find_first_of('/');
+						fnstr.find_last_of(NEWSBEUTER_PATH_SEP);
 					if (base == std::string::npos) {
 						fn.append(fnstr);
 					} else {
 						fn.append(fnstr,
-							base,
+							base + 1,
 							std::string::npos);
 					}
-					f->set("filenametext", fn);
+					f.set("filenametext", fn);
 					do_redraw = true;
 				}
 				break;
 				case '-': {
 					std::string fn = utils::getcwd();
-					fn.append(NEWSBEUTER_PATH_SEP);
+					if (fn.back() != NEWSBEUTER_PATH_SEP) {
+						fn.push_back(NEWSBEUTER_PATH_SEP);
+					}
 					fn.append(filename);
-					f->set("filenametext", fn);
-					f->set_focus("filename");
+					f.set("filenametext", fn);
+					f.set_focus("filename");
 				}
 				break;
 				default:
@@ -120,7 +108,7 @@ void FileBrowserFormAction::process_operation(Operation op,
 				}
 			} else {
 				bool do_pop = true;
-				std::string fn = f->get("filenametext");
+				std::string fn = f.get("filenametext");
 				struct stat sbuf;
 				/*
 				 * this check is very important, as people will
@@ -128,18 +116,16 @@ void FileBrowserFormAction::process_operation(Operation op,
 				 * files with no further warning...
 				 */
 				if (::stat(fn.c_str(), &sbuf) != -1) {
-					f->set_focus("files");
+					f.set_focus("files");
 					if (v->confirm(
 							strprintf::fmt(
-								_("Do you really "
-									"want to "
-									"overwrite `%s' "
+								_("Do you really want to overwrite `%s' "
 									"(y:Yes n:No)? "),
 								fn),
 							_("yn")) == *_("n")) {
 						do_pop = false;
 					}
-					f->set_focus("filenametext");
+					f.set_focus("filenametext");
 				}
 				if (do_pop) {
 					curs_set(0);
@@ -149,29 +135,94 @@ void FileBrowserFormAction::process_operation(Operation op,
 		}
 	}
 	break;
+	case OP_SWITCH_FOCUS: {
+		LOG(Level::DEBUG, "view::filebrowser: focusing different widget");
+		const std::string focus = f.get_focus();
+		if (focus == "files") {
+			f.set_focus("filename");
+		} else {
+			f.set_focus("files");
+		}
+		break;
+	}
+	case OP_SK_UP:
+		if (f.get_focus() == "files") {
+			files_list.move_up(cfg->get_configvalue_as_bool("wrap-scroll"));
+		} else {
+			f.set_focus("files");
+		}
+		break;
+	case OP_SK_DOWN:
+		if (f.get_focus() == "files") {
+			if (!files_list.move_down(cfg->get_configvalue_as_bool("wrap-scroll"))) {
+				f.set_focus("filename");
+			}
+		}
+		break;
+	case OP_SK_HOME:
+		if (f.get_focus() == "files") {
+			files_list.move_to_first();
+		} else {
+			f.set("filenametext_pos", "0");
+		}
+		break;
+	case OP_SK_END:
+		if (f.get_focus() == "files") {
+			files_list.move_to_last();
+		} else {
+			const std::size_t text_length = f.get("filenametext").length();
+			f.set("filenametext_pos", std::to_string(text_length));
+		}
+		break;
+	case OP_SK_PGUP:
+		if (f.get_focus() == "files") {
+			files_list.move_page_up(cfg->get_configvalue_as_bool("wrap-scroll"));
+		}
+		break;
+	case OP_SK_PGDOWN:
+		if (f.get_focus() == "files") {
+			files_list.move_page_down(cfg->get_configvalue_as_bool("wrap-scroll"));
+		}
+		break;
 	case OP_QUIT:
 		LOG(Level::DEBUG, "view::filebrowser: quitting");
 		curs_set(0);
 		v->pop_current_formaction();
-		f->set("filenametext", "");
+		f.set("filenametext", "");
 		break;
 	case OP_HARDQUIT:
 		LOG(Level::DEBUG, "view::filebrowser: hard quitting");
 		while (v->formaction_stack_size() > 0) {
 			v->pop_current_formaction();
 		}
-		f->set("filenametext", "");
+		f.set("filenametext", "");
 		break;
 	default:
 		break;
 	}
+	return true;
+}
+
+void FileBrowserFormAction::update_title(const std::string& working_directory)
+{
+	const unsigned int width = files_list.get_width();
+
+	FmtStrFormatter fmt;
+	fmt.register_fmt('N', PROGRAM_NAME);
+	fmt.register_fmt('V', utils::program_version());
+	fmt.register_fmt('f', working_directory);
+
+	const std::string title = fmt.do_format(
+			cfg->get_configvalue("filebrowser-title-format"), width);
+
+	f.set("head", title);
 }
 
 std::vector<std::string> get_sorted_filelist()
 {
 	std::vector<std::string> ret;
 
-	auto cwdtmp = utils::getcwd();
+	const std::string cwdtmp = utils::getcwd();
 
 	DIR* dirp = ::opendir(cwdtmp.c_str());
 	if (dirp) {
@@ -205,19 +256,17 @@ void FileBrowserFormAction::prepare()
 	if (do_redraw) {
 		std::vector<std::string> files = get_sorted_filelist();
 
-		std::string code = "{list";
+		ListFormatter listfmt;
 
 		for (std::string filename : files) {
-			code.append(add_file(filename));
+			add_file(listfmt, filename);
 		}
 
-		code.append("}");
-
-		f->modify("files", "replace_inner", code);
+		files_list.stfl_replace_lines(listfmt);
 		do_redraw = false;
 	}
 
-	std::string focus = f->get_focus();
+	std::string focus = f.get_focus();
 	if (focus == "files") {
 		curs_set(0);
 	} else {
@@ -229,7 +278,7 @@ void FileBrowserFormAction::init()
 {
 	set_keymap_hints();
 
-	f->set("fileprompt", _("File: "));
+	f.set("fileprompt", _("File: "));
 
 	if (dir == "") {
 		std::string save_path = cfg->get_configvalue("save-path");
@@ -244,19 +293,15 @@ void FileBrowserFormAction::init()
 	int status = ::chdir(dir.c_str());
 	LOG(Level::DEBUG, "view::filebrowser: chdir(%s) = %i", dir, status);
 
-	auto cwdtmp = utils::getcwd();
+	const std::string cwdtmp = utils::getcwd();
 
-	f->set("filenametext", default_filename);
+	f.set("filenametext", default_filename);
 
 	// Set position to 0 and back to ensure that the text is visible
-	f->run(-1);
-	f->set("filenametext_pos", std::to_string(default_filename.length()));
+	f.run(-1);
+	f.set("filenametext_pos", std::to_string(default_filename.length()));
 
-	f->set("head",
-		strprintf::fmt(_("%s %s - Save File - %s"),
-			PROGRAM_NAME,
-			utils::program_version(),
-			cwdtmp));
+	update_title(cwdtmp);
 }
 
 KeyMapHintEntry* FileBrowserFormAction::get_keymap_hint()
@@ -268,9 +313,9 @@ KeyMapHintEntry* FileBrowserFormAction::get_keymap_hint()
 	return hints;
 }
 
-std::string FileBrowserFormAction::add_file(std::string filename)
+void FileBrowserFormAction::add_file(ListFormatter& listfmt,
+	std::string filename)
 {
-	std::string retval;
 	struct stat sb;
 	if (::lstat(filename.c_str(), &sb) == 0) {
 		char ftype = get_filetype(sb.st_mode);
@@ -294,12 +339,9 @@ std::string FileBrowserFormAction::add_file(std::string filename)
 				group,
 				sizestr,
 				formattedfilename);
-		retval = strprintf::fmt("{listitem[%c%s] text:%s}",
-				ftype,
-				Stfl::quote(filename),
-				Stfl::quote(line));
+		std::string id = strprintf::fmt("%c%s", ftype, Stfl::quote(filename));
+		listfmt.add_line(utils::quote_for_stfl(line), id);
 	}
-	return retval;
 }
 
 std::string FileBrowserFormAction::get_formatted_filename(std::string filename,
