@@ -2,6 +2,7 @@
 
 #include <cassert>
 #include <cinttypes>
+#include <ncurses.h>
 
 #include "config.h"
 #include "configexception.h"
@@ -19,26 +20,26 @@ History FormAction::cmdlinehistory;
 FormAction::FormAction(View* vv, std::string formstr, ConfigContainer* cfg)
 	: v(vv)
 	, cfg(cfg)
-	, f(new Stfl::Form(formstr))
+	, f(formstr)
 	, do_redraw(true)
 	, finish_operation(OP_NIL)
 	, qna_history(nullptr)
 {
 	if (v) {
 		if (cfg->get_configvalue_as_bool("show-keymap-hint") == false) {
-			f->set("showhint", "0");
+			f.set("showhint", "0");
 		}
 		if (cfg->get_configvalue_as_bool("show-title-bar") == false) {
-			f->set("showtitle", "0");
+			f.set("showtitle", "0");
 		}
 		if (cfg->get_configvalue_as_bool("swap-title-and-hints") ==
 			true) {
-			std::string hints = f->dump("hints", "", 0);
-			std::string title = f->dump("title", "", 0);
-			f->modify("title", "replace", "label[swap-title]");
-			f->modify("hints", "replace", "label[swap-hints]");
-			f->modify("swap-title", "replace", hints);
-			f->modify("swap-hints", "replace", title);
+			std::string hints = f.dump("hints", "", 0);
+			std::string title = f.dump("title", "", 0);
+			f.modify("title", "replace", "label[swap-title]");
+			f.modify("hints", "replace", "label[swap-hints]");
+			f.modify("swap-title", "replace", hints);
+			f.modify("swap-hints", "replace", title);
 		}
 	}
 	valid_cmds.push_back("set");
@@ -46,21 +47,22 @@ FormAction::FormAction(View* vv, std::string formstr, ConfigContainer* cfg)
 	valid_cmds.push_back("source");
 	valid_cmds.push_back("dumpconfig");
 	valid_cmds.push_back("dumpform");
+	valid_cmds.push_back("exec");
 }
 
 void FormAction::set_keymap_hints()
 {
-	f->set("help", prepare_keymap_hint(this->get_keymap_hint()));
+	f.set("help", prepare_keymap_hint(this->get_keymap_hint()));
 }
 
 void FormAction::recalculate_form()
 {
-	f->run(-3);
+	f.run(-3);
 }
 
 FormAction::~FormAction() {}
 
-std::shared_ptr<Stfl::Form> FormAction::get_form()
+Stfl::Form& FormAction::get_form()
 {
 	return f;
 }
@@ -75,8 +77,12 @@ std::string FormAction::prepare_keymap_hint(KeyMapHintEntry* hints)
 	 */
 	std::string keymap_hint;
 	for (int i = 0; hints[i].op != OP_NIL; ++i) {
-		keymap_hint.append(
-			v->get_keys()->getkey(hints[i].op, this->id()));
+		std::string bound_keys = utils::join(v->get_keymap()->get_keys(hints[i].op,
+					this->id()), ",");
+		if (bound_keys.empty()) {
+			bound_keys = "<none>";
+		}
+		keymap_hint.append(bound_keys);
 		keymap_hint.append(":");
 		keymap_hint.append(hints[i].text);
 		keymap_hint.append(" ");
@@ -86,7 +92,7 @@ std::string FormAction::prepare_keymap_hint(KeyMapHintEntry* hints)
 
 std::string FormAction::get_value(const std::string& value)
 {
-	return f->get(value);
+	return f.get(value);
 }
 
 void FormAction::start_cmdline(std::string default_value)
@@ -97,7 +103,7 @@ void FormAction::start_cmdline(std::string default_value)
 	this->start_qna(qna, OP_INT_END_CMDLINE, &FormAction::cmdlinehistory);
 }
 
-void FormAction::process_op(Operation op,
+bool FormAction::process_op(Operation op,
 	bool automatic,
 	std::vector<std::string>* args)
 {
@@ -131,7 +137,7 @@ void FormAction::process_op(Operation op,
 		}
 		break;
 	case OP_INT_CANCEL_QNA:
-		f->modify("lastline",
+		f.modify("lastline",
 			"replace",
 			"{hbox[lastline] .expand:0 {label[msglabel] .expand:h "
 			"text[msg]:\"\"}}");
@@ -141,15 +147,15 @@ void FormAction::process_op(Operation op,
 	case OP_INT_QNA_NEXTHIST:
 		if (qna_history) {
 			std::string entry = qna_history->next();
-			f->set("qna_value", entry);
-			f->set("qna_value_pos", std::to_string(entry.length()));
+			f.set("qna_value", entry);
+			f.set("qna_value_pos", std::to_string(entry.length()));
 		}
 		break;
 	case OP_INT_QNA_PREVHIST:
 		if (qna_history) {
 			std::string entry = qna_history->prev();
-			f->set("qna_value", entry);
-			f->set("qna_value_pos", std::to_string(entry.length()));
+			f.set("qna_value", entry);
+			f.set("qna_value_pos", std::to_string(entry.length()));
 		}
 		break;
 	case OP_INT_END_QUESTION:
@@ -157,7 +163,7 @@ void FormAction::process_op(Operation op,
 		 * An answer has been entered, we save the value, and ask the
 		 * next question.
 		 */
-		qna_responses.push_back(f->get("qna_value"));
+		qna_responses.push_back(f.get("qna_value"));
 		start_next_question();
 		break;
 	case OP_VIEWDIALOGS:
@@ -170,8 +176,9 @@ void FormAction::process_op(Operation op,
 		v->goto_prev_dialog();
 		break;
 	default:
-		this->process_operation(op, automatic, args);
+		return this->process_operation(op, automatic, args);
 	}
+	return true;
 }
 
 std::vector<std::string> FormAction::get_suggestions(
@@ -223,6 +230,19 @@ std::vector<std::string> FormAction::get_suggestions(
 							line);
 					}
 				}
+			} else if (tokens[0] == "exec") {
+				if (tokens.size() <= 2) {
+					const std::string start = (tokens.size() == 2) ? tokens[1] : "";
+					const std::vector<KeyMapDesc> descs = v->get_keymap()->get_keymap_descriptions(
+							this->id()
+						);
+					for (const KeyMapDesc& desc: descs) {
+						const std::string cmd = desc.cmd;
+						if (cmd.rfind(start, 0) == 0) {
+							result.push_back(std::string("exec ") + cmd);
+						}
+					}
+				}
 			}
 		}
 	}
@@ -315,6 +335,17 @@ void FormAction::handle_cmdline(const std::string& cmdline)
 			}
 		} else if (cmd == "dumpform") {
 			v->dump_current_form();
+		} else if (cmd == "exec") {
+			if (tokens.size() != 1) {
+				v->show_error(_("usage: exec <operation>"));
+			} else {
+				const auto op = v->get_keymap()->get_opcode(tokens[0]);
+				if (op != OP_NIL) {
+					process_op(op);
+				} else {
+					v->show_error(_("Operation not found"));
+				}
+			}
 		} else {
 			v->show_error(strprintf::fmt(
 					_("Not a command: %s"), cmdline));
@@ -385,7 +416,7 @@ void FormAction::finished_qna(Operation op)
 	}
 	break;
 	case OP_INT_END_CMDLINE: {
-		f->set_focus("feeds");
+		f.set_focus("feeds");
 		std::string cmdline = qna_responses[0];
 		FormAction::cmdlinehistory.add_line(cmdline);
 		LOG(Level::DEBUG, "FormAction: commandline = `%s'", cmdline);
@@ -421,7 +452,7 @@ void FormAction::start_bookmark_qna(const std::string& default_title,
 		new_title = utils::make_title(default_url);
 		prompts.push_back(QnaPair(_("Title: "), new_title));
 	} else {
-		prompts.push_back(QnaPair(_("Title: "), default_title));
+		prompts.push_back(QnaPair(_("Title: "), utils::utf8_to_locale(default_title)));
 	}
 	prompts.push_back(QnaPair(_("Description: "), default_desc));
 	prompts.push_back(QnaPair(_("Feed title: "), default_feed_title));
@@ -433,7 +464,7 @@ void FormAction::start_bookmark_qna(const std::string& default_title,
 					default_url); // try to make the title from url
 		} else {
 			// assignment just to make the call to bookmark() below easier
-			new_title = default_title;
+			new_title = utils::utf8_to_locale(default_title);
 		}
 
 		// if url or title is missing, abort autopilot and ask user
@@ -480,12 +511,12 @@ void FormAction::start_next_question()
 		replacestr.append(Stfl::quote(qna_prompts[0].second));
 		replacestr.append(" pos[qna_value_pos]:0");
 		replacestr.append("}}");
-		f->modify("lastline", "replace", replacestr);
-		f->set_focus("qnainput");
+		f.modify("lastline", "replace", replacestr);
+		f.set_focus("qnainput");
 
 		// Set position to 0 and back to ensure that the text is visible
-		f->run(-1);
-		f->set("qna_value_pos",
+		f.run(-1);
+		f.set("qna_value_pos",
 			std::to_string(qna_prompts[0].second.length()));
 
 		qna_prompts.erase(qna_prompts.begin());
@@ -495,7 +526,7 @@ void FormAction::start_next_question()
 		 * usual label, and signal the end of the "Q&A" to the
 		 * finished_qna() method.
 		 */
-		f->modify("lastline",
+		f.modify("lastline",
 			"replace",
 			"{hbox[lastline] .expand:0 {label[msglabel] .expand:h "
 			"text[msg]:\"\"}}");
@@ -544,10 +575,10 @@ std::string FormAction::bookmark(const std::string& url,
 			v->pop_current_formaction();
 			return "";
 		} else {
-			char* my_argv[4];
-			my_argv[0] = const_cast<char*>("/bin/sh");
-			my_argv[1] = const_cast<char*>("-c");
-			my_argv[2] = const_cast<char*>(cmdline.c_str());
+			const char* my_argv[4];
+			my_argv[0] = "/bin/sh";
+			my_argv[1] = "-c";
+			my_argv[2] = cmdline.c_str();
 			my_argv[3] = nullptr;
 			return utils::run_program(my_argv, "");
 		}
