@@ -5,6 +5,7 @@
 #include <cstdio>
 #include <langinfo.h>
 #include <sstream>
+#include <string>
 #include <sys/stat.h>
 
 #include "config.h"
@@ -24,8 +25,9 @@ namespace newsboat {
 ItemListFormAction::ItemListFormAction(View* vv,
 	std::string formstr,
 	Cache* cc,
-	FilterContainer* f,
-	ConfigContainer* cfg)
+	FilterContainer& f,
+	ConfigContainer* cfg,
+	RegexManager& r)
 	: ListFormAction(vv, formstr, cfg)
 	, pos(0)
 	, apply_filter(false)
@@ -33,21 +35,22 @@ ItemListFormAction::ItemListFormAction(View* vv,
 	, search_dummy_feed(new RssFeed(cc))
 	, set_filterpos(false)
 	, filterpos(0)
-	, rxman(0)
+	, rxman(r)
 	, old_width(0)
 	, old_itempos(-1)
-	, old_sort_strategy({ArtSortMethod::TITLE, SortDirection::DESC})
-, invalidated(false)
-, invalidation_mode(InvalidationMode::COMPLETE)
-, rsscache(cc)
-, filters(f)
+	, invalidation_mode(InvalidationMode::NONE)
+	, listfmt(&rxman, "articlelist")
+	, rsscache(cc)
+	, filters(f)
+	, items_list("items", FormAction::f)
 {
 	search_dummy_feed->set_search_feed(true);
+	register_format_styles();
 }
 
 ItemListFormAction::~ItemListFormAction() {}
 
-void ItemListFormAction::process_operation(Operation op,
+bool ItemListFormAction::process_operation(Operation op,
 	bool automatic,
 	std::vector<std::string>* args)
 {
@@ -60,15 +63,12 @@ void ItemListFormAction::process_operation(Operation op,
 	 *   - if an item was selected, then fetch it and do something with it
 	 */
 
-	std::string itemposname = f->get("itempos");
-	unsigned int itempos = utils::to_u(itemposname);
+	const unsigned int itempos = items_list.get_position();
 
 	switch (op) {
 	case OP_OPEN: {
-		LOG(Level::INFO,
-			"ItemListFormAction: opening item at pos `%s'",
-			itemposname);
-		if (itemposname.length() > 0 && visible_items.size() != 0) {
+		LOG(Level::INFO, "ItemListFormAction: opening item at pos `%u'", itempos);
+		if (!visible_items.empty()) {
 			// no need to mark item as read, the itemview already do
 			// that
 			old_itempos = itempos;
@@ -82,9 +82,27 @@ void ItemListFormAction::process_operation(Operation op,
 		}
 	}
 	break;
+	case OP_SK_UP:
+		items_list.move_up(cfg->get_configvalue_as_bool("wrap-scroll"));
+		break;
+	case OP_SK_DOWN:
+		items_list.move_down(cfg->get_configvalue_as_bool("wrap-scroll"));
+		break;
+	case OP_SK_HOME:
+		items_list.move_to_first();
+		break;
+	case OP_SK_END:
+		items_list.move_to_last();
+		break;
+	case OP_SK_PGUP:
+		items_list.move_page_up(cfg->get_configvalue_as_bool("wrap-scroll"));
+		break;
+	case OP_SK_PGDOWN:
+		items_list.move_page_down(cfg->get_configvalue_as_bool("wrap-scroll"));
+		break;
 	case OP_DELETE: {
 		ScopeMeasure m1("OP_DELETE");
-		if (itemposname.length() > 0 && visible_items.size() != 0) {
+		if (!visible_items.empty()) {
 			// mark as read
 			v->get_ctrl()->mark_article_read(
 				visible_items[itempos].first->guid(), true);
@@ -95,9 +113,9 @@ void ItemListFormAction::process_operation(Operation op,
 			rsscache->mark_item_deleted(
 				visible_items[itempos].first->guid(),
 				visible_items[itempos].first->deleted());
-			if (itempos < visible_items.size() - 1)
-				f->set("itempos",
-					strprintf::fmt("%u", itempos + 1));
+			if (itempos < visible_items.size() - 1) {
+				items_list.set_position(itempos + 1);
+			}
 			invalidate(itempos);
 		} else {
 			v->show_error(
@@ -130,10 +148,8 @@ void ItemListFormAction::process_operation(Operation op,
 	}
 	break;
 	case OP_OPENBROWSER_AND_MARK: {
-		LOG(Level::INFO,
-			"ItemListFormAction: opening item at pos `%s'",
-			itemposname);
-		if (itemposname.length() > 0 && visible_items.size() != 0) {
+		LOG(Level::INFO, "ItemListFormAction: opening item at pos `%u'", itempos);
+		if (!visible_items.empty()) {
 			if (itempos < visible_items.size()) {
 				auto link = visible_items[itempos].first->link();
 				if (int err = v->open_in_browser(link)) {
@@ -149,9 +165,7 @@ void ItemListFormAction::process_operation(Operation op,
 						"next-unread")) {
 					if (itempos <
 						visible_items.size() - 1) {
-						f->set("itempos",
-							strprintf::fmt("%u",
-								itempos + 1));
+						items_list.set_position(itempos + 1);
 					}
 				} else {
 					process_operation(OP_NEXTUNREAD);
@@ -165,15 +179,13 @@ void ItemListFormAction::process_operation(Operation op,
 	}
 	break;
 	case OP_OPENINBROWSER: {
-		LOG(Level::INFO,
-			"ItemListFormAction: opening item at pos `%s'",
-			itemposname);
-		if (itemposname.length() > 0 && visible_items.size() != 0) {
+		LOG(Level::INFO, "ItemListFormAction: opening item at pos `%u'", itempos);
+		if (!visible_items.empty()) {
 			if (itempos < visible_items.size()) {
 				auto link = visible_items[itempos].first->link();
 				if (int err = v->open_in_browser(link)) {
 					v->show_error(strprintf::fmt(_("Browser returned error code %i"), err));
-					break;
+					return false;
 				}
 				invalidate(itempos);
 			}
@@ -191,7 +203,7 @@ void ItemListFormAction::process_operation(Operation op,
 				"browser");
 			if (int err = open_unread_items_in_browser(feed, false)) {
 				v->show_error(strprintf::fmt(_("Browser returned error code %i"), err));
-				break;
+				return false;
 			}
 		}
 	}
@@ -204,17 +216,15 @@ void ItemListFormAction::process_operation(Operation op,
 				"browser and marking read");
 			if (int err = open_unread_items_in_browser(feed, true)) {
 				v->show_error(strprintf::fmt(_("Browser returned error code %i"), err));
-				break;
+				return false;
 			}
 			invalidate_everything();
 		}
 	}
 	break;
 	case OP_TOGGLEITEMREAD: {
-		LOG(Level::INFO,
-			"ItemListFormAction: toggling item read at pos `%s'",
-			itemposname);
-		if (itemposname.length() > 0 && visible_items.size() != 0) {
+		LOG(Level::INFO, "ItemListFormAction: toggling item read at pos `%u'", itempos);
+		if (!visible_items.empty()) {
 			v->set_status(_("Toggling read flag for article..."));
 			try {
 				if (automatic && args->size() > 0) {
@@ -262,10 +272,9 @@ void ItemListFormAction::process_operation(Operation op,
 			}
 			if (!cfg->get_configvalue_as_bool(
 					"toggleitemread-jumps-to-next-unread")) {
-				if (itempos < visible_items.size() - 1)
-					f->set("itempos",
-						strprintf::fmt(
-							"%u", itempos + 1));
+				if (itempos < visible_items.size() - 1) {
+					items_list.set_position(itempos + 1);
+				}
 			} else {
 				process_operation(OP_NEXTUNREAD);
 			}
@@ -274,7 +283,7 @@ void ItemListFormAction::process_operation(Operation op,
 	}
 	break;
 	case OP_SHOWURLS:
-		if (itemposname.length() > 0 && visible_items.size() != 0) {
+		if (!visible_items.empty()) {
 			if (itempos < visible_items.size()) {
 				std::string urlviewer = cfg->get_configvalue(
 						"external-url-viewer");
@@ -294,8 +303,7 @@ void ItemListFormAction::process_operation(Operation op,
 						: visible_items[itempos]
 						.first->feedurl();
 					rnd.render(
-						visible_items[itempos]
-						.first->description(),
+						utils::utf8_to_locale(visible_items[itempos].first->description()),
 						lines,
 						links,
 						baseurl);
@@ -317,19 +325,16 @@ void ItemListFormAction::process_operation(Operation op,
 		}
 		break;
 	case OP_BOOKMARK: {
-		LOG(Level::INFO,
-			"ItemListFormAction: bookmarking item at pos `%s'",
-			itemposname);
-		if (itemposname.length() > 0 && visible_items.size() != 0) {
+		LOG(Level::INFO, "ItemListFormAction: bookmarking item at pos `%u'", itempos);
+		if (!visible_items.empty()) {
 			if (itempos < visible_items.size()) {
 				if (automatic) {
 					qna_responses.clear();
 					qna_responses.push_back(
 						visible_items[itempos]
 						.first->link());
-					qna_responses.push_back(
-						visible_items[itempos]
-						.first->title());
+					qna_responses.push_back(utils::utf8_to_locale(
+							visible_items[itempos].first->title()));
 					qna_responses.push_back(args->size() > 0
 						? (*args)[0]
 						: "");
@@ -337,10 +342,8 @@ void ItemListFormAction::process_operation(Operation op,
 					this->finished_qna(OP_INT_BM_END);
 				} else {
 					this->start_bookmark_qna(
-						visible_items[itempos]
-						.first->title(),
-						visible_items[itempos]
-						.first->link(),
+						visible_items[itempos].first->title(),
+						visible_items[itempos].first->link(),
 						"",
 						feed->title());
 				}
@@ -352,7 +355,7 @@ void ItemListFormAction::process_operation(Operation op,
 	}
 	break;
 	case OP_EDITFLAGS: {
-		if (itemposname.length() > 0 && visible_items.size() != 0) {
+		if (!visible_items.empty()) {
 			if (itempos < visible_items.size()) {
 				if (automatic) {
 					if (args->size() > 0) {
@@ -378,20 +381,17 @@ void ItemListFormAction::process_operation(Operation op,
 	}
 	break;
 	case OP_SAVE: {
-		LOG(Level::INFO,
-			"ItemListFormAction: saving item at pos `%s'",
-			itemposname);
-		if (itemposname.length() > 0 && visible_items.size() != 0) {
+		LOG(Level::INFO, "ItemListFormAction: saving item at pos `%u'", itempos);
+		if (!visible_items.empty()) {
 			std::string filename;
 			if (automatic) {
 				if (args->size() > 0) {
 					filename = (*args)[0];
 				}
 			} else {
-				filename = v->run_filebrowser(
-						v->get_filename_suggestion(
-							visible_items[itempos]
-							.first->title()));
+				const auto title = utils::utf8_to_locale(visible_items[itempos].first->title());
+				const auto suggestion = v->get_filename_suggestion(title);
+				filename = v->run_filebrowser(suggestion);
 			}
 			save_article(filename, visible_items[itempos].first);
 		} else {
@@ -527,14 +527,13 @@ void ItemListFormAction::process_operation(Operation op,
 					LOG(Level::DEBUG,
 						"ItemListFormAction:: "
 						"reset itempos to last");
-					f->set("itempos",
-						std::to_string(visible_items.size() - 1));
+					items_list.set_position(visible_items.size() - 1);
 				}
 				if (sortorder == "date-asc") {
 					LOG(Level::DEBUG,
 						"ItemListFormAction:: "
 						"reset itempos to first");
-					f->set("itempos", "0");
+					items_list.set_position(0);
 				}
 			}
 			invalidate_everything();
@@ -549,8 +548,7 @@ void ItemListFormAction::process_operation(Operation op,
 		LOG(Level::INFO,
 			"ItemListFormAction: marking all above as read");
 		v->set_status(_("Marking all above as read..."));
-		if (itemposname.length() > 0 &&
-			itempos < visible_items.size()) {
+		if (itempos < visible_items.size()) {
 			for (unsigned int i = 0; i < itempos; ++i) {
 				if (visible_items[i].first->unread()) {
 					visible_items[i].first->set_unread(
@@ -562,7 +560,7 @@ void ItemListFormAction::process_operation(Operation op,
 			}
 			if (!cfg->get_configvalue_as_bool(
 					"show-read-articles")) {
-				f->set("itempos", "0");
+				items_list.set_position(0);
 			}
 			invalidate_everything();
 		}
@@ -617,7 +615,7 @@ void ItemListFormAction::process_operation(Operation op,
 		v->get_ctrl()->edit_urls_file();
 		break;
 	case OP_SELECTFILTER:
-		if (filters->size() > 0) {
+		if (filters.size() > 0) {
 			std::string newfilter;
 			if (automatic) {
 				if (args->size() > 0) {
@@ -625,7 +623,7 @@ void ItemListFormAction::process_operation(Operation op,
 				}
 			} else {
 				newfilter = v->select_filter(
-						filters->get_filters());
+						filters.get_filters());
 				LOG(Level::DEBUG,
 					"ItemListFormAction::run: newfilters "
 					"= %s",
@@ -767,6 +765,7 @@ void ItemListFormAction::process_operation(Operation op,
 	} else if (quit) {
 		v->pop_current_formaction();
 	}
+	return true;
 }
 
 void ItemListFormAction::finished_qna(Operation op)
@@ -787,9 +786,8 @@ void ItemListFormAction::finished_qna(Operation op)
 		break;
 
 	case OP_PIPE_TO: {
-		std::string itemposname = f->get("itempos");
-		unsigned int itempos = utils::to_u(itemposname);
-		if (itemposname.length() > 0) {
+		if (!visible_items.empty()) {
+			unsigned int itempos = items_list.get_position();
 			std::string cmd = qna_responses[0];
 			std::ostringstream ostr;
 			v->get_ctrl()->write_item(
@@ -832,13 +830,12 @@ void ItemListFormAction::qna_end_setfilter()
 
 void ItemListFormAction::qna_end_editflags()
 {
-	std::string itemposname = f->get("itempos");
-	if (itemposname.length() == 0) {
+	if (visible_items.empty()) {
 		v->show_error(_("No item selected!")); // should not happen
 		return;
 	}
 
-	unsigned int itempos = utils::to_u(itemposname);
+	const unsigned int itempos = items_list.get_position();
 	if (itempos < visible_items.size()) {
 		visible_items[itempos].first->set_flags(qna_responses[0]);
 		v->get_ctrl()->update_flags(visible_items[itempos].first);
@@ -894,7 +891,7 @@ void ItemListFormAction::qna_start_search()
 
 void ItemListFormAction::do_update_visible_items()
 {
-	if (!(invalidated && invalidation_mode == InvalidationMode::COMPLETE)) {
+	if (invalidation_mode != InvalidationMode::COMPLETE) {
 		return;
 	}
 
@@ -933,7 +930,7 @@ void ItemListFormAction::prepare()
 	std::lock_guard<std::mutex> mtx(redraw_mtx);
 
 	const auto sort_strategy = cfg->get_article_sort_strategy();
-	if (sort_strategy != old_sort_strategy) {
+	if (!old_sort_strategy || sort_strategy != *old_sort_strategy) {
 		feed->sort(sort_strategy);
 		old_sort_strategy = sort_strategy;
 		invalidate_everything();
@@ -948,9 +945,8 @@ void ItemListFormAction::prepare()
 	}
 
 	if (cfg->get_configvalue_as_bool("mark-as-read-on-hover")) {
-		std::string itemposname = f->get("itempos");
-		if (itemposname.length() > 0) {
-			unsigned int itempos = utils::to_u(itemposname);
+		if (!visible_items.empty()) {
+			const unsigned int itempos = items_list.get_position();
 			if (visible_items[itempos].first->unread()) {
 				visible_items[itempos].first->set_unread(false);
 				v->get_ctrl()->mark_article_read(
@@ -961,14 +957,14 @@ void ItemListFormAction::prepare()
 		}
 	}
 
-	unsigned int width = utils::to_u(f->get("items:w"));
+	const unsigned int width = items_list.get_width();
 
 	if (old_width != width) {
 		invalidate_everything();
 		old_width = width;
 	}
 
-	if (!invalidated) {
+	if (invalidation_mode == InvalidationMode::NONE) {
 		return;
 	}
 
@@ -985,7 +981,7 @@ void ItemListFormAction::prepare()
 					width,
 					itemlist_format,
 					datetime_format);
-			listfmt.add_line(line, item.second);
+			listfmt.add_line(line, std::to_string(item.second));
 		}
 		break;
 
@@ -996,17 +992,17 @@ void ItemListFormAction::prepare()
 					width,
 					itemlist_format,
 					datetime_format);
-			listfmt.set_line(itempos, line, item.second);
+			listfmt.set_line(itempos, line, std::to_string(item.second));
 		}
 		break;
+	case InvalidationMode::NONE:
+		break;
 	}
 
-	f->modify("items",
-		"replace_inner",
-		listfmt.format_list(rxman, "articlelist"));
+	items_list.stfl_replace_lines(listfmt);
 
 	invalidated_itempos.clear();
-	invalidated = false;
+	invalidation_mode = InvalidationMode::NONE;
 
 	set_head(feed->title(),
 		feed->unread_item_count(),
@@ -1024,36 +1020,36 @@ std::string ItemListFormAction::item2formatted_line(const ItemPtrPosPair& item,
 	FmtStrFormatter fmt;
 	fmt.register_fmt('i', strprintf::fmt("%u", item.second + 1));
 	fmt.register_fmt('f', gen_flags(item.first));
+	fmt.register_fmt('n', item.first->unread() ? "N" : " ");
+	fmt.register_fmt('d', item.first->deleted() ? "D" : " ");
+	fmt.register_fmt('F', item.first->flags());
 	fmt.register_fmt('D',
 		utils::mt_strf_localtime(
 			datetime_format,
 			item.first->pubDate_timestamp()));
 	if (feed->rssurl() != item.first->feedurl() &&
 		item.first->get_feedptr() != nullptr) {
-		auto feedtitle = utils::quote_for_stfl(
-				item.first->get_feedptr()->title());
+		auto feedtitle = item.first->get_feedptr()->title();
 		utils::remove_soft_hyphens(feedtitle);
 		fmt.register_fmt('T', feedtitle);
 	}
 
-	auto itemtitle = utils::quote_for_stfl(item.first->title());
+	auto itemtitle = utils::utf8_to_locale(item.first->title());
 	utils::remove_soft_hyphens(itemtitle);
 	fmt.register_fmt('t', itemtitle);
 
-	auto itemauthor = utils::quote_for_stfl(item.first->author());
+	auto itemauthor = utils::utf8_to_locale(item.first->author());
 	utils::remove_soft_hyphens(itemauthor);
 	fmt.register_fmt('a', itemauthor);
 
 	fmt.register_fmt('L', item.first->length());
 
 	auto formattedLine = fmt.do_format(itemlist_format, width);
+	formattedLine = utils::quote_for_stfl(formattedLine);
 
-	if (rxman) {
-		int id;
-		if ((id = rxman->article_matches(item.first.get())) != -1) {
-			formattedLine =
-				strprintf::fmt("<%d>%s</>", id, formattedLine);
-		}
+	const int id = rxman.article_matches(item.first.get());
+	if (id != -1) {
+		formattedLine = strprintf::fmt("<%d>%s</>", id, formattedLine);
 	}
 
 	if (item.first->unread()) {
@@ -1065,15 +1061,15 @@ std::string ItemListFormAction::item2formatted_line(const ItemPtrPosPair& item,
 
 void ItemListFormAction::init()
 {
-	f->set("itempos", "0");
-	f->set("msg", "");
+	items_list.set_position(0);
+	f.set("msg", "");
 	set_keymap_hints();
 	invalidate_everything();
 	do_update_visible_items();
 	if (cfg->get_configvalue_as_bool("goto-first-unread")) {
 		jump_to_next_unread_item(true);
 	}
-	f->run(-3); // FRUN - compute all widget dimensions
+	f.run(-3); // FRUN - compute all widget dimensions
 }
 
 void ItemListFormAction::set_head(const std::string& s,
@@ -1100,21 +1096,24 @@ void ItemListFormAction::set_head(const std::string& s,
 
 	fmt.register_fmt('U', utils::censor_url(url));
 
+	fmt.register_fmt('F', apply_filter ? matcher.get_expression() : "");
+
+	const unsigned int width = utils::to_u(f.get("title:w"));
 	if (!show_searchresult) {
 		title = fmt.do_format(
-				cfg->get_configvalue("articlelist-title-format"));
+				cfg->get_configvalue("articlelist-title-format"),
+				width);
 	} else {
 		title = fmt.do_format(
-				cfg->get_configvalue("searchresult-title-format"));
+				cfg->get_configvalue("searchresult-title-format"),
+				width);
 	}
-	f->set("head", title);
+	f.set("head", title);
 }
 
 bool ItemListFormAction::jump_to_previous_unread_item(bool start_with_last)
 {
-	int itempos;
-	std::istringstream is(f->get("itempos"));
-	is >> itempos;
+	const int itempos =  items_list.get_position();
 	for (int i = (start_with_last ? itempos : (itempos - 1)); i >= 0; --i) {
 		LOG(Level::DEBUG,
 			"ItemListFormAction::jump_to_previous_unread_item: "
@@ -1122,13 +1121,13 @@ bool ItemListFormAction::jump_to_previous_unread_item(bool start_with_last)
 			i,
 			visible_items[i].first->unread() ? "true" : "false");
 		if (visible_items[i].first->unread()) {
-			f->set("itempos", std::to_string(i));
+			items_list.set_position(i);
 			return true;
 		}
 	}
 	for (int i = visible_items.size() - 1; i >= itempos; --i) {
 		if (visible_items[i].first->unread()) {
-			f->set("itempos", std::to_string(i));
+			items_list.set_position(i);
 			return true;
 		}
 	}
@@ -1149,7 +1148,7 @@ bool ItemListFormAction::jump_to_random_unread_item()
 			unsigned int pos =
 				utils::get_random_value(visible_items.size());
 			if (visible_items[pos].first->unread()) {
-				f->set("itempos", std::to_string(pos));
+				items_list.set_position(pos);
 				break;
 			}
 		}
@@ -1159,7 +1158,7 @@ bool ItemListFormAction::jump_to_random_unread_item()
 
 bool ItemListFormAction::jump_to_next_unread_item(bool start_with_first)
 {
-	unsigned int itempos = utils::to_u(f->get("itempos"));
+	const unsigned int itempos = items_list.get_position();
 	LOG(Level::DEBUG,
 		"ItemListFormAction::jump_to_next_unread_item: itempos = %u "
 		"visible_items.size = %" PRIu64,
@@ -1172,7 +1171,7 @@ bool ItemListFormAction::jump_to_next_unread_item(bool start_with_first)
 			"ItemListFormAction::jump_to_next_unread_item: i = %u",
 			i);
 		if (visible_items[i].first->unread()) {
-			f->set("itempos", std::to_string(i));
+			items_list.set_position(i);
 			return true;
 		}
 	}
@@ -1182,7 +1181,7 @@ bool ItemListFormAction::jump_to_next_unread_item(bool start_with_first)
 			"ItemListFormAction::jump_to_next_unread_item: i = %u",
 			i);
 		if (visible_items[i].first->unread()) {
-			f->set("itempos", std::to_string(i));
+			items_list.set_position(i);
 			return true;
 		}
 	}
@@ -1191,9 +1190,7 @@ bool ItemListFormAction::jump_to_next_unread_item(bool start_with_first)
 
 bool ItemListFormAction::jump_to_previous_item(bool start_with_last)
 {
-	int itempos;
-	std::istringstream is(f->get("itempos"));
-	is >> itempos;
+	const unsigned int itempos = items_list.get_position();
 
 	int i = (start_with_last ? itempos : (itempos - 1));
 	if (i >= 0) {
@@ -1201,7 +1198,7 @@ bool ItemListFormAction::jump_to_previous_item(bool start_with_last)
 			"ItemListFormAction::jump_to_previous_item: "
 			"visible_items[%u]",
 			i);
-		f->set("itempos", std::to_string(i));
+		items_list.set_position(i);
 		return true;
 	}
 	return false;
@@ -1209,7 +1206,7 @@ bool ItemListFormAction::jump_to_previous_item(bool start_with_last)
 
 bool ItemListFormAction::jump_to_next_item(bool start_with_first)
 {
-	unsigned int itempos = utils::to_u(f->get("itempos"));
+	const unsigned int itempos = items_list.get_position();
 	LOG(Level::DEBUG,
 		"ItemListFormAction::jump_to_next_item: itempos = %" PRIu64
 		" visible_items.size = %" PRIu64,
@@ -1220,7 +1217,7 @@ bool ItemListFormAction::jump_to_next_item(bool start_with_first)
 		LOG(Level::DEBUG,
 			"ItemListFormAction::jump_to_next_item: i = %u",
 			i);
-		f->set("itempos", std::to_string(i));
+		items_list.set_position(i);
 		return true;
 	}
 	return false;
@@ -1228,7 +1225,7 @@ bool ItemListFormAction::jump_to_next_item(bool start_with_first)
 
 std::string ItemListFormAction::get_guid()
 {
-	unsigned int itempos = utils::to_u(f->get("itempos"));
+	const unsigned int itempos = items_list.get_position();
 	return visible_items[itempos].first->guid();
 }
 
@@ -1255,7 +1252,7 @@ void ItemListFormAction::handle_cmdline_num(unsigned int idx)
 		if (i == -1) {
 			v->show_error(_("Position not visible!"));
 		} else {
-			f->set("itempos", std::to_string(i));
+			items_list.set_position(i);
 		}
 	} else {
 		v->show_error(_("Invalid position!"));
@@ -1274,17 +1271,13 @@ void ItemListFormAction::handle_cmdline(const std::string& cmd)
 		}
 		if (tokens[0] == "save" && tokens.size() >= 2) {
 			std::string filename = utils::resolve_tilde(tokens[1]);
-			std::string itemposname = f->get("itempos");
+			const unsigned int itempos = items_list.get_position();
 			LOG(Level::INFO,
-				"ItemListFormAction::handle_cmdline: saving "
-				"item "
-				"at pos `%s' to `%s'",
-				itemposname,
+				"ItemListFormAction::handle_cmdline: saving item at pos `%u' to `%s'",
+				itempos,
 				filename);
-			if (itemposname.length() > 0) {
-				unsigned int itempos = utils::to_u(itemposname);
-				save_article(
-					filename, visible_items[itempos].first);
+			if (!visible_items.empty()) {
+				save_article(filename, visible_items[itempos].first);
 			} else {
 				v->show_error(_("Error: no item selected!"));
 			}
@@ -1309,8 +1302,7 @@ void ItemListFormAction::recalculate_form()
 	FormAction::recalculate_form();
 	invalidate_everything();
 
-	std::string itemposname = f->get("itempos");
-	unsigned int itempos = utils::to_u(itemposname);
+	const unsigned int itempos = items_list.get_position();
 
 	// If the old position was set and it is less than the itempos, use it
 	// for the feed's itempos Correct the problem when you open itemview and
@@ -1318,7 +1310,7 @@ void ItemListFormAction::recalculate_form()
 	// applies when "show-read-articles" is set to false
 	if ((old_itempos != -1) && itempos > (unsigned int)old_itempos &&
 		!cfg->get_configvalue_as_bool("show-read-articles")) {
-		f->set("itempos", strprintf::fmt("%u", old_itempos));
+		items_list.set_position(old_itempos);
 		old_itempos = -1; // Reset
 	}
 }
@@ -1343,32 +1335,22 @@ void ItemListFormAction::save_article(const std::string& filename,
 
 void ItemListFormAction::save_filterpos()
 {
-	unsigned int i = utils::to_u(f->get("itempos"));
+	const unsigned int i = items_list.get_position();
 	if (i < visible_items.size()) {
 		filterpos = visible_items[i].second;
 		set_filterpos = true;
 	}
 }
 
-void ItemListFormAction::set_regexmanager(RegexManager* r)
+void ItemListFormAction::register_format_styles()
 {
-	rxman = r;
-	std::vector<std::string>& attrs = r->get_attrs("articlelist");
-	unsigned int i = 0;
-	std::string attrstr;
-	for (const auto& attribute : attrs) {
-		attrstr.append(
-			strprintf::fmt("@style_%u_normal:%s ", i, attribute));
-		attrstr.append(
-			strprintf::fmt("@style_%u_focus:%s ", i, attribute));
-		i++;
-	}
-	std::string textview = strprintf::fmt(
+	const std::string attrstr = rxman.get_attrs_stfl_string("articlelist", true);
+	const std::string textview = strprintf::fmt(
 			"{list[items] .expand:vh style_normal[listnormal]: "
 			"style_focus[listfocus]:fg=yellow,bg=blue,attr=bold "
-			"pos_name[itemposname]: pos[itempos]:0 %s richtext:1}",
+			"pos[items_pos]:0 %s richtext:1}",
 			attrstr);
-	f->modify("items", "replace", textview);
+	items_list.stfl_replace_list(0, textview);
 }
 
 std::string ItemListFormAction::gen_flags(std::shared_ptr<RssItem> item)
@@ -1396,12 +1378,12 @@ void ItemListFormAction::prepare_set_filterpos()
 		unsigned int i = 0;
 		for (const auto& item : visible_items) {
 			if (item.second == filterpos) {
-				f->set("itempos", std::to_string(i));
+				items_list.set_position(i);
 				return;
 			}
 			i++;
 		}
-		f->set("itempos", "0");
+		items_list.set_position(0);
 	}
 }
 
@@ -1450,13 +1432,13 @@ void ItemListFormAction::handle_op_saveall()
 		return;
 	}
 
-	if (directory.back() != '/') {
-		directory = directory + "/";
+	if (directory.back() != NEWSBEUTER_PATH_SEP) {
+		directory.push_back(NEWSBEUTER_PATH_SEP);
 	}
 
 	if (visible_items.size() == 1) {
-		const std::string filename = v->get_filename_suggestion(
-				visible_items[0].first->title());
+		const std::string filename = v->get_filename_suggestion( utils::utf8_to_locale(
+					visible_items[0].first->title()));
 		const std::string fpath = directory + filename;
 
 		struct stat sbuf;
@@ -1493,8 +1475,8 @@ void ItemListFormAction::handle_op_saveall()
 	} else {
 		std::vector<std::string> filenames;
 		for (const auto& item : visible_items) {
-			filenames.emplace_back(
-				v->get_filename_suggestion(item.first->title()));
+			filenames.emplace_back( utils::utf8_to_locale(v->get_filename_suggestion(
+						item.first->title())));
 		}
 
 		const auto unique_filenames = std::set<std::string>(
