@@ -9,6 +9,7 @@
 #include "dbexception.h"
 #include "fmtstrformatter.h"
 #include "itemrenderer.h"
+#include "htmlrenderer.h"
 #include "logger.h"
 #include "rssfeed.h"
 #include "scopemeasure.h"
@@ -23,34 +24,37 @@ ItemViewFormAction::ItemViewFormAction(View* vv,
 	std::shared_ptr<ItemListFormAction> il,
 	std::string formstr,
 	Cache* cc,
-	ConfigContainer* cfg)
+	ConfigContainer* cfg,
+	RegexManager& r)
 	: FormAction(vv, formstr, cfg)
 	, show_source(false)
 	, quit(false)
-	, rxman(0)
+	, rxman(r)
 	, num_lines(0)
 	, itemlist(il)
 	, in_search(false)
 	, rsscache(cc)
+	, textview("article", FormAction::f)
 {
 	valid_cmds.push_back("save");
 	std::sort(valid_cmds.begin(), valid_cmds.end());
+	register_format_styles();
 }
 
 ItemViewFormAction::~ItemViewFormAction() {}
 
 void ItemViewFormAction::init()
 {
-	f->set("msg", "");
+	f.set("msg", "");
 	do_redraw = true;
 	quit = false;
 	links.clear();
 	num_lines = 0;
 	if (!cfg->get_configvalue_as_bool("display-article-progress")) {
-		f->set("percentwidth", "0");
+		f.set("percentwidth", "0");
 	} else {
 		const size_t min_field_width = 6;
-		f->set("percentwidth",
+		f.set("percentwidth",
 			std::to_string(
 		std::max({
 			min_field_width,
@@ -72,7 +76,7 @@ void ItemViewFormAction::update_head(const std::shared_ptr<RssItem>& item)
 	if (item->unread()) {
 		unread_item_count--;
 	}
-	set_head(item->title(),
+	set_head(utils::utf8_to_locale(item->title()),
 		feedtitle,
 		unread_item_count,
 		feed->total_item_count());
@@ -90,13 +94,12 @@ void ItemViewFormAction::prepare()
 		{
 			ScopeMeasure("itemview::prepare: rendering");
 			// XXX HACK: render once so that we get a proper widget width
-			f->run(-3);
+			f.run(-3);
 		}
 
 		update_head(item);
 
-		const std::string widthstr = f->get("article:w");
-		const unsigned int window_width = utils::to_u(widthstr, 0);
+		const unsigned int window_width = textview.get_width();
 
 		unsigned int text_width =
 			cfg->get_configvalue_as_int("text-width");
@@ -114,9 +117,17 @@ void ItemViewFormAction::prepare()
 					item,
 					text_width,
 					window_width,
-					rxman,
+					&rxman,
 					"article");
 		} else {
+			links.clear();
+			if (!item->enclosure_url().empty()) {
+				const auto link_type = utils::podcast_mime_to_link_type(item->enclosure_type());
+				if (link_type.has_value()) {
+					links.push_back(LinkPair(item->enclosure_url(), link_type.value()));
+				}
+			}
+
 			std::tie(formatted_text, num_lines) =
 				item_renderer::to_stfl_list(
 					// cfg can't be nullptr because that's a long-lived object
@@ -125,16 +136,16 @@ void ItemViewFormAction::prepare()
 					item,
 					text_width,
 					window_width,
-					rxman,
+					&rxman,
 					"article",
 					links);
 		}
 
-		f->modify("article", "replace_inner", formatted_text);
-		f->set("articleoffset", "0");
+		textview.stfl_replace_lines(num_lines, formatted_text);
+		f.set("article_offset", "0");
 
 		if (in_search) {
-			rxman->remove_last_regex("article");
+			rxman.remove_last_regex("article");
 			in_search = false;
 		}
 
@@ -142,7 +153,7 @@ void ItemViewFormAction::prepare()
 	}
 }
 
-void ItemViewFormAction::process_operation(Operation op,
+bool ItemViewFormAction::process_operation(Operation op,
 	bool automatic,
 	std::vector<std::string>* args)
 {
@@ -168,6 +179,24 @@ void ItemViewFormAction::process_operation(Operation op,
 	}
 
 	switch (op) {
+	case OP_SK_UP:
+		textview.scroll_up();
+		break;
+	case OP_SK_DOWN:
+		textview.scroll_down();
+		break;
+	case OP_SK_HOME:
+		textview.scroll_to_top();
+		break;
+	case OP_SK_END:
+		textview.scroll_to_bottom();
+		break;
+	case OP_SK_PGUP:
+		textview.scroll_page_up();
+		break;
+	case OP_SK_PGDOWN:
+		textview.scroll_page_down();
+		break;
 	case OP_TOGGLESOURCEVIEW:
 		LOG(Level::INFO, "ItemViewFormAction::process_operation: toggling source view");
 		show_source = !show_source;
@@ -194,8 +223,8 @@ void ItemViewFormAction::process_operation(Operation op,
 				filename = (*args)[0];
 			}
 		} else {
-			filename = v->run_filebrowser(
-					v->get_filename_suggestion(item->title()));
+			filename = v->run_filebrowser( utils::utf8_to_locale(v->get_filename_suggestion(
+							item->title())));
 		}
 		if (filename == "") {
 			v->show_error(_("Aborted saving."));
@@ -214,22 +243,27 @@ void ItemViewFormAction::process_operation(Operation op,
 	}
 	break;
 	case OP_OPENINBROWSER:
+	case OP_OPENBROWSER_AND_MARK:
 		LOG(Level::INFO, "ItemViewFormAction::process_operation: starting browser");
 		v->set_status(_("Starting browser..."));
-		v->open_in_browser(item->link());
+		if (int err = v->open_in_browser(item->link())) {
+			v->show_error(strprintf::fmt(_("Browser returned error code %i"), err));
+			return false;
+		}
+
 		v->set_status("");
 		break;
 	case OP_BOOKMARK:
 		if (automatic) {
 			qna_responses.clear();
 			qna_responses.push_back(item->link());
-			qna_responses.push_back(item->title());
+			qna_responses.push_back(utils::utf8_to_locale(item->title()));
 			qna_responses.push_back(
 				args->size() > 0 ? (*args)[0] : "");
 			qna_responses.push_back(feed->title());
 		} else {
-			this->start_bookmark_qna(
-				item->title(), item->link(), "", feed->title());
+			this->start_bookmark_qna(utils::utf8_to_locale(item->title()), item->link(), "",
+				feed->title());
 		}
 		break;
 	case OP_SEARCH: {
@@ -375,25 +409,30 @@ void ItemViewFormAction::process_operation(Operation op,
 	case OP_HELP:
 		v->push_help();
 		break;
-	case OP_1:
-	case OP_2:
-	case OP_3:
-	case OP_4:
-	case OP_5:
-	case OP_6:
-	case OP_7:
-	case OP_8:
-	case OP_9:
-	case OP_0: {
-		unsigned int idx = op - OP_1;
+	case OP_OPEN_URL_1:
+	case OP_OPEN_URL_2:
+	case OP_OPEN_URL_3:
+	case OP_OPEN_URL_4:
+	case OP_OPEN_URL_5:
+	case OP_OPEN_URL_6:
+	case OP_OPEN_URL_7:
+	case OP_OPEN_URL_8:
+	case OP_OPEN_URL_9:
+	case OP_OPEN_URL_10: {
+		unsigned int idx = op - OP_OPEN_URL_1;
 		LOG(Level::DEBUG,
-			"ItemViewFormAction::process_operation: OP_1 = %d op = %d idx = %u",
-			OP_1,
+			"ItemViewFormAction::process_operation: OP_OPEN_URL_1 = %d op = %d idx = %u",
+			OP_OPEN_URL_1,
 			op,
 			idx);
 		if (idx < links.size()) {
 			v->set_status(_("Starting browser..."));
-			v->open_in_browser(links[idx].first);
+
+			if (int err = v->open_in_browser(links[idx].first)) {
+				v->show_error(strprintf::fmt(_("Browser returned error code %i"), err));
+				return false;
+			}
+
 			v->set_status("");
 		}
 	}
@@ -423,6 +462,10 @@ void ItemViewFormAction::process_operation(Operation op,
 	} else if (quit) {
 		v->pop_current_formaction();
 	}
+
+	update_percent();
+
+	return true;
 }
 
 KeyMapHintEntry* ItemViewFormAction::get_keymap_hint()
@@ -458,10 +501,9 @@ void ItemViewFormAction::set_head(const std::string& s,
 	fmt.register_fmt('u', std::to_string(unread));
 	fmt.register_fmt('t', std::to_string(total));
 
-	std::string listwidth = f->get("article:w");
-	unsigned int width = utils::to_u(listwidth);
+	const unsigned int width = textview.get_width();
 
-	f->set("head",
+	f.set("head",
 		fmt.do_format(
 			cfg->get_configvalue("itemview-title-format"), width));
 }
@@ -540,33 +582,25 @@ void ItemViewFormAction::finished_qna(Operation op)
 	}
 }
 
-void ItemViewFormAction::set_regexmanager(RegexManager* r)
+void ItemViewFormAction::register_format_styles()
 {
-	rxman = r;
-	std::vector<std::string>& attrs = r->get_attrs("article");
-	unsigned int i = 0;
-	std::string attrstr;
-	for (const auto& attribute : attrs) {
-		attrstr.append(
-			strprintf::fmt("@style_%u_normal:%s ", i, attribute));
-		i++;
-	}
+	std::string attrstr = rxman.get_attrs_stfl_string("article", false);
 	attrstr.append(
 		"@style_b_normal[color_bold]:attr=bold "
 		"@style_u_normal[color_underline]:attr=underline ");
-	std::string textview = strprintf::fmt(
+	std::string stfl_textview = strprintf::fmt(
 			"{textview[article] style_normal[article]: "
-			"style_end[styleend]:fg=blue,attr=bold %s .expand:vh "
-			"offset[articleoffset]:0 richtext:1}",
+			"style_end[end-of-text-marker]:fg=blue,attr=bold %s .expand:vh "
+			"offset[article_offset]:0 richtext:1}",
 			attrstr);
-	f->modify("article", "replace", textview);
+	textview.stfl_replace_textview(0, stfl_textview);
 }
 
 void ItemViewFormAction::update_percent()
 {
 	if (cfg->get_configvalue_as_bool("display-article-progress")) {
 		unsigned int percent = 0;
-		unsigned int offset = utils::to_u(f->get("articleoffset"), 0);
+		unsigned int offset = utils::to_u(f.get("article_offset"), 0);
 
 		if (num_lines > 0) {
 			percent = (100 * (offset + 1)) / num_lines;
@@ -582,11 +616,11 @@ void ItemViewFormAction::update_percent()
 			percent);
 
 		if (offset == 0 || percent == 0) {
-			f->set("percent", _("Top"));
+			f.set("percent", _("Top"));
 		} else if (offset == (num_lines - 1)) {
-			f->set("percent", _("Bottom"));
+			f.set("percent", _("Bottom"));
 		} else {
-			f->set("percent", strprintf::fmt("%3u %% ", percent));
+			f.set("percent", strprintf::fmt("%3u %% ", percent));
 		}
 	}
 }
@@ -595,7 +629,7 @@ std::string ItemViewFormAction::title()
 {
 	auto title = item->title();
 	utils::remove_soft_hyphens(title);
-	return strprintf::fmt(_("Article - %s"), title);
+	return strprintf::fmt(_("Article - %s"), utils::utf8_to_locale(title));
 }
 
 void ItemViewFormAction::set_highlightphrase(const std::string& text)
@@ -630,13 +664,13 @@ void ItemViewFormAction::highlight_text(const std::string& searchphrase)
 	std::copy(colors.begin(), colors.end(), std::back_inserter(params));
 
 	try {
-		rxman->handle_action("highlight", params);
+		rxman.handle_action("highlight", params);
 
 		LOG(Level::DEBUG,
 			"ItemViewFormAction::highlight_text: configuration "
 			"manipulation was successful");
 
-		set_regexmanager(rxman);
+		register_format_styles();
 
 		in_search = true;
 		do_redraw = true;
