@@ -1,6 +1,8 @@
 #include "keymap.h"
 
 #include <iostream>
+#include <map>
+#include <string>
 #include <vector>
 
 #include "config.h"
@@ -12,18 +14,18 @@
 namespace newsboat {
 
 struct OpDesc {
-	Operation op;
-	const char* opstr;
-	const char* default_key;
-	const char* help_text;
-	unsigned short flags;
+	const Operation op;
+	const std::string opstr;
+	const std::string default_key;
+	const std::string help_text;
+	const unsigned short flags;
 };
 
 /*
  * This is the list of operations, defining operation, operation name (for
  * keybindings), default key, description, and where it's valid
  */
-static OpDesc opdescs[] = {
+static const std::vector<OpDesc> opdescs = {
 	{
 		OP_OPEN,
 		"open",
@@ -32,6 +34,13 @@ static OpDesc opdescs[] = {
 		KM_FEEDLIST | KM_FILEBROWSER | KM_ARTICLELIST | KM_TAGSELECT |
 		KM_FILTERSELECT | KM_URLVIEW | KM_DIALOGS | KM_DIRBROWSER
 	},
+	{
+		OP_SWITCH_FOCUS,
+		"switch-focus",
+		"TAB",
+		_("Switch focus between widgets"),
+		KM_FILEBROWSER | KM_DIRBROWSER
+	},
 	{OP_QUIT, "quit", "q", _("Return to previous dialog/Quit"), KM_BOTH},
 	{
 		OP_HARDQUIT,
@@ -111,7 +120,7 @@ static OpDesc opdescs[] = {
 		"open-in-browser-and-mark-read",
 		"O",
 		_("Open article in browser and mark read"),
-		KM_ARTICLELIST
+		KM_ARTICLELIST | KM_ARTICLE | KM_URLVIEW
 	},
 	{
 		OP_OPENALLUNREADINBROWSER,
@@ -133,7 +142,7 @@ static OpDesc opdescs[] = {
 		"open-in-browser",
 		"o",
 		_("Open article in browser"),
-		KM_FEEDLIST | KM_ARTICLELIST | KM_ARTICLE
+		KM_FEEDLIST | KM_ARTICLELIST | KM_ARTICLE | KM_URLVIEW
 	},
 	{
 		OP_HELP,
@@ -378,30 +387,40 @@ static OpDesc opdescs[] = {
 		KM_FEEDLIST | KM_ARTICLELIST
 	},
 
-	{OP_0, "zero", "0", _("Open URL 10"), KM_URLVIEW | KM_ARTICLE},
-	{OP_1, "one", "1", _("Open URL 1"), KM_URLVIEW | KM_ARTICLE},
-	{OP_2, "two", "2", _("Open URL 2"), KM_URLVIEW | KM_ARTICLE},
-	{OP_3, "three", "3", _("Open URL 3"), KM_URLVIEW | KM_ARTICLE},
-	{OP_4, "four", "4", _("Open URL 4"), KM_URLVIEW | KM_ARTICLE},
-	{OP_5, "five", "5", _("Open URL 5"), KM_URLVIEW | KM_ARTICLE},
-	{OP_6, "six", "6", _("Open URL 6"), KM_URLVIEW | KM_ARTICLE},
-	{OP_7, "seven", "7", _("Open URL 7"), KM_URLVIEW | KM_ARTICLE},
-	{OP_8, "eight", "8", _("Open URL 8"), KM_URLVIEW | KM_ARTICLE},
-	{OP_9, "nine", "9", _("Open URL 9"), KM_URLVIEW | KM_ARTICLE},
+	{OP_OPEN_URL_10, "zero", "0", _("Open URL 10"), KM_URLVIEW | KM_ARTICLE},
+	{OP_OPEN_URL_1, "one", "1", _("Open URL 1"), KM_URLVIEW | KM_ARTICLE},
+	{OP_OPEN_URL_2, "two", "2", _("Open URL 2"), KM_URLVIEW | KM_ARTICLE},
+	{OP_OPEN_URL_3, "three", "3", _("Open URL 3"), KM_URLVIEW | KM_ARTICLE},
+	{OP_OPEN_URL_4, "four", "4", _("Open URL 4"), KM_URLVIEW | KM_ARTICLE},
+	{OP_OPEN_URL_5, "five", "5", _("Open URL 5"), KM_URLVIEW | KM_ARTICLE},
+	{OP_OPEN_URL_6, "six", "6", _("Open URL 6"), KM_URLVIEW | KM_ARTICLE},
+	{OP_OPEN_URL_7, "seven", "7", _("Open URL 7"), KM_URLVIEW | KM_ARTICLE},
+	{OP_OPEN_URL_8, "eight", "8", _("Open URL 8"), KM_URLVIEW | KM_ARTICLE},
+	{OP_OPEN_URL_9, "nine", "9", _("Open URL 9"), KM_URLVIEW | KM_ARTICLE},
+
+	{OP_CMD_START_1, "cmd-one", "1", _("Start cmdline with 1"), KM_FEEDLIST | KM_ARTICLELIST | KM_TAGSELECT | KM_FILTERSELECT},
+	{OP_CMD_START_2, "cmd-two", "2", _("Start cmdline with 2"), KM_FEEDLIST | KM_ARTICLELIST | KM_TAGSELECT | KM_FILTERSELECT},
+	{OP_CMD_START_3, "cmd-three", "3", _("Start cmdline with 3"), KM_FEEDLIST | KM_ARTICLELIST | KM_TAGSELECT | KM_FILTERSELECT},
+	{OP_CMD_START_4, "cmd-four", "4", _("Start cmdline with 4"), KM_FEEDLIST | KM_ARTICLELIST | KM_TAGSELECT | KM_FILTERSELECT},
+	{OP_CMD_START_5, "cmd-five", "5", _("Start cmdline with 5"), KM_FEEDLIST | KM_ARTICLELIST | KM_TAGSELECT | KM_FILTERSELECT},
+	{OP_CMD_START_6, "cmd-six", "6", _("Start cmdline with 6"), KM_FEEDLIST | KM_ARTICLELIST | KM_TAGSELECT | KM_FILTERSELECT},
+	{OP_CMD_START_7, "cmd-seven", "7", _("Start cmdline with 7"), KM_FEEDLIST | KM_ARTICLELIST | KM_TAGSELECT | KM_FILTERSELECT},
+	{OP_CMD_START_8, "cmd-eight", "8", _("Start cmdline with 8"), KM_FEEDLIST | KM_ARTICLELIST | KM_TAGSELECT | KM_FILTERSELECT},
+	{OP_CMD_START_9, "cmd-nine", "9", _("Start cmdline with 9"), KM_FEEDLIST | KM_ARTICLELIST | KM_TAGSELECT | KM_FILTERSELECT},
 
 	{OP_SK_UP, "up", "UP", _("Move to the previous entry"), KM_SYSKEYS},
 	{OP_SK_DOWN, "down", "DOWN", _("Move to the next entry"), KM_SYSKEYS},
 	{
 		OP_SK_PGUP,
 		"pageup",
-		"PAGEUP",
+		"PPAGE",
 		_("Move to the previous page"),
 		KM_SYSKEYS
 	},
 	{
 		OP_SK_PGDOWN,
 		"pagedown",
-		"PAGEDOWN",
+		"NPAGE",
 		_("Move to the next page"),
 		KM_SYSKEYS
 	},
@@ -425,55 +444,50 @@ static OpDesc opdescs[] = {
 		OP_INT_END_QUESTION,
 		"XXXNOKEY-end-question",
 		"end-question",
-		nullptr,
+		"",
 		KM_INTERNAL
 	},
 	{
 		OP_INT_CANCEL_QNA,
 		"XXXNOKEY-cancel-qna",
 		"cancel-qna",
-		nullptr,
+		"",
 		KM_INTERNAL
 	},
 	{
 		OP_INT_QNA_NEXTHIST,
 		"XXXNOKEY-qna-next-history",
 		"qna-next-history",
-		nullptr,
+		"",
 		KM_INTERNAL
 	},
 	{
 		OP_INT_QNA_PREVHIST,
 		"XXXNOKEY-qna-prev-history",
 		"qna-prev-history",
-		nullptr,
+		"",
 		KM_INTERNAL
 	},
 
-	{OP_INT_RESIZE, "RESIZE", "internal-resize", nullptr, KM_INTERNAL},
-	{OP_INT_SET, "set", "internal-set", nullptr, KM_INTERNAL},
+	{OP_INT_RESIZE, "RESIZE", "internal-resize", "", KM_INTERNAL},
+	{OP_INT_SET, "set", "internal-set", "", KM_INTERNAL},
 
-	{OP_INT_GOTO_URL, "gotourl", "internal-goto-url", nullptr, KM_INTERNAL},
-
-	{OP_NIL, nullptr, nullptr, nullptr, 0}
+	{OP_INT_GOTO_URL, "gotourl", "internal-goto-url", "", KM_INTERNAL},
 };
 
-// "all" must be first, the following positions must be the same as the KM_*
-// flag definitions (get_flag_from_context() relies on this).
-static const char* contexts[] = {"all",
-		"feedlist",
-		"filebrowser",
-		"help",
-		"articlelist",
-		"article",
-		"tagselection",
-		"filterselection",
-		"urlview",
-		"podbeuter",
-		"dialogs",
-		"dirbrowser",
-		nullptr
-	};
+static const std::map<std::string, std::uint32_t> contexts = {
+	{"feedlist", KM_FEEDLIST},
+	{"filebrowser", KM_FILEBROWSER},
+	{"help", KM_HELP},
+	{"articlelist", KM_ARTICLELIST},
+	{"article", KM_ARTICLE},
+	{"tagselection", KM_TAGSELECT},
+	{"filterselection", KM_FILTERSELECT},
+	{"urlview", KM_URLVIEW},
+	{"podboat", KM_PODBOAT},
+	{"dialogs", KM_DIALOGS},
+	{"dirbrowser", KM_DIRBROWSER},
+};
 
 KeyMap::KeyMap(unsigned flags)
 {
@@ -482,81 +496,58 @@ KeyMap::KeyMap(unsigned flags)
 	 * list above.
 	 */
 	LOG(Level::DEBUG, "KeyMap::KeyMap: flags = %x", flags);
-	for (unsigned int j = 1; contexts[j] != nullptr; j++) {
-		std::string ctx(contexts[j]);
-		for (int i = 0; opdescs[i].op != OP_NIL; ++i) {
-			if (opdescs[i].flags &
-				(flags | KM_INTERNAL | KM_SYSKEYS)) {
-				keymap_[ctx][opdescs[i].default_key] =
-					opdescs[i].op;
+	for (const auto& op_desc : opdescs) {
+		if (!(op_desc.flags & (flags | KM_INTERNAL | KM_SYSKEYS))) {
+			continue;
+		}
+
+		// Skip operations without a default key
+		if (op_desc.default_key.empty()) {
+			continue;
+		}
+
+		for (const auto& ctx : contexts) {
+			const std::string& context = ctx.first;
+			const std::uint32_t context_flag = ctx.second;
+			if ((op_desc.flags & (context_flag | KM_INTERNAL | KM_SYSKEYS))) {
+				keymap_[context][op_desc.default_key] = op_desc.op;
 			}
 		}
 	}
+
+	keymap_["help"]["b"] = OP_SK_PGUP;
+	keymap_["help"]["SPACE"] = OP_SK_PGDOWN;
+	keymap_["article"]["b"] = OP_SK_PGUP;
+	keymap_["article"]["SPACE"] = OP_SK_PGDOWN;
 }
 
-void KeyMap::get_keymap_descriptions(std::vector<KeyMapDesc>& descs,
-	unsigned short flags)
+std::vector<KeyMapDesc> KeyMap::get_keymap_descriptions(std::string context)
 {
-	/*
-	 * Here we return the keymap descriptions for the specified application
-	 * (handed to us via flags) This is used for the help screen.
-	 */
-	for (unsigned int i = 1; contexts[i] != nullptr; i++) {
-		std::string ctx(contexts[i]);
-
-		if (flags & KM_PODBOAT && ctx != "podbeuter") {
-			continue;
-		} else if (flags & KM_NEWSBOAT && ctx == "podbeuter") {
+	std::vector<KeyMapDesc> descs;
+	for (const auto& opdesc : opdescs) {
+		if (!(opdesc.flags & get_flag_from_context(context))) {
+			// Ignore operation if it is not valid in this context
 			continue;
 		}
 
-		for (unsigned int j = 0; opdescs[j].op != OP_NIL; ++j) {
-			bool already_added = false;
-			for (const auto& keymap : keymap_[ctx]) {
-				Operation op = keymap.second;
-				if (op != OP_NIL) {
-					if (opdescs[j].op == op &&
-						opdescs[j].flags & flags) {
-						KeyMapDesc desc;
-						desc.key = keymap.first;
-						desc.ctx = ctx;
-						if (!already_added) {
-							desc.cmd =
-								opdescs[j]
-								.opstr;
-							if (opdescs[j].help_text)
-								desc.desc = gettext(
-										opdescs[j]
-										.help_text);
-							already_added = true;
-						}
-						desc.flags = opdescs[j].flags;
-						descs.push_back(desc);
-					}
-				}
-			}
-			if (!already_added) {
-				if (opdescs[j].flags & flags) {
-					LOG(Level::DEBUG,
-						"KeyMap::get_keymap_"
-						"descriptions: "
-						"found unbound function: %s "
-						"ctx = "
-						"%s",
-						opdescs[j].opstr,
-						ctx);
-					KeyMapDesc desc;
-					desc.ctx = ctx;
-					desc.cmd = opdescs[j].opstr;
-					if (opdescs[j].help_text)
-						desc.desc = gettext(
-								opdescs[j].help_text);
-					desc.flags = opdescs[j].flags;
-					descs.push_back(desc);
-				}
+		bool bound_to_key = false;
+		for (const auto& keymap : keymap_[context]) {
+			const std::string& key = keymap.first;
+			const Operation op = keymap.second;
+			if (opdesc.op == op) {
+				descs.push_back({key, opdesc.opstr, opdesc.help_text, context, opdesc.flags});
+				bound_to_key = true;
 			}
 		}
+		if (!bound_to_key) {
+			LOG(Level::DEBUG,
+				"KeyMap::get_keymap_descriptions: found unbound function: %s context = %s",
+				opdesc.opstr,
+				context);
+			descs.push_back({"", opdesc.opstr, opdesc.help_text, context, opdesc.flags});
+		}
 	}
+	return descs;
 }
 
 KeyMap::~KeyMap() {}
@@ -567,8 +558,8 @@ void KeyMap::set_key(Operation op,
 {
 	LOG(Level::DEBUG, "KeyMap::set_key(%d,%s) called", op, key);
 	if (context == "all") {
-		for (unsigned int i = 0; contexts[i] != nullptr; i++) {
-			keymap_[contexts[i]][key] = op;
+		for (const auto& ctx : contexts) {
+			keymap_[ctx.first][key] = op;
 		}
 	} else {
 		keymap_[context][key] = op;
@@ -579,8 +570,8 @@ void KeyMap::unset_key(const std::string& key, const std::string& context)
 {
 	LOG(Level::DEBUG, "KeyMap::unset_key(%s) called", key);
 	if (context == "all") {
-		for (unsigned int i = 0; contexts[i] != nullptr; i++) {
-			keymap_[contexts[i]][key] = OP_NIL;
+		for (const auto& ctx : contexts) {
+			keymap_[ctx.first][key] = OP_NIL;
 		}
 	} else {
 		keymap_[context][key] = OP_NIL;
@@ -592,8 +583,8 @@ void KeyMap::unset_all_keys(const std::string& context)
 	LOG(Level::DEBUG, "KeyMap::unset_all_keys(%s) called", context);
 	auto internal_ops_only = get_internal_operations();
 	if (context == "all") {
-		for (unsigned int i = 0; contexts[i] != nullptr; i++) {
-			keymap_[contexts[i]] = internal_ops_only;
+		for (const auto& ctx : contexts) {
+			keymap_[ctx.first] = internal_ops_only;
 		}
 	} else {
 		keymap_[context] = std::move(internal_ops_only);
@@ -602,9 +593,9 @@ void KeyMap::unset_all_keys(const std::string& context)
 
 Operation KeyMap::get_opcode(const std::string& opstr)
 {
-	for (int i = 0; opdescs[i].opstr; ++i) {
-		if (opstr == opdescs[i].opstr) {
-			return opdescs[i].op;
+	for (const auto& opdesc : opdescs) {
+		if (opstr == opdesc.opstr) {
+			return opdesc.op;
 		}
 	}
 	return OP_NIL;
@@ -643,9 +634,9 @@ Operation KeyMap::get_operation(const std::string& keycode,
 
 void KeyMap::dump_config(std::vector<std::string>& config_output)
 {
-	for (unsigned int i = 1; contexts[i] != nullptr;
-		i++) { // TODO: optimize
-		std::map<std::string, Operation>& x = keymap_[contexts[i]];
+	for (const auto& ctx : contexts) {
+		const std::string& context = ctx.first;
+		std::map<std::string, Operation>& x = keymap_[context];
 		for (const auto& keymap : x) {
 			if (keymap.second < OP_INT_MIN) {
 				std::string configline = "bind-key ";
@@ -653,7 +644,7 @@ void KeyMap::dump_config(std::vector<std::string>& config_output)
 				configline.append(" ");
 				configline.append(getopname(keymap.second));
 				configline.append(" ");
-				configline.append(contexts[i]);
+				configline.append(context);
 				config_output.push_back(configline);
 			}
 		}
@@ -662,8 +653,8 @@ void KeyMap::dump_config(std::vector<std::string>& config_output)
 		std::string configline = "macro ";
 		configline.append(macro.first);
 		configline.append(" ");
-		unsigned int i = 0;
-		for (const auto& cmd : macro.second) {
+		for (unsigned int i = 0; i < macro.second.size(); ++i) {
+			const auto& cmd = macro.second[i];
 			configline.append(getopname(cmd.op));
 			for (const auto& arg : cmd.args) {
 				configline.append(" ");
@@ -679,9 +670,9 @@ void KeyMap::dump_config(std::vector<std::string>& config_output)
 
 std::string KeyMap::getopname(Operation op)
 {
-	for (unsigned int i = 0; opdescs[i].op != OP_NIL; i++) {
-		if (opdescs[i].op == op) {
-			return opdescs[i].opstr;
+	for (const auto& opdesc : opdescs) {
+		if (opdesc.op == op) {
+			return opdesc.opstr;
 		}
 	}
 	return "<none>";
@@ -713,9 +704,6 @@ void KeyMap::handle_action(const std::string& action,
 						"key command"),
 					params[1]));
 		}
-		if (op > OP_SK_MIN && op < OP_SK_MAX) {
-			unset_key(getkey(op, context), context);
-		}
 		set_key(op, params[0], context);
 	} else if (action == "unbind-key") {
 		if (params.size() < 1) {
@@ -732,9 +720,9 @@ void KeyMap::handle_action(const std::string& action,
 			unset_key(params[0], context);
 		}
 	} else if (action == "macro") {
-		if (params.size() < 1)
-			throw ConfigHandlerException(
-				ActionHandlerStatus::TOO_FEW_PARAMS);
+		if (params.size() < 2) {
+			throw ConfigHandlerException(ActionHandlerStatus::TOO_FEW_PARAMS);
+		}
 		auto it = params.begin();
 		std::string macrokey = *it;
 		std::vector<MacroCmd> cmds;
@@ -747,17 +735,15 @@ void KeyMap::handle_action(const std::string& action,
 			if (first && *it != ";") {
 				tmpcmd.op = get_opcode(*it);
 				LOG(Level::DEBUG,
-					"KeyMap::handle_action: new operation "
-					"`%s' "
-					"(op = %u)",
+					"KeyMap::handle_action: new operation `%s' (op = %u)",
 					*it,
 					tmpcmd.op);
-				if (tmpcmd.op == OP_NIL)
+				if (tmpcmd.op == OP_NIL) {
 					throw ConfigHandlerException(
 						strprintf::fmt(
-							_("`%s' is not a valid "
-								"key command"),
+							_("`%s' is not a valid operation"),
 							*it));
+				}
 				first = false;
 			} else {
 				if (*it == ";") {
@@ -769,8 +755,7 @@ void KeyMap::handle_action(const std::string& action,
 					first = true;
 				} else {
 					LOG(Level::DEBUG,
-						"KeyMap::handle_action: new "
-						"parameter `%s' (op = %u)",
+						"KeyMap::handle_action: new parameter `%s' (op = %u)",
 						*it,
 						tmpcmd.op);
 					tmpcmd.args.push_back(*it);
@@ -783,59 +768,50 @@ void KeyMap::handle_action(const std::string& action,
 		}
 
 		macros_[macrokey] = cmds;
-	} else
-		throw ConfigHandlerException(
-			ActionHandlerStatus::INVALID_PARAMS);
+	} else {
+		throw ConfigHandlerException(ActionHandlerStatus::INVALID_PARAMS);
+	}
 }
 
-std::string KeyMap::getkey(Operation op, const std::string& context)
+std::vector<std::string> KeyMap::get_keys(Operation op,
+	const std::string& context)
 {
-	if (context == "all") {
-		for (unsigned int i = 0; contexts[i] != nullptr; i++) {
-			std::string ctx(contexts[i]);
-			for (const auto& keymap : keymap_[ctx]) {
-				if (keymap.second == op) {
-					return keymap.first;
-				}
-			}
-		}
-	} else {
-		for (const auto& keymap : keymap_[context]) {
-			if (keymap.second == op) {
-				return keymap.first;
-			}
+	std::vector<std::string> keys;
+	for (const auto& keymap : keymap_[context]) {
+		if (keymap.second == op) {
+			keys.push_back(keymap.first);
 		}
 	}
-	return "<none>";
+	return keys;
 }
 
 std::vector<MacroCmd> KeyMap::get_macro(const std::string& key)
 {
-	for (const auto& macro : macros_) {
-		if (macro.first == key) {
-			return macro.second;
-		}
+	if (macros_.count(key) >= 1) {
+		return macros_.at(key);
 	}
-	std::vector<MacroCmd> dummyvector;
-	return dummyvector;
+	return {};
 }
 
 bool KeyMap::is_valid_context(const std::string& context)
 {
-	for (unsigned int i = 0; contexts[i] != nullptr; i++) {
-		if (context == contexts[i]) {
-			return true;
-		}
+	if (context == "all") {
+		return true;
 	}
+
+	if (contexts.count(context) >= 1) {
+		return true;
+	}
+
 	return false;
 }
 
 std::map<std::string, Operation> KeyMap::get_internal_operations() const
 {
 	std::map<std::string, Operation> internal_ops;
-	for (int i = 0; opdescs[i].op != OP_NIL; ++i) {
-		if (opdescs[i].flags & KM_INTERNAL) {
-			internal_ops[opdescs[i].default_key] = opdescs[i].op;
+	for (const auto& opdesc : opdescs) {
+		if (opdesc.flags & KM_INTERNAL) {
+			internal_ops[opdesc.default_key] = opdesc.op;
 		}
 	}
 	return internal_ops;
@@ -843,11 +819,10 @@ std::map<std::string, Operation> KeyMap::get_internal_operations() const
 
 unsigned short KeyMap::get_flag_from_context(const std::string& context)
 {
-	for (unsigned int i = 1; contexts[i] != nullptr; i++) {
-		if (context == contexts[i]) {
-			return (1 << (i - 1)) | KM_SYSKEYS;
-		}
+	if (contexts.count(context) >= 1) {
+		return contexts.at(context) | KM_SYSKEYS;
 	}
+
 	return 0; // shouldn't happen
 }
 
