@@ -5,7 +5,6 @@
 #include <ctime>
 #include <regex.h>
 #include <sstream>
-#include <sys/time.h>
 #include <vector>
 
 #include "logger.h"
@@ -16,9 +15,6 @@
 
 namespace newsboat {
 
-Matchable::Matchable() {}
-Matchable::~Matchable() {}
-
 Matcher::Matcher() {}
 
 Matcher::Matcher(const std::string& expr)
@@ -27,32 +23,28 @@ Matcher::Matcher(const std::string& expr)
 	parse(expr);
 }
 
-const std::string& Matcher::get_expression()
+std::string Matcher::get_expression()
 {
 	return exp;
 }
 
 bool Matcher::parse(const std::string& expr)
 {
-	struct timeval tv1, tv2;
-	gettimeofday(&tv1, nullptr);
+	ScopeMeasure measurer("Matcher::parse");
 
 	errmsg = "";
 
 	bool b = p.parse_string(expr);
 
-	if (!b) {
+	if (b) {
+		exp = expr;
+	} else {
 		errmsg = utils::wstr2str(p.get_error());
 	}
 
-	gettimeofday(&tv2, nullptr);
-	const uint64_t diff =
-		(((tv2.tv_sec - tv1.tv_sec) * 1000000) + tv2.tv_usec) -
-		tv1.tv_usec;
 	LOG(Level::DEBUG,
-		"Matcher::parse: parsing `%s' took %" PRIu64 " µs (success = %d)",
+		"Matcher::parse: parsing `%s' succeeded: %d",
 		expr,
-		diff,
 		b ? 1 : 0);
 
 	return b;
@@ -68,8 +60,7 @@ bool Matcher::matches(Matchable* item)
 	 * This makes it easy to use the Matcher virtually everywhere, since C++
 	 * allows multiple inheritance (i.e. deriving from Matchable can even be
 	 * used in class hierarchies), and deriving from Matchable means that
-	 * you only have to implement two methods has_attribute() and
-	 * get_attribute().
+	 * you only have to implement the method attribute_value().
 	 *
 	 * The whole matching code is speed-critical, as the matching happens on
 	 * a lot of different occassions, and slow matching can be easily
@@ -84,13 +75,26 @@ bool Matcher::matches(Matchable* item)
 	return retval;
 }
 
+std::string get_attr_or_throw(Matchable* item, const std::string& attr_name)
+{
+	const auto attr = item->attribute_value(attr_name);
+
+	if (!attr.has_value()) {
+		LOG(Level::WARN,
+			"Matcher::matches: attribute %s is not available",
+			attr_name);
+		throw MatcherException(MatcherException::Type::ATTRIB_UNAVAIL, attr_name);
+	}
+
+	return attr.value();
+}
+
 bool Matcher::matchop_lt(expression* e, Matchable* item)
 {
-	if (!item->has_attribute(e->name))
-		throw MatcherException(
-			MatcherException::Type::ATTRIB_UNAVAIL, e->name);
+	const auto attr = get_attr_or_throw(item, e->name);
+
 	std::istringstream islit(e->literal);
-	std::istringstream isatt(item->get_attribute(e->name));
+	std::istringstream isatt(attr);
 	int ilit, iatt;
 	islit >> ilit;
 	isatt >> iatt;
@@ -99,11 +103,10 @@ bool Matcher::matchop_lt(expression* e, Matchable* item)
 
 bool Matcher::matchop_between(expression* e, Matchable* item)
 {
-	if (!item->has_attribute(e->name))
-		throw MatcherException(
-			MatcherException::Type::ATTRIB_UNAVAIL, e->name);
-	std::vector<std::string> lit = utils::tokenize(e->literal, ":");
-	std::istringstream isatt(item->get_attribute(e->name));
+	const auto attr = get_attr_or_throw(item, e->name);
+
+	const std::vector<std::string> lit = utils::tokenize(e->literal, ":");
+	std::istringstream isatt(attr);
 	int att;
 	isatt >> att;
 	if (lit.size() < 2) {
@@ -123,11 +126,10 @@ bool Matcher::matchop_between(expression* e, Matchable* item)
 
 bool Matcher::matchop_gt(expression* e, Matchable* item)
 {
-	if (!item->has_attribute(e->name))
-		throw MatcherException(
-			MatcherException::Type::ATTRIB_UNAVAIL, e->name);
+	const auto attr = get_attr_or_throw(item, e->name);
+
 	std::istringstream islit(e->literal);
-	std::istringstream isatt(item->get_attribute(e->name));
+	std::istringstream isatt(attr);
 	int ilit, iatt;
 	islit >> ilit;
 	isatt >> iatt;
@@ -136,9 +138,8 @@ bool Matcher::matchop_gt(expression* e, Matchable* item)
 
 bool Matcher::matchop_rxeq(expression* e, Matchable* item)
 {
-	if (!item->has_attribute(e->name))
-		throw MatcherException(
-			MatcherException::Type::ATTRIB_UNAVAIL, e->name);
+	const auto attr = get_attr_or_throw(item, e->name);
+
 	if (!e->regex) {
 		e->regex = new regex_t;
 		int err;
@@ -156,7 +157,7 @@ bool Matcher::matchop_rxeq(expression* e, Matchable* item)
 		}
 	}
 	if (regexec(e->regex,
-			item->get_attribute(e->name).c_str(),
+			attr.c_str(),
 			0,
 			nullptr,
 			0) == 0) {
@@ -167,12 +168,10 @@ bool Matcher::matchop_rxeq(expression* e, Matchable* item)
 
 bool Matcher::matchop_cont(expression* e, Matchable* item)
 {
-	if (!item->has_attribute(e->name))
-		throw MatcherException(
-			MatcherException::Type::ATTRIB_UNAVAIL, e->name);
-	std::vector<std::string> elements =
-		utils::tokenize(item->get_attribute(e->name), " ");
-	std::string literal = e->literal;
+	const auto attr = get_attr_or_throw(item, e->name);
+
+	const std::vector<std::string> elements = utils::tokenize(attr, " ");
+	const std::string literal = e->literal;
 	for (const auto& elem : elements) {
 		if (literal == elem) {
 			return true;
@@ -183,14 +182,9 @@ bool Matcher::matchop_cont(expression* e, Matchable* item)
 
 bool Matcher::matchop_eq(expression* e, Matchable* item)
 {
-	if (!item->has_attribute(e->name)) {
-		LOG(Level::WARN,
-			"Matcher::matches_r: attribute %s not available",
-			e->name);
-		throw MatcherException(
-			MatcherException::Type::ATTRIB_UNAVAIL, e->name);
-	}
-	return (item->get_attribute(e->name) == e->literal);
+	const auto attr = get_attr_or_throw(item, e->name);
+
+	return (attr == e->literal);
 }
 
 bool Matcher::matches_r(expression* e, Matchable* item)
@@ -248,7 +242,7 @@ bool Matcher::matches_r(expression* e, Matchable* item)
 	}
 }
 
-const std::string& Matcher::get_parse_error()
+std::string Matcher::get_parse_error()
 {
 	return errmsg;
 }
