@@ -55,7 +55,7 @@ bool PbController::setup_dirs_xdg(const char* env_home)
 		xdg_config_dir = env_xdg_config;
 	} else {
 		xdg_config_dir = env_home;
-		xdg_config_dir.append(NEWSBEUTER_PATH_SEP);
+		xdg_config_dir.push_back(NEWSBEUTER_PATH_SEP);
 		xdg_config_dir.append(".config");
 	}
 
@@ -64,16 +64,16 @@ bool PbController::setup_dirs_xdg(const char* env_home)
 		xdg_data_dir = env_xdg_data;
 	} else {
 		xdg_data_dir = env_home;
-		xdg_data_dir.append(NEWSBEUTER_PATH_SEP);
+		xdg_data_dir.push_back(NEWSBEUTER_PATH_SEP);
 		xdg_data_dir.append(".local");
-		xdg_data_dir.append(NEWSBEUTER_PATH_SEP);
+		xdg_data_dir.push_back(NEWSBEUTER_PATH_SEP);
 		xdg_data_dir.append("share");
 	}
 
-	xdg_config_dir.append(NEWSBEUTER_PATH_SEP);
+	xdg_config_dir.push_back(NEWSBEUTER_PATH_SEP);
 	xdg_config_dir.append(NEWSBOAT_SUBDIR_XDG);
 
-	xdg_data_dir.append(NEWSBEUTER_PATH_SEP);
+	xdg_data_dir.push_back(NEWSBEUTER_PATH_SEP);
 	xdg_data_dir.append(NEWSBOAT_SUBDIR_XDG);
 
 	bool config_dir_exists =
@@ -108,20 +108,20 @@ bool PbController::setup_dirs_xdg(const char* env_home)
 	}
 
 	/* in config */
-	url_file = config_dir + std::string(NEWSBEUTER_PATH_SEP) + url_file;
+	url_file = config_dir + NEWSBEUTER_PATH_SEP + url_file;
 	config_file =
-		config_dir + std::string(NEWSBEUTER_PATH_SEP) + config_file;
+		config_dir + NEWSBEUTER_PATH_SEP + config_file;
 
 	/* in data */
 	cache_file =
-		xdg_data_dir + std::string(NEWSBEUTER_PATH_SEP) + cache_file;
+		xdg_data_dir + NEWSBEUTER_PATH_SEP + cache_file;
 	lock_file = cache_file + LOCK_SUFFIX;
 	queue_file =
-		xdg_data_dir + std::string(NEWSBEUTER_PATH_SEP) + queue_file;
+		xdg_data_dir + NEWSBEUTER_PATH_SEP + queue_file;
 	searchfile = strprintf::fmt(
-			"%s%shistory.search", xdg_data_dir, NEWSBEUTER_PATH_SEP);
+			"%s%chistory.search", xdg_data_dir, NEWSBEUTER_PATH_SEP);
 	cmdlinefile = strprintf::fmt(
-			"%s%shistory.cmdline", xdg_data_dir, NEWSBEUTER_PATH_SEP);
+			"%s%chistory.cmdline", xdg_data_dir, NEWSBEUTER_PATH_SEP);
 
 	return true;
 }
@@ -160,7 +160,7 @@ PbController::PbController()
 		return;
 	}
 
-	config_dir.append(NEWSBEUTER_PATH_SEP);
+	config_dir.push_back(NEWSBEUTER_PATH_SEP);
 	config_dir.append(NEWSBOAT_CONFIG_SUBDIR);
 
 	// create configuration directory if it doesn't exist
@@ -176,10 +176,9 @@ PbController::PbController()
 		::exit(EXIT_FAILURE);
 	}
 
-	config_file =
-		config_dir + std::string(NEWSBEUTER_PATH_SEP) + config_file;
-	queue_file = config_dir + std::string(NEWSBEUTER_PATH_SEP) + queue_file;
-	lock_file = config_dir + std::string(NEWSBEUTER_PATH_SEP) + lock_file;
+	config_file = config_dir + NEWSBEUTER_PATH_SEP + config_file;
+	queue_file = config_dir + NEWSBEUTER_PATH_SEP + queue_file;
+	lock_file = config_dir + NEWSBEUTER_PATH_SEP + lock_file;
 }
 
 PbController::~PbController()
@@ -301,12 +300,13 @@ int PbController::run(int argc, char* argv[])
 
 	std::cout << _("done.") << std::endl;
 
-	ql = new QueueLoader(queue_file, this);
+	ql = new QueueLoader(queue_file, *cfg,
+		std::bind(&PbController::set_view_update_necessary, this, true));
 	ql->reload(downloads_);
 
 	v->set_keymap(&keys);
 
-	v->run(automatic_dl);
+	v->run(automatic_dl, cfg->get_configvalue_as_bool("wrap-scroll"));
 
 	Stfl::reset();
 
@@ -381,11 +381,6 @@ void PbController::print_usage(const char* argv0)
 	}
 }
 
-std::string PbController::get_dlpath()
-{
-	return cfg->get_configvalue("download-path");
-}
-
 std::string PbController::get_formatstr()
 {
 	return cfg->get_configvalue("podlist-format");
@@ -407,10 +402,10 @@ unsigned int PbController::get_maxdownloads()
 	return max_dls;
 }
 
-void PbController::reload_queue(bool remove_unplayed)
+void PbController::purge_queue()
 {
 	if (ql) {
-		ql->reload(downloads_, remove_unplayed);
+		ql->reload(downloads_, true);
 	}
 }
 
