@@ -13,6 +13,7 @@
 #include "download.h"
 #include "fmtstrformatter.h"
 #include "help.h"
+#include "listformatter.h"
 #include "logger.h"
 #include "pbcontroller.h"
 #include "poddlthread.h"
@@ -28,6 +29,8 @@ PbView::PbView(PbController* c)
 	, dllist_form(dllist_str)
 	, help_form(help_str)
 	, keys(0)
+	, downloads_list("dls", dllist_form)
+	, help_textview("helptext", help_form)
 {
 	if (getenv("ESCDELAY") == nullptr) {
 		set_escdelay(25);
@@ -39,15 +42,21 @@ PbView::~PbView()
 	Stfl::reset();
 }
 
-void PbView::run(bool auto_download)
+void PbView::run(bool auto_download, bool wrap_scroll)
 {
 	bool quit = false;
 
+	// Make sure curses is initialized
+	dllist_form.run(-3);
+	// Hide cursor using curses
+	curs_set(0);
+
 	set_dllist_keymap_hint();
 
 	do {
 		if (ctrl->view_update_necessary()) {
-			double total_kbps = ctrl->get_total_kbps();
+			const double total_kbps = ctrl->get_total_kbps();
+			const auto speed = get_speed_human_readable(total_kbps);
 
 			char parbuf[128] = "";
 			if (ctrl->get_maxdownloads() > 1) {
@@ -61,12 +70,13 @@ void PbView::run(bool auto_download)
 			snprintf(buf,
 				sizeof(buf),
 				_("Queue (%u downloads in progress, %u total) "
-					"- %.2f kb/s total%s"),
+					"- %.2f %s total%s"),
 				static_cast<unsigned int>(
 					ctrl->downloads_in_progress()),
 				static_cast<unsigned int>(
 					ctrl->downloads().size()),
-				total_kbps,
+				speed.first,
+				speed.second.c_str(),
 				parbuf);
 
 			dllist_form.set("head", buf);
@@ -77,25 +87,20 @@ void PbView::run(bool auto_download)
 				"= %" PRIu64,
 				static_cast<uint64_t>(ctrl->downloads().size()));
 
-			std::string code = "{list";
+			ListFormatter listfmt;
 			std::string formatstring = ctrl->get_formatstr();
 
 			dllist_form.run(-3); // compute all widget dimensions
-			unsigned int width = utils::to_u(dllist_form.get("dls:w"));
+			const unsigned int width = downloads_list.get_width();
 
 			unsigned int i = 0;
 			for (const auto& dl : ctrl->downloads()) {
 				auto lbuf = format_line(formatstring, dl, i, width);
-				code.append(
-					strprintf::fmt("{listitem[%u] text:%s}",
-						i,
-						Stfl::quote(lbuf)));
+				listfmt.add_line(lbuf, std::to_string(i));
 				i++;
 			}
 
-			code.append("}");
-
-			dllist_form.modify("dls", "replace_inner", code);
+			downloads_list.stfl_replace_lines(listfmt);
 
 			ctrl->set_view_update_necessary(false);
 		}
@@ -113,7 +118,7 @@ void PbView::run(bool auto_download)
 			continue;
 		}
 
-		Operation op = keys->get_operation(event, "podbeuter");
+		Operation op = keys->get_operation(event, "podboat");
 
 		if (dllist_form.get("msg").length() > 0) {
 			dllist_form.set("msg", "");
@@ -121,6 +126,27 @@ void PbView::run(bool auto_download)
 		}
 
 		switch (op) {
+		case OP_REDRAW:
+			Stfl::reset();
+			break;
+		case OP_SK_UP:
+			downloads_list.move_up(wrap_scroll);
+			break;
+		case OP_SK_DOWN:
+			downloads_list.move_down(wrap_scroll);
+			break;
+		case OP_SK_HOME:
+			downloads_list.move_to_first();
+			break;
+		case OP_SK_END:
+			downloads_list.move_to_last();
+			break;
+		case OP_SK_PGUP:
+			downloads_list.move_page_up(wrap_scroll);
+			break;
+		case OP_SK_PGDOWN:
+			downloads_list.move_page_down(wrap_scroll);
+			break;
 		case OP_PB_TOGGLE_DLALL:
 			auto_download = !auto_download;
 			break;
@@ -225,7 +251,7 @@ void PbView::run(bool auto_download)
 					_("Error: unable to perform operation: "
 						"download(s) in progress."));
 			} else {
-				ctrl->reload_queue(true);
+				ctrl->purge_queue();
 			}
 			ctrl->set_view_update_necessary(true);
 			break;
@@ -239,35 +265,14 @@ void PbView::run(bool auto_download)
 	} while (!quit);
 }
 
-void PbView::set_bindings()
+std::pair<double, std::string> PbView::get_speed_human_readable(double kbps)
 {
-	if (keys) {
-		std::string upkey("** ");
-		upkey.append(keys->getkey(OP_SK_UP, "podbeuter"));
-		std::string downkey("** ");
-		downkey.append(keys->getkey(OP_SK_DOWN, "podbeuter"));
-		std::string pgupkey("** ");
-		pgupkey.append(keys->getkey(OP_SK_PGUP, "podbeuter"));
-		std::string pgdownkey("** ");
-		pgdownkey.append(keys->getkey(OP_SK_PGDOWN, "podbeuter"));
-		std::string homekey("** ");
-		homekey.append(keys->getkey(OP_SK_HOME, "podbeuter"));
-		std::string endkey("** ");
-		endkey.append(keys->getkey(OP_SK_END, "podbeuter"));
-
-		dllist_form.set("bind_up", upkey);
-		dllist_form.set("bind_down", downkey);
-		dllist_form.set("bind_page_up", pgupkey);
-		dllist_form.set("bind_page_down", pgdownkey);
-		dllist_form.set("bind_home", homekey);
-		dllist_form.set("bind_end", endkey);
-
-		help_form.set("bind_up", upkey);
-		help_form.set("bind_down", downkey);
-		help_form.set("bind_page_up", pgupkey);
-		help_form.set("bind_page_down", pgdownkey);
-		help_form.set("bind_home", homekey);
-		help_form.set("bind_end", endkey);
+	if (kbps < 1024) {
+		return std::make_pair(kbps, _("KB/s"));
+	} else if (kbps < 1024 * 1024) {
+		return std::make_pair(kbps / 1024, _("MB/s"));
+	} else {
+		return std::make_pair(kbps / 1024 / 1024, _("GB/s"));
 	}
 }
 
@@ -277,30 +282,23 @@ void PbView::run_help()
 
 	help_form.set("head", _("Help"));
 
-	std::vector<KeyMapDesc> descs;
-	keys->get_keymap_descriptions(descs, KM_PODBOAT);
+	const auto descs = keys->get_keymap_descriptions("podboat");
 
-	std::string code = "{list";
+	ListFormatter listfmt;
 
 	for (const auto& desc : descs) {
-		std::string line = "{listitem text:";
-
 		std::string descline;
 		descline.append(desc.key);
 		descline.append(8 - desc.key.length(), ' ');
 		descline.append(desc.cmd);
 		descline.append(24 - desc.cmd.length(), ' ');
 		descline.append(desc.desc);
-		line.append(Stfl::quote(descline));
 
-		line.append("}");
-
-		code.append(line);
+		listfmt.add_line(descline);
 	}
 
-	code.append("}");
-
-	help_form.modify("helptext", "replace_inner", code);
+	help_textview.stfl_replace_lines(listfmt.get_lines_count(),
+		listfmt.format_list());
 
 	bool quit = false;
 
@@ -313,6 +311,24 @@ void PbView::run_help()
 		Operation op = keys->get_operation(event, "help");
 
 		switch (op) {
+		case OP_SK_UP:
+			help_textview.scroll_up();
+			break;
+		case OP_SK_DOWN:
+			help_textview.scroll_down();
+			break;
+		case OP_SK_HOME:
+			help_textview.scroll_to_top();
+			break;
+		case OP_SK_END:
+			help_textview.scroll_to_bottom();
+			break;
+		case OP_SK_PGUP:
+			help_textview.scroll_page_up();
+			break;
+		case OP_SK_PGDOWN:
+			help_textview.scroll_page_down();
+			break;
 		case OP_HARDQUIT:
 		case OP_QUIT:
 			quit = true;
@@ -327,7 +343,12 @@ std::string PbView::prepare_keymaphint(KeyMapHintEntry* hints)
 {
 	std::string keymap_hint;
 	for (int i = 0; hints[i].op != OP_NIL; ++i) {
-		keymap_hint.append(keys->getkey(hints[i].op, "podbeuter"));
+		std::string bound_keys = utils::join(keys->get_keys(hints[i].op, "podboat"),
+				",");
+		if (bound_keys.empty()) {
+			bound_keys = "<none>";
+		}
+		keymap_hint.append(bound_keys);
 		keymap_hint.append(":");
 		keymap_hint.append(hints[i].text);
 		keymap_hint.append(" ");
@@ -367,19 +388,24 @@ std::string PbView::format_line(const std::string& podlist_format,
 {
 	FmtStrFormatter fmt;
 
+	const double speed_kbps = dl.kbps();
+	const auto speed = get_speed_human_readable(speed_kbps);
+
 	fmt.register_fmt('i', strprintf::fmt("%u", pos + 1));
 	fmt.register_fmt('d',
 		strprintf::fmt("%.1f", dl.current_size() / (1024 * 1024)));
 	fmt.register_fmt(
 		't', strprintf::fmt("%.1f", dl.total_size() / (1024 * 1024)));
 	fmt.register_fmt('p', strprintf::fmt("%.1f", dl.percents_finished()));
-	fmt.register_fmt('k', strprintf::fmt("%.2f", dl.kbps()));
+	fmt.register_fmt('k', strprintf::fmt("%.2f", speed_kbps));
+	fmt.register_fmt('K', strprintf::fmt("%.2f %s", speed.first, speed.second));
 	fmt.register_fmt('S', strprintf::fmt("%s", dl.status_text()));
 	fmt.register_fmt('u', strprintf::fmt("%s", dl.url()));
 	fmt.register_fmt('F', strprintf::fmt("%s", dl.filename()));
 	fmt.register_fmt('b', strprintf::fmt("%s", dl.basename()));
 
 	auto formattedLine = fmt.do_format(podlist_format, width);
+	formattedLine = utils::quote_for_stfl(formattedLine);
 	return formattedLine;
 }
 
