@@ -2,6 +2,7 @@
 
 #include <cassert>
 #include <sstream>
+#include <string>
 
 #include "config.h"
 #include "fmtstrformatter.h"
@@ -25,6 +26,7 @@ SelectFormAction::SelectFormAction(View* vv,
 	: FormAction(vv, formstr, cfg)
 	, quit(false)
 	, type(SelectionType::TAG)
+	, tags_list("taglist", FormAction::f)
 {
 }
 
@@ -38,19 +40,64 @@ void SelectFormAction::handle_cmdline(const std::string& cmd)
 			idx <= ((type == SelectionType::TAG)
 				? tags.size()
 				: filters.size())) {
-			f->set("tagpos", std::to_string(idx - 1));
+			tags_list.set_position(idx - 1);
 		}
 	} else {
 		FormAction::handle_cmdline(cmd);
 	}
 }
 
-void SelectFormAction::process_operation(Operation op,
+bool SelectFormAction::process_operation(Operation op,
 	bool /* automatic */,
 	std::vector<std::string>* /* args */)
 {
 	bool hardquit = false;
 	switch (op) {
+	case OP_SK_UP:
+		tags_list.move_up(cfg->get_configvalue_as_bool("wrap-scroll"));
+		break;
+	case OP_SK_DOWN:
+		tags_list.move_down(cfg->get_configvalue_as_bool("wrap-scroll"));
+		break;
+	case OP_SK_HOME:
+		tags_list.move_to_first();
+		break;
+	case OP_SK_END:
+		tags_list.move_to_last();
+		break;
+	case OP_SK_PGUP:
+		tags_list.move_page_up(cfg->get_configvalue_as_bool("wrap-scroll"));
+		break;
+	case OP_SK_PGDOWN:
+		tags_list.move_page_down(cfg->get_configvalue_as_bool("wrap-scroll"));
+		break;
+	case OP_CMD_START_1:
+		FormAction::start_cmdline("1");
+		break;
+	case OP_CMD_START_2:
+		FormAction::start_cmdline("2");
+		break;
+	case OP_CMD_START_3:
+		FormAction::start_cmdline("3");
+		break;
+	case OP_CMD_START_4:
+		FormAction::start_cmdline("4");
+		break;
+	case OP_CMD_START_5:
+		FormAction::start_cmdline("5");
+		break;
+	case OP_CMD_START_6:
+		FormAction::start_cmdline("6");
+		break;
+	case OP_CMD_START_7:
+		FormAction::start_cmdline("7");
+		break;
+	case OP_CMD_START_8:
+		FormAction::start_cmdline("8");
+		break;
+	case OP_CMD_START_9:
+		FormAction::start_cmdline("9");
+		break;
 	case OP_QUIT:
 		value = "";
 		quit = true;
@@ -60,7 +107,7 @@ void SelectFormAction::process_operation(Operation op,
 		hardquit = true;
 		break;
 	case OP_OPEN: {
-		std::string tagposname = f->get("tagposname");
+		std::string tagposname = f.get("tagposname");
 		unsigned int pos = utils::to_u(tagposname);
 		if (tagposname.length() > 0) {
 			switch (type) {
@@ -73,7 +120,7 @@ void SelectFormAction::process_operation(Operation op,
 			break;
 			case SelectionType::FILTER: {
 				if (pos < filters.size()) {
-					value = filters[pos].second;
+					value = filters[pos].expr;
 					quit = true;
 				}
 			}
@@ -95,6 +142,7 @@ void SelectFormAction::process_operation(Operation op,
 	} else if (quit) {
 		v->pop_current_formaction();
 	}
+	return true;
 }
 
 void SelectFormAction::prepare()
@@ -103,31 +151,33 @@ void SelectFormAction::prepare()
 		ListFormatter listfmt;
 		unsigned int i = 0;
 		const auto selecttag_format = cfg->get_configvalue("selecttag-format");
-		const auto width = utils::to_u(f->get("tags:w"));
+		const auto width = tags_list.get_width();
 
 		switch (type) {
 		case SelectionType::TAG:
 			for (const auto& tag : tags) {
-				listfmt.add_line(format_line(selecttag_format,
-						tag,
-						i + 1,
-						width),
-					i);
+				listfmt.add_line(
+					utils::quote_for_stfl(
+						format_line(selecttag_format,
+							tag,
+							i + 1,
+							width)),
+					std::to_string(i));
 				i++;
 			}
 			break;
 		case SelectionType::FILTER:
 			for (const auto& filter : filters) {
 				std::string tagstr = strprintf::fmt(
-						"%4u  %s", i + 1, filter.first);
-				listfmt.add_line(tagstr, i);
+						"%4u  %s", i + 1, filter.name);
+				listfmt.add_line(utils::quote_for_stfl(tagstr), std::to_string(i));
 				i++;
 			}
 			break;
 		default:
 			assert(0);
 		}
-		f->modify("taglist", "replace_inner", listfmt.format_list());
+		tags_list.stfl_replace_lines(listfmt);
 
 		do_redraw = false;
 	}
@@ -140,8 +190,9 @@ void SelectFormAction::init()
 	quit = false;
 	value = "";
 
-	std::string viewwidth = f->get("taglist:w");
-	unsigned int width = utils::to_u(viewwidth, 80);
+	f.run(-3); // compute all widget dimensions
+
+	const unsigned int width = tags_list.get_width();
 
 	set_keymap_hints();
 
@@ -162,7 +213,7 @@ void SelectFormAction::init()
 	default:
 		assert(0); // should never happen
 	}
-	f->set("head", title);
+	f.set("head", title);
 }
 
 std::string SelectFormAction::format_line(const std::string& selecttag_format,
