@@ -1,6 +1,7 @@
 #include "urlviewformaction.h"
 
 #include <sstream>
+#include <string>
 
 #include "config.h"
 #include "fmtstrformatter.h"
@@ -25,54 +26,69 @@ UrlViewFormAction::UrlViewFormAction(View* vv,
 	: FormAction(vv, formstr, cfg)
 	, quit(false)
 	, feed(feed)
+	, urls_list("urls", FormAction::f)
 {
 }
 
 UrlViewFormAction::~UrlViewFormAction() {}
 
-void UrlViewFormAction::process_operation(Operation op,
+bool UrlViewFormAction::process_operation(Operation op,
 	bool /* automatic */,
 	std::vector<std::string>* /* args */)
 {
 	bool hardquit = false;
 	switch (op) {
+	case OP_SK_UP:
+		urls_list.move_up(cfg->get_configvalue_as_bool("wrap-scroll"));
+		break;
+	case OP_SK_DOWN:
+		urls_list.move_down(cfg->get_configvalue_as_bool("wrap-scroll"));
+		break;
+	case OP_SK_HOME:
+		urls_list.move_to_first();
+		break;
+	case OP_SK_END:
+		urls_list.move_to_last();
+		break;
+	case OP_SK_PGUP:
+		urls_list.move_page_up(cfg->get_configvalue_as_bool("wrap-scroll"));
+		break;
+	case OP_SK_PGDOWN:
+		urls_list.move_page_down(cfg->get_configvalue_as_bool("wrap-scroll"));
+		break;
 	case OP_OPENINBROWSER:
+	case OP_OPENBROWSER_AND_MARK:
 	case OP_OPEN: {
-		std::string posstr = f->get("feedpos");
-		if (posstr.length() > 0) {
-			unsigned int idx = utils::to_u(posstr, 0);
+		if (!links.empty()) {
+			const unsigned int pos = urls_list.get_position();
 			v->set_status(_("Starting browser..."));
-			v->open_in_browser(links[idx].first);
+			v->open_in_browser(links[pos].first);
 			v->set_status("");
 		} else {
-			v->show_error(_("No link selected!"));
+			v->show_error(_("No links available!"));
 		}
 	}
 	break;
 	case OP_BOOKMARK: {
-		std::string posstr = f->get("feedpos");
-		if (posstr.length() > 0) {
-			unsigned int idx = utils::to_u(posstr, 0);
-
-			this->start_bookmark_qna(
-				"", links[idx].first, "", feed->title());
-
+		if (!links.empty()) {
+			const unsigned int pos = urls_list.get_position();
+			this->start_bookmark_qna("", links[pos].first, "", feed->title());
 		} else {
-			v->show_error(_("No link selected!"));
+			v->show_error(_("No links available!"));
 		}
 	}
 	break;
-	case OP_1:
-	case OP_2:
-	case OP_3:
-	case OP_4:
-	case OP_5:
-	case OP_6:
-	case OP_7:
-	case OP_8:
-	case OP_9:
-	case OP_0: {
-		unsigned int idx = op - OP_1;
+	case OP_OPEN_URL_1:
+	case OP_OPEN_URL_2:
+	case OP_OPEN_URL_3:
+	case OP_OPEN_URL_4:
+	case OP_OPEN_URL_5:
+	case OP_OPEN_URL_6:
+	case OP_OPEN_URL_7:
+	case OP_OPEN_URL_8:
+	case OP_OPEN_URL_9:
+	case OP_OPEN_URL_10: {
+		unsigned int idx = op - OP_OPEN_URL_1;
 
 		if (idx < links.size()) {
 			v->set_status(_("Starting browser..."));
@@ -97,6 +113,7 @@ void UrlViewFormAction::process_operation(Operation op,
 	} else if (quit) {
 		v->pop_current_formaction();
 	}
+	return true;
 }
 
 void UrlViewFormAction::prepare()
@@ -106,11 +123,11 @@ void UrlViewFormAction::prepare()
 		unsigned int i = 0;
 		for (const auto& link : links) {
 			listfmt.add_line(
-				strprintf::fmt("%2u  %s", i + 1, link.first),
-				i);
+				utils::quote_for_stfl(strprintf::fmt("%2u  %s", i + 1, link.first)),
+				std::to_string(i));
 			i++;
 		}
-		f->modify("urls", "replace_inner", listfmt.format_list());
+		urls_list.stfl_replace_lines(listfmt);
 	}
 }
 
@@ -118,14 +135,15 @@ void UrlViewFormAction::init()
 {
 	v->set_status("");
 
-	std::string viewwidth = f->get("urls:w");
-	unsigned int width = utils::to_u(viewwidth, 80);
+	f.run(-3); // compute all widget dimensions
+
+	const unsigned int width = urls_list.get_width();
 
 	FmtStrFormatter fmt;
 	fmt.register_fmt('N', PROGRAM_NAME);
 	fmt.register_fmt('V', utils::program_version());
 
-	f->set("head",
+	f.set("head",
 		fmt.do_format(
 			cfg->get_configvalue("urlview-title-format"), width));
 	do_redraw = true;
@@ -150,7 +168,7 @@ void UrlViewFormAction::handle_cmdline(const std::string& cmd)
 		if (idx < 1 || idx > links.size()) {
 			v->show_error(_("Invalid position!"));
 		} else {
-			f->set("feedpos", std::to_string(idx - 1));
+			urls_list.set_position(idx - 1);
 		}
 	} else {
 		FormAction::handle_cmdline(cmd);
