@@ -27,7 +27,9 @@
 #include <unordered_set>
 
 #include "config.h"
+#include "htmlrenderer.h"
 #include "logger.h"
+#include "ruststring.h"
 #include "strprintf.h"
 
 #if HAVE_GCRYPT
@@ -68,9 +70,10 @@ void append_escapes(std::string& str, char c)
 		str.append("\\`");
 		break;
 	case '\\':
+		str.append("\\");
 		break;
 	default:
-		str.append(1, c);
+		str.push_back(c);
 		break;
 	}
 }
@@ -106,7 +109,6 @@ std::vector<std::string> utils::tokenize_quoted(const std::string& str,
 	 * know from C/C++ strings.
 	 *
 	 */
-	bool attach_backslash = true;
 	std::vector<std::string> tokens;
 	std::string::size_type last_pos = str.find_first_not_of(delimiters, 0);
 	std::string::size_type pos = last_pos;
@@ -119,71 +121,33 @@ std::vector<std::string> utils::tokenize_quoted(const std::string& str,
 		if (str[last_pos] == '"') {
 			++last_pos;
 			pos = last_pos;
-			int backslash_count = 0;
-			while (pos < str.length() &&
-				(str[pos] != '"' || (backslash_count % 2))) {
-				if (str[pos] == '\\') {
-					++backslash_count;
+
+			std::string token;
+			while (pos < str.length()) {
+				if (str[pos] == '"') {
+					// We've reached the end of this quoted token.
+					++pos;
+					break;
+				} else if (str[pos] == '\\') {
+					if (pos + 1 < str.length()) {
+						append_escapes(token, str[pos + 1]);
+					}
+					pos += 2;
 				} else {
-					backslash_count = 0;
+					token.push_back(str[pos]);
+					++pos;
 				}
-				++pos;
 			}
-			if (pos >= str.length()) {
+			tokens.push_back(token);
+
+			if (pos == str.length()) {
 				pos = std::string::npos;
-				std::string token;
-				while (last_pos < str.length()) {
-					if (str[last_pos] == '\\') {
-						if (str[last_pos - 1] == '\\') {
-							if (attach_backslash) {
-								token.append(
-									"\\");
-							}
-							attach_backslash =
-								!attach_backslash;
-						}
-					} else {
-						if (str[last_pos - 1] == '\\') {
-							append_escapes(token,
-								str[last_pos]);
-						} else {
-							token.append(1,
-								str[last_pos]);
-						}
-					}
-					++last_pos;
-				}
-				tokens.push_back(token);
-			} else {
-				std::string token;
-				while (last_pos < pos) {
-					if (str[last_pos] == '\\') {
-						if (str[last_pos - 1] == '\\') {
-							if (attach_backslash) {
-								token.append(
-									"\\");
-							}
-							attach_backslash =
-								!attach_backslash;
-						}
-					} else {
-						if (str[last_pos - 1] == '\\') {
-							append_escapes(token,
-								str[last_pos]);
-						} else {
-							token.append(1,
-								str[last_pos]);
-						}
-					}
-					++last_pos;
-				}
-				tokens.push_back(token);
-				++pos;
 			}
 		} else {
 			pos = str.find_first_of(delimiters, last_pos);
 			tokens.push_back(str.substr(last_pos, pos - last_pos));
 		}
+
 		last_pos = str.find_first_not_of(delimiters, pos);
 	}
 
@@ -270,6 +234,10 @@ std::vector<std::string> utils::tokenize_nl(const std::string& str,
 		for (i = 0; i < last_pos; ++i) {
 			tokens.push_back(std::string("\n"));
 		}
+	} else {
+		for (i = 0; i < str.length(); ++i) {
+			tokens.push_back(std::string("\n"));
+		}
 	}
 
 	while (std::string::npos != pos || std::string::npos != last_pos) {
@@ -419,6 +387,15 @@ std::string utils::convert_text(const std::string& text,
 	return result;
 }
 
+std::string utils::utf8_to_locale(const std::string& text)
+{
+	if (text.empty()) {
+		return {};
+	}
+
+	return utils::convert_text(text, nl_langinfo(CODESET), "utf-8");
+}
+
 std::string utils::get_command_output(const std::string& cmd)
 {
 	return RustString(rs_get_command_output(cmd.c_str()));
@@ -428,16 +405,10 @@ void utils::extract_filter(const std::string& line,
 	std::string& filter,
 	std::string& url)
 {
-	std::string::size_type pos = line.find_first_of(":", 0);
-	std::string::size_type pos1 = line.find_first_of(":", pos + 1);
-	filter = line.substr(pos + 1, pos1 - pos - 1);
-	pos = pos1;
-	url = line.substr(pos + 1, line.length() - pos);
-	LOG(Level::DEBUG,
-		"utils::extract_filter: %s -> filter: %s url: %s",
-		line,
-		filter,
-		url);
+	FilterUrl filterUrl = rs_extract_filter(line.c_str());
+
+	filter = RustString(filterUrl.filter);
+	url = RustString(filterUrl.url);
 }
 
 static size_t my_write_data(void* buffer, size_t size, size_t nmemb,
@@ -504,7 +475,7 @@ void utils::run_command(const std::string& cmd, const std::string& input)
 	rs_run_command(cmd.c_str(), input.c_str());
 }
 
-std::string utils::run_program(char* argv[], const std::string& input)
+std::string utils::run_program(const char* argv[], const std::string& input)
 {
 	return RustString(rs_run_program(argv, input.c_str()));
 }
@@ -649,60 +620,13 @@ size_t utils::wcswidth_stfl(const std::wstring& str, size_t size)
 std::string utils::substr_with_width(const std::string& str,
 	const size_t max_width)
 {
-	// Returns a longest substring fits to the given width.
-	// Returns an empty string if `str` is an empty string or `max_width` is
-	// zero,
-	//
-	// Each chararacter width is calculated with wcwidth(3). If wcwidth()
-	// returns < 1, the character width is treated as 0. A STFL tag (e.g.
-	// `<b>`, `<foobar>`, `</>`) width is treated as 0, but escaped
-	// less-than (`<>`) width is treated as 1.
+	return RustString(rs_substr_with_width(str.c_str(), max_width));
+}
 
-	if (str.empty() || max_width == 0) {
-		return std::string("");
-	}
-
-	const std::wstring wstr = utils::str2wstr(str);
-	size_t total_width = 0;
-	bool in_bracket = false;
-	std::wstring result;
-	std::wstring tagbuf;
-
-	for (const auto& wc : wstr) {
-		if (in_bracket) {
-			tagbuf += wc;
-			if (wc == L'>') { // tagbuf is escaped less-than or tag
-				in_bracket = false;
-				if (tagbuf == L"<>") {
-					if (total_width + 1 > max_width) {
-						break;
-					}
-					result += L"<>"; // escaped less-than
-					tagbuf.clear();
-					total_width++;
-				} else {
-					result += tagbuf;
-					tagbuf.clear();
-				}
-			}
-		} else {
-			if (wc == L'<') {
-				in_bracket = true;
-				tagbuf += wc;
-			} else {
-				int w = wcwidth(wc);
-				if (w < 1) {
-					w = 0;
-				}
-				if (total_width + w > max_width) {
-					break;
-				}
-				total_width += w;
-				result += wc;
-			}
-		}
-	}
-	return utils::wstr2str(result);
+std::string utils::substr_with_width_stfl(const std::string& str,
+	const size_t max_width)
+{
+	return RustString(rs_substr_with_width_stfl(str.c_str(), max_width));
 }
 
 std::string utils::join(const std::vector<std::string>& strings,
@@ -946,22 +870,10 @@ std::string utils::make_title(const std::string& const_url)
 	return RustString(rs_make_title(const_url.c_str()));
 }
 
-int utils::run_interactively(const std::string& command,
+int32_t utils::run_interactively(const std::string& command,
 	const std::string& caller)
 {
-	LOG(Level::DEBUG, "%s: running `%s'", caller, command);
-
-	int status = ::system(command.c_str());
-
-	if (status == -1) {
-		LOG(Level::DEBUG,
-			"%s: couldn't create a child process",
-			caller);
-	} else if (status == 127) {
-		LOG(Level::DEBUG, "%s: couldn't run shell", caller);
-	}
-
-	return status;
+	return rs_run_interactively(command.c_str(), caller.c_str());
 }
 
 std::string utils::getcwd()
@@ -987,6 +899,18 @@ bool utils::is_valid_podcast_type(const std::string& mimetype)
 	return rs_is_valid_podcast_type(mimetype.c_str());
 }
 
+nonstd::optional<LinkType> utils::podcast_mime_to_link_type(
+	const std::string& mimetype)
+{
+	bool ok = false;
+	const auto result = rs_podcast_mime_to_link_type(mimetype.c_str(), &ok);
+	if (ok) {
+		return static_cast<LinkType>(result);
+	}
+
+	return nonstd::nullopt;
+}
+
 /*
  * See
  * http://curl.haxx.se/libcurl/c/libcurl-tutorial.html#Multi-threading
