@@ -64,12 +64,12 @@ View::View(Controller* c)
 	, cfg(0)
 	, keys(0)
 	, current_formaction(0)
-	, rxman(nullptr)
+	, rxman(c->get_regexmanager())
 	, is_inside_qna(false)
 	, is_inside_cmdline(false)
 	, tab_count(0)
 	, rsscache(nullptr)
-	, filters(nullptr)
+	, filters(ctrl->get_filtercontainer())
 {
 	if (getenv("ESCDELAY") == nullptr) {
 		set_escdelay(25);
@@ -91,48 +91,6 @@ void View::set_keymap(KeyMap* k)
 	keys = k;
 }
 
-void View::update_bindings()
-{
-	for (const auto& form : formaction_stack) {
-		if (form) {
-			set_bindings(form);
-		}
-	}
-}
-
-void View::set_bindings(std::shared_ptr<FormAction> fa)
-{
-	std::string upkey("** ");
-	upkey.append(keys->getkey(OP_SK_UP, fa->id()));
-	std::string downkey("** ");
-	downkey.append(keys->getkey(OP_SK_DOWN, fa->id()));
-	fa->get_form()->set("bind_up", upkey);
-	fa->get_form()->set("bind_down", downkey);
-
-	std::string pgupkey;
-	std::string pgdownkey;
-	if (fa->id() == "article" || fa->id() == "help") {
-		pgupkey.append("** b ");
-		pgdownkey.append("** SPACE ");
-	} else {
-		pgupkey.append("** ");
-		pgdownkey.append("** ");
-	}
-
-	pgupkey.append(keys->getkey(OP_SK_PGUP, fa->id()));
-	pgdownkey.append(keys->getkey(OP_SK_PGDOWN, fa->id()));
-
-	fa->get_form()->set("bind_page_up", pgupkey);
-	fa->get_form()->set("bind_page_down", pgdownkey);
-
-	std::string homekey("** ");
-	homekey.append(keys->getkey(OP_SK_HOME, fa->id()));
-	std::string endkey("** ");
-	endkey.append(keys->getkey(OP_SK_END, fa->id()));
-	fa->get_form()->set("bind_home", homekey);
-	fa->get_form()->set("bind_end", endkey);
-}
-
 std::shared_ptr<FormAction> View::get_current_formaction()
 {
 	if (formaction_stack.size() > 0 &&
@@ -143,27 +101,16 @@ std::shared_ptr<FormAction> View::get_current_formaction()
 	}
 }
 
-void View::set_status_unlocked(const std::string& msg)
-{
-	auto fa = get_current_formaction();
-	if (fa) {
-		std::shared_ptr<Stfl::Form> form = fa->get_form();
-		if (form) {
-			form->set("msg", msg);
-			form->run(-1);
-		} else {
-			LOG(Level::ERROR,
-				"View::set_status_unlocked: "
-				"form for formaction of type %s is nullptr!",
-				fa->id());
-		}
-	}
-}
-
 void View::set_status(const std::string& msg)
 {
 	std::lock_guard<std::mutex> lock(mtx);
-	set_status_unlocked(msg);
+
+	auto fa = get_current_formaction();
+	if (fa) {
+		Stfl::Form& form = fa->get_form();
+		form.set("msg", msg);
+		form.run(-1);
+	}
 }
 
 void View::show_error(const std::string& msg)
@@ -178,10 +125,7 @@ int View::run()
 
 	// create feedlist
 	auto feedlist = std::make_shared<FeedListFormAction>(
-			this, feedlist_str, rsscache, filters, cfg);
-	set_bindings(feedlist);
-	feedlist->set_regexmanager(rxman);
-	feedlist->set_tags(tags);
+			this, feedlist_str, rsscache, filters, cfg, rxman);
 	apply_colors(feedlist);
 	formaction_stack.push_back(feedlist);
 	current_formaction = formaction_stack_size() - 1;
@@ -207,15 +151,19 @@ int View::run()
 			// if there is any macro command left to process, we do
 			// so
 
-			fa->get_form()->run(-1);
-			fa->process_op(
-				macrocmds[0].op, true, &macrocmds[0].args);
+			fa->get_form().run(-1);
+			if (!fa->process_op(
+					macrocmds[0].op, true, &macrocmds[0].args)) {
 
-			// remove first macro command, since it has already been processed
-			macrocmds.erase(macrocmds.begin());
+				// Operation failed, abort
+				macrocmds.clear();
+			} else {
+				// remove first macro command, since it has already been processed
+				macrocmds.erase(macrocmds.begin());
+			}
 		} else {
 			// we then receive the event and ignore timeouts.
-			const char* event = fa->get_form()->run(60000);
+			const char* event = fa->get_form().run(60000);
 
 			if (ctrl_c_hit) {
 				ctrl_c_hit = false;
@@ -231,11 +179,6 @@ int View::run()
 			}
 
 			if (!event || strcmp(event, "TIMEOUT") == 0) {
-				if (fa->id() == "article")
-					std::dynamic_pointer_cast<
-					ItemViewFormAction,
-					FormAction>(fa)
-					->update_percent();
 				continue;
 			}
 
@@ -264,9 +207,6 @@ int View::run()
 
 			LOG(Level::DEBUG, "View::run: event = %s", event);
 
-			// retrieve operation code through the keymap
-			Operation op;
-
 			if (have_macroprefix) {
 				have_macroprefix = false;
 				LOG(Level::DEBUG,
@@ -275,7 +215,7 @@ int View::run()
 				macrocmds = keys->get_macro(event);
 				set_status("");
 			} else {
-				op = keys->get_operation(event, fa->id());
+				const Operation op = keys->get_operation(event, fa->id());
 
 				LOG(Level::DEBUG,
 					"View::run: event = %s op = %u",
@@ -312,7 +252,7 @@ std::string View::run_modal(std::shared_ptr<FormAction> f,
 
 		fa->prepare();
 
-		const char* event = fa->get_form()->run(1000);
+		const char* event = fa->get_form().run(1000);
 		LOG(Level::DEBUG, "View::run: event = %s", event);
 		if (!event || strcmp(event, "TIMEOUT") == 0) {
 			continue;
@@ -344,10 +284,10 @@ std::string View::get_filename_suggestion(const std::string& s)
 	std::string retval;
 	for (unsigned int i = 0; i < s.length(); ++i) {
 		if (isalnum(s[i])) {
-			retval.append(1, s[i]);
+			retval.push_back(s[i]);
 		} else if (s[i] == '/' || s[i] == ' ' || s[i] == '\r' ||
 			s[i] == '\n') {
-			retval.append(1, '_');
+			retval.push_back('_');
 		}
 	}
 	if (retval.length() == 0) {
@@ -397,27 +337,20 @@ int View::open_in_browser(const std::string& url)
 	formaction_stack.push_back(std::shared_ptr<FormAction>());
 	current_formaction = formaction_stack_size() - 1;
 	std::string cmdline;
-	std::string browser = cfg->get_configvalue("browser");
+	const std::string browser = cfg->get_configvalue("browser");
+	const std::string escaped_url = "'" + utils::replace_all(url, "'", "%27") + "'";
 	if (browser.find("%u") != std::string::npos) {
-		FmtStrFormatter fmt;
-		std::string newurl;
-		newurl = utils::replace_all(url, "'", "%27");
-		newurl.insert(0, "'");
-		newurl.append("'");
-		fmt.register_fmt('u', newurl);
-		cmdline = fmt.do_format(browser, 0);
+		cmdline = utils::replace_all(browser, "%u", escaped_url);
 	} else {
 		if (browser != "") {
 			cmdline.append(browser);
 		} else {
 			cmdline.append("lynx");
 		}
-		cmdline.append(" '");
-		cmdline.append(utils::replace_all(url, "'", "%27"));
-		cmdline.append("'");
+		cmdline.append(" " + escaped_url);
 	}
 	Stfl::reset();
-	int ret = utils::run_interactively(cmdline, "View::open_in_browser");
+	const int ret = utils::run_interactively(cmdline, "View::open_in_browser");
 	pop_current_formaction();
 	return ret;
 }
@@ -478,9 +411,7 @@ void View::push_searchresult(std::shared_ptr<RssFeed> feed,
 	if (feed->total_item_count() > 0) {
 		std::shared_ptr<ItemListFormAction> searchresult(
 			new ItemListFormAction(
-				this, itemlist_str, rsscache, filters, cfg));
-		set_bindings(searchresult);
-		searchresult->set_regexmanager(rxman);
+				this, itemlist_str, rsscache, filters, cfg, rxman));
 		searchresult->set_feed(feed);
 		searchresult->set_show_searchresult(true);
 		searchresult->set_searchphrase(phrase);
@@ -505,9 +436,7 @@ void View::push_itemlist(std::shared_ptr<RssFeed> feed)
 	if (feed->total_item_count() > 0) {
 		std::shared_ptr<ItemListFormAction> itemlist(
 			new ItemListFormAction(
-				this, itemlist_str, rsscache, filters, cfg));
-		set_bindings(itemlist);
-		itemlist->set_regexmanager(rxman);
+				this, itemlist_str, rsscache, filters, cfg, rxman));
 		itemlist->set_feed(feed);
 		itemlist->set_show_searchresult(false);
 		apply_colors(itemlist);
@@ -532,7 +461,9 @@ void View::push_itemlist(unsigned int pos)
 		std::shared_ptr<ItemListFormAction> itemlist =
 			std::dynamic_pointer_cast<ItemListFormAction,
 			FormAction>(get_current_formaction());
-		itemlist->set_pos(pos);
+		if (itemlist) {
+			itemlist->set_pos(pos);
+		}
 	}
 }
 
@@ -549,9 +480,7 @@ void View::push_itemview(std::shared_ptr<RssFeed> f,
 		assert(itemlist != nullptr);
 		std::shared_ptr<ItemViewFormAction> itemview(
 			new ItemViewFormAction(
-				this, itemlist, itemview_str, rsscache, cfg));
-		set_bindings(itemview);
-		itemview->set_regexmanager(rxman);
+				this, itemlist, itemview_str, rsscache, cfg, rxman));
 		itemview->set_feed(f);
 		itemview->set_guid(guid);
 		itemview->set_parent_formaction(fa);
@@ -589,7 +518,6 @@ void View::view_dialogs()
 		std::shared_ptr<DialogsFormAction> dialogs(
 			new DialogsFormAction(this, dialogs_str, cfg));
 		dialogs->set_parent_formaction(fa);
-		set_bindings(dialogs);
 		apply_colors(dialogs);
 		dialogs->init();
 		formaction_stack.push_back(dialogs);
@@ -603,7 +531,6 @@ void View::push_help()
 
 	std::shared_ptr<HelpFormAction> helpview(
 		new HelpFormAction(this, help_str, cfg));
-	set_bindings(helpview);
 	apply_colors(helpview);
 	helpview->set_context(fa->id());
 	helpview->set_parent_formaction(fa);
@@ -617,7 +544,6 @@ void View::push_urlview(const std::vector<LinkPair>& links,
 {
 	std::shared_ptr<UrlViewFormAction> urlview(
 		new UrlViewFormAction(this, feed, urlview_str, cfg));
-	set_bindings(urlview);
 	apply_colors(urlview);
 	urlview->set_parent_formaction(get_current_formaction());
 	urlview->init();
@@ -631,7 +557,6 @@ std::string View::run_filebrowser(const std::string& default_filename,
 {
 	std::shared_ptr<FileBrowserFormAction> filebrowser(
 		new FileBrowserFormAction(this, filebrowser_str, cfg));
-	set_bindings(filebrowser);
 	apply_colors(filebrowser);
 	filebrowser->set_dir(dir);
 	filebrowser->set_default_filename(default_filename);
@@ -643,7 +568,6 @@ std::string View::run_dirbrowser(const std::string& dir)
 {
 	std::shared_ptr<DirBrowserFormAction> dirbrowser(
 		new DirBrowserFormAction(this, filebrowser_str, cfg));
-	set_bindings(dirbrowser);
 	apply_colors(dirbrowser);
 	dirbrowser->set_dir(dir);
 	dirbrowser->set_parent_formaction(get_current_formaction());
@@ -659,7 +583,6 @@ std::string View::select_tag()
 	std::shared_ptr<SelectFormAction> selecttag(
 		new SelectFormAction(this, selecttag_str, cfg));
 	selecttag->set_type(SelectFormAction::SelectionType::TAG);
-	set_bindings(selecttag);
 	apply_colors(selecttag);
 	selecttag->set_parent_formaction(get_current_formaction());
 	selecttag->set_tags(tags);
@@ -672,7 +595,6 @@ std::string View::select_filter(const std::vector<FilterNameExprPair>& filters)
 	std::shared_ptr<SelectFormAction> selecttag(
 		new SelectFormAction(this, selecttag_str, cfg));
 	selecttag->set_type(SelectFormAction::SelectionType::FILTER);
-	set_bindings(selecttag);
 	apply_colors(selecttag);
 	selecttag->set_parent_formaction(get_current_formaction());
 	selecttag->set_filters(filters);
@@ -687,12 +609,12 @@ char View::confirm(const std::string& prompt, const std::string& charset)
 	std::shared_ptr<FormAction> f = get_current_formaction();
 	formaction_stack.push_back(std::shared_ptr<FormAction>());
 	current_formaction = formaction_stack_size() - 1;
-	f->get_form()->set("msg", prompt);
+	f->get_form().set("msg", prompt);
 
 	char result = 0;
 
 	do {
-		const char* event = f->get_form()->run(0);
+		const char* event = f->get_form().run(0);
 		LOG(Level::DEBUG, "View::confirm: event = %s", event);
 		if (!event) {
 			continue;
@@ -711,8 +633,8 @@ char View::confirm(const std::string& prompt, const std::string& charset)
 			result);
 	} while (!result || strchr(charset.c_str(), result) == nullptr);
 
-	f->get_form()->set("msg", "");
-	f->get_form()->run(-1);
+	f->get_form().set("msg", "");
+	f->get_form().run(-1);
 
 	pop_current_formaction();
 
@@ -1015,7 +937,7 @@ void View::force_redraw()
 	if (fa != nullptr) {
 		fa->set_redraw(true);
 		fa->prepare();
-		fa->get_form()->run(-1);
+		fa->get_form().run(-1);
 	}
 }
 
@@ -1053,7 +975,7 @@ void View::pop_current_formaction()
 		std::shared_ptr<FormAction> f = get_current_formaction();
 		if (f) {
 			f->set_redraw(true);
-			f->get_form()->set("msg", "");
+			f->get_form().set("msg", "");
 			f->recalculate_form();
 		}
 	}
@@ -1150,8 +1072,8 @@ void View::apply_colors(std::shared_ptr<FormAction> fa)
 				}
 				bold.append("attr=bold");
 				ul.append("attr=underline");
-				fa->get_form()->set("color_bold", bold.c_str());
-				fa->get_form()->set(
+				fa->get_form().set("color_bold", bold.c_str());
+				fa->get_form().set(
 					"color_underline", ul.c_str());
 			}
 		}
@@ -1162,24 +1084,7 @@ void View::apply_colors(std::shared_ptr<FormAction> fa)
 			fgcit->first,
 			colorattr);
 
-		fa->get_form()->set(fgcit->first, colorattr);
-
-		if (fgcit->first == "article") {
-			if (fa->id() == "article" || fa->id() == "help") {
-				std::string styleend_str;
-				if (bgcit->second != "default") {
-					styleend_str.append("bg=");
-					styleend_str.append(bgcit->second);
-				}
-				if (styleend_str.length() > 0) {
-					styleend_str.append(",");
-				}
-				styleend_str.append("attr=bold");
-
-				fa->get_form()->set(
-					"styleend", styleend_str.c_str());
-			}
-		}
+		fa->get_form().set(fgcit->first, colorattr);
 	}
 }
 
@@ -1192,21 +1097,11 @@ void View::feedlist_mark_pos_if_visible(unsigned int pos)
 	}
 }
 
-void View::set_regexmanager(RegexManager* r)
-{
-	rxman = r;
-}
-
 void View::set_cache(Cache* c)
 {
 	rsscache = c;
 }
 
-void View::set_filters(FilterContainer* f)
-{
-	filters = f;
-}
-
 std::vector<std::pair<unsigned int, std::string>> View::get_formaction_names()
 {
 	std::vector<std::pair<unsigned int, std::string>> formaction_names;
@@ -1252,18 +1147,18 @@ void View::inside_cmdline(bool f)
 
 void View::clear_line(std::shared_ptr<FormAction> fa)
 {
-	fa->get_form()->set("qna_value", "");
-	fa->get_form()->set("qna_value_pos", "0");
+	fa->get_form().set("qna_value", "");
+	fa->get_form().set("qna_value_pos", "0");
 	LOG(Level::DEBUG, "View::clear_line: cleared line");
 }
 
 void View::clear_eol(std::shared_ptr<FormAction> fa)
 {
-	unsigned int pos = utils::to_u(fa->get_form()->get("qna_value_pos"), 0);
-	std::string val = fa->get_form()->get("qna_value");
+	unsigned int pos = utils::to_u(fa->get_form().get("qna_value_pos"), 0);
+	std::string val = fa->get_form().get("qna_value");
 	val.erase(pos, val.length());
-	fa->get_form()->set("qna_value", val);
-	fa->get_form()->set("qna_value_pos", std::to_string(val.length()));
+	fa->get_form().set("qna_value", val);
+	fa->get_form().set("qna_value_pos", std::to_string(val.length()));
 	LOG(Level::DEBUG, "View::clear_eol: cleared to end of line");
 }
 
@@ -1276,8 +1171,8 @@ void View::cancel_input(std::shared_ptr<FormAction> fa)
 void View::delete_word(std::shared_ptr<FormAction> fa)
 {
 	std::string::size_type curpos =
-		utils::to_u(fa->get_form()->get("qna_value_pos"), 0);
-	std::string val = fa->get_form()->get("qna_value");
+		utils::to_u(fa->get_form().get("qna_value_pos"), 0);
+	std::string val = fa->get_form().get("qna_value");
 	std::string::size_type firstpos = curpos;
 	LOG(Level::DEBUG, "View::delete_word: before val = %s", val);
 	if (firstpos >= val.length() || ::isspace(val[firstpos])) {
@@ -1296,13 +1191,13 @@ void View::delete_word(std::shared_ptr<FormAction> fa)
 	}
 	val.erase(firstpos, curpos - firstpos);
 	LOG(Level::DEBUG, "View::delete_word: after val = %s", val);
-	fa->get_form()->set("qna_value", val);
-	fa->get_form()->set("qna_value_pos", std::to_string(firstpos));
+	fa->get_form().set("qna_value", val);
+	fa->get_form().set("qna_value_pos", std::to_string(firstpos));
 }
 
 void View::handle_cmdline_completion(std::shared_ptr<FormAction> fa)
 {
-	std::string fragment = fa->get_form()->get("qna_value");
+	std::string fragment = fa->get_form().get("qna_value");
 	if (fragment != last_fragment || fragment == "") {
 		last_fragment = fragment;
 		suggestions = fa->get_suggestions(fragment);
@@ -1327,8 +1222,8 @@ void View::handle_cmdline_completion(std::shared_ptr<FormAction> fa)
 		suggestion = suggestions[(tab_count - 1) % suggestions.size()];
 		break;
 	}
-	fa->get_form()->set("qna_value", suggestion);
-	fa->get_form()->set(
+	fa->get_form().set("qna_value", suggestion);
+	fa->get_form().set(
 		"qna_value_pos", std::to_string(suggestion.length()));
 	last_fragment = suggestion;
 }
@@ -1336,7 +1231,7 @@ void View::handle_cmdline_completion(std::shared_ptr<FormAction> fa)
 void View::dump_current_form()
 {
 	std::string formtext =
-		formaction_stack[current_formaction]->get_form()->dump(
+		formaction_stack[current_formaction]->get_form().dump(
 			"", "", 0);
 	time_t t = time(nullptr);
 	const auto fnbuf = utils::mt_strf_localtime(
