@@ -1,6 +1,8 @@
 #include "configcontainer.h"
 
 #include "3rd-party/catch.hpp"
+
+#include "configdata.h"
 #include "confighandlerexception.h"
 #include "configparser.h"
 #include "keymap.h"
@@ -122,6 +124,13 @@ TEST_CASE("reset_to_default changes setting to its default value",
 	}
 }
 
+TEST_CASE("get_configvalue() returns empty string if settings doesn't exist",
+	"[ConfigContainer]")
+{
+	ConfigContainer cfg;
+	REQUIRE(cfg.get_configvalue("nonexistent-key") == "");
+}
+
 TEST_CASE("get_configvalue_as_bool() recognizes several boolean formats",
 	"[ConfigContainer]")
 {
@@ -149,6 +158,27 @@ TEST_CASE("get_configvalue_as_bool() recognizes several boolean formats",
 	}
 }
 
+TEST_CASE("get_configvalue_as_int() returns zero if setting doesn't exist",
+	"[ConfigContainer]")
+{
+	ConfigContainer cfg;
+
+	REQUIRE(cfg.get_configvalue_as_int("setting-name") == 0);
+}
+
+TEST_CASE("get_configvalue_as_int() returns zero if value can't be parsed as int",
+	"[ConfigContainer]")
+{
+	const auto key = std::string("auto-reload");
+	const auto value = std::string("true");
+
+	ConfigContainer cfg;
+	cfg.set_configvalue(key, value);
+
+	REQUIRE(cfg.get_configvalue(key) == value);
+	REQUIRE(cfg.get_configvalue_as_int(key) == 0);
+}
+
 TEST_CASE("toggle() inverts the value of a boolean setting",
 	"[ConfigContainer]")
 {
@@ -258,6 +288,7 @@ TEST_CASE(
 	const std::string key1("d");
 	const std::unordered_set<std::string> expected1{
 		"datetime-format",
+		"delete-played-files",
 		"delete-read-articles-on-quit",
 		"dialogs-title-format",
 		"display-article-progress",
@@ -429,6 +460,59 @@ TEST_CASE(
 	}
 }
 
+TEST_CASE("get_feed_sort_strategy() returns \"none\" method if it can't parse it",
+	"[ConfigContainer]")
+{
+	ConfigContainer cfg;
+
+	const auto check = [&cfg]() {
+		const auto s = cfg.get_feed_sort_strategy();
+		REQUIRE(s.sm == FeedSortMethod::NONE);
+		REQUIRE(s.sd == SortDirection::DESC);
+	};
+
+	SECTION("empty value") {
+		cfg.set_configvalue("feed-sort-order", "");
+		check();
+	}
+
+	SECTION("unknown method") {
+		SECTION("without a direction") {
+			cfg.set_configvalue("feed-sort-order", "funniness");
+			check();
+		}
+
+		SECTION("with an unknown direction") {
+			cfg.set_configvalue("feed-sort-order", "funniness-increasing");
+			check();
+		}
+
+		SECTION("with a valid direction") {
+			cfg.set_configvalue("feed-sort-order", "funniness-asc");
+			const auto s = cfg.get_feed_sort_strategy();
+			REQUIRE(s.sm == FeedSortMethod::NONE);
+			REQUIRE(s.sd == SortDirection::ASC);
+		}
+	}
+}
+
+TEST_CASE("get_feed_sort_strategy() returns descending direction "
+	"if it can't parse it",
+	"[ConfigContainer]")
+{
+	ConfigContainer cfg;
+
+	SECTION("no direction specified") {
+		cfg.set_configvalue("feed-sort-order", "title");
+		REQUIRE(cfg.get_feed_sort_strategy().sd == SortDirection::DESC);
+	}
+
+	SECTION("unknown direction") {
+		cfg.set_configvalue("feed-sort-order", "title-increasing");
+		REQUIRE(cfg.get_feed_sort_strategy().sd == SortDirection::DESC);
+	}
+}
+
 TEST_CASE(
 	"get_article_sort_strategy() returns correctly filled "
 	"ArticleSortStrategy struct",
@@ -538,4 +622,75 @@ TEST_CASE(
 		REQUIRE(sort_strategy.sm == ArtSortMethod::DATE);
 		REQUIRE(sort_strategy.sd == SortDirection::DESC);
 	}
+
+	SECTION("random") {
+		cfg.set_configvalue("article-sort-order", "random");
+		sort_strategy = cfg.get_article_sort_strategy();
+		REQUIRE(sort_strategy.sm == ArtSortMethod::RANDOM);
+		REQUIRE(sort_strategy.sd == SortDirection::ASC);
+
+		cfg.set_configvalue("article-sort-order", "random-asc");
+		sort_strategy = cfg.get_article_sort_strategy();
+		REQUIRE(sort_strategy.sm == ArtSortMethod::RANDOM);
+		REQUIRE(sort_strategy.sd == SortDirection::ASC);
+
+		cfg.set_configvalue("article-sort-order", "random-desc");
+		sort_strategy = cfg.get_article_sort_strategy();
+		REQUIRE(sort_strategy.sm == ArtSortMethod::RANDOM);
+		REQUIRE(sort_strategy.sd == SortDirection::DESC);
+	}
+}
+
+TEST_CASE("get_article_sort_strategy() returns \"date\" method "
+	"if it can't parse it",
+	"[ConfigContainer]")
+{
+	ConfigContainer cfg;
+
+	const auto check = [&cfg]() {
+		const auto s = cfg.get_article_sort_strategy();
+		REQUIRE(s.sm == ArtSortMethod::DATE);
+		REQUIRE(s.sd == SortDirection::ASC);
+	};
+
+	SECTION("empty value") {
+		cfg.set_configvalue("article-sort-order", "");
+		check();
+	}
+
+	SECTION("unknown method") {
+		SECTION("without a direction") {
+			cfg.set_configvalue("article-sort-order", "funniness");
+			check();
+		}
+
+		SECTION("with an unknown direction") {
+			cfg.set_configvalue("article-sort-order", "funniness-increasing");
+			check();
+		}
+
+		SECTION("with a valid direction") {
+			cfg.set_configvalue("article-sort-order", "funniness-desc");
+			const auto s = cfg.get_article_sort_strategy();
+			REQUIRE(s.sm == ArtSortMethod::DATE);
+			REQUIRE(s.sd == SortDirection::DESC);
+		}
+	}
+}
+
+TEST_CASE("get_article_sort_strategy() returns ascending direction "
+	"if it can't parse it",
+	"[ConfigContainer]")
+{
+	ConfigContainer cfg;
+
+	SECTION("no direction specified and method is not \"date\"") {
+		cfg.set_configvalue("article-sort-order", "author");
+		REQUIRE(cfg.get_article_sort_strategy().sd == SortDirection::ASC);
+	}
+
+	SECTION("unknown direction") {
+		cfg.set_configvalue("article-sort-order", "author-increasing");
+		REQUIRE(cfg.get_article_sort_strategy().sd == SortDirection::ASC);
+	}
 }
