@@ -1,8 +1,11 @@
 #include "configparser.h"
 
+#include <fstream>
+
 #include "3rd-party/catch.hpp"
-#include "test-helpers.h"
 #include "keymap.h"
+#include "test-helpers/envvar.h"
+#include "test-helpers/tempfile.h"
 
 using namespace newsboat;
 
@@ -52,7 +55,7 @@ TEST_CASE("evaluate_backticks replaces command in backticks with its output",
 		KeyMap keys(KM_NEWSBOAT);
 		cfgparser.register_handler("bind-key", &keys);
 		REQUIRE_NOTHROW(cfgparser.parse("data/config-space-backticks"));
-		REQUIRE_FALSE(keys.get_operation("s", "all") == OP_NIL);
+		REQUIRE(keys.get_operation("s", "feedlist") == OP_SORT);
 	}
 
 	SECTION("Unbalanced backtick does *not* start a command") {
@@ -92,7 +95,10 @@ TEST_CASE("\"unbind-key -a\" removes all key bindings", "[ConfigParser]")
 		cfgparser.parse("data/config-unbind-all");
 
 		for (int i = OP_QUIT; i < OP_NB_MAX; ++i) {
-			REQUIRE(keys.getkey(static_cast<Operation>(i), "all") == "<none>");
+			REQUIRE(keys.get_keys(static_cast<Operation>(i),
+					"feedlist") == std::vector<std::string>());
+			REQUIRE(keys.get_keys(static_cast<Operation>(i),
+					"podboat") == std::vector<std::string>());
 		}
 	}
 
@@ -105,11 +111,12 @@ TEST_CASE("\"unbind-key -a\" removes all key bindings", "[ConfigParser]")
 		KeyMap default_keys(KM_NEWSBOAT);
 		for (int i = OP_QUIT; i < OP_NB_MAX; ++i) {
 			const auto op = static_cast<Operation>(i);
-			REQUIRE(keys.getkey(op, "help") == default_keys.getkey(op, "help"));
+			REQUIRE(keys.get_keys(op, "help") == default_keys.get_keys(op, "help"));
 		}
 
 		for (int i = OP_QUIT; i < OP_NB_MAX; ++i) {
-			REQUIRE(keys.getkey(static_cast<Operation>(i), "article") == "<none>");
+			REQUIRE(keys.get_keys(static_cast<Operation>(i),
+					"article") == std::vector<std::string>());
 		}
 	}
 }
