@@ -1,22 +1,26 @@
 #include "itemlistformaction.h"
 
+#include <fstream>
 #include <unistd.h>
 
 #include "3rd-party/catch.hpp"
 #include "cache.h"
+#include "configpaths.h"
 #include "feedlistformaction.h"
 #include "itemlist.h"
 #include "keymap.h"
 #include "regexmanager.h"
 #include "rssfeed.h"
-#include "test-helpers.h"
+#include "test-helpers/misc.h"
+#include "test-helpers/tempfile.h"
 
 using namespace newsboat;
 
 TEST_CASE("OP_OPEN displays article using an external pager",
 	"[ItemListFormAction]")
 {
-	Controller c;
+	ConfigPaths paths;
+	Controller c(paths);
 	newsboat::View v(&c);
 	TestHelpers::TempFile pagerfile;
 
@@ -33,6 +37,7 @@ TEST_CASE("OP_OPEN displays article using an external pager",
 
 	ConfigContainer cfg;
 	FilterContainer filters;
+	RegexManager rxman;
 
 	Cache rsscache(":memory:", &cfg);
 	cfg.set_configvalue("pager", "cat %f > " + pagerfile.get_path());
@@ -51,12 +56,12 @@ TEST_CASE("OP_OPEN displays article using an external pager",
 	v.set_config_container(&cfg);
 	c.set_view(&v);
 
-	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, &filters, &cfg);
+	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, filters, &cfg, rxman);
 	itemlist.set_feed(feed);
 
 	REQUIRE_NOTHROW(itemlist.process_op(OP_OPEN));
 
-	TestHelpers::AssertArticleFileContent(pagerfile.get_path(),
+	TestHelpers::assert_article_file_content(pagerfile.get_path(),
 		test_title,
 		test_author,
 		test_pubDate_str,
@@ -67,11 +72,13 @@ TEST_CASE("OP_OPEN displays article using an external pager",
 TEST_CASE("OP_PURGE_DELETED purges previously deleted items",
 	"[ItemListFormAction]")
 {
-	Controller c;
+	ConfigPaths paths;
+	Controller c(paths);
 	newsboat::View v(&c);
 	ConfigContainer cfg;
 	Cache rsscache(":memory:", &cfg);
 	FilterContainer filters;
+	RegexManager rxman;
 	std::shared_ptr<RssFeed> feed = std::make_shared<RssFeed>(&rsscache);
 	std::shared_ptr<RssItem> item = std::make_shared<RssItem>(&rsscache);
 	feed->add_item(item);
@@ -79,7 +86,7 @@ TEST_CASE("OP_PURGE_DELETED purges previously deleted items",
 	v.set_config_container(&cfg);
 	c.set_view(&v);
 
-	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, &filters, &cfg);
+	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, filters, &cfg, rxman);
 	itemlist.set_feed(feed);
 
 	SECTION("No items to purge") {
@@ -98,7 +105,8 @@ TEST_CASE(
 	"OP_OPENBROWSER_AND_MARK passes the url to the browser and marks read",
 	"[ItemListFormAction]")
 {
-	Controller c;
+	ConfigPaths paths;
+	Controller c(paths);
 	newsboat::View v(&c);
 	TestHelpers::TempFile browserfile;
 
@@ -110,6 +118,7 @@ TEST_CASE(
 
 	Cache rsscache(":memory:", &cfg);
 	FilterContainer filters;
+	RegexManager rxman;
 
 	std::shared_ptr<RssFeed> feed = std::make_shared<RssFeed>(&rsscache);
 	std::shared_ptr<RssItem> item = std::make_shared<RssItem>(&rsscache);
@@ -120,7 +129,7 @@ TEST_CASE(
 	v.set_config_container(&cfg);
 	c.set_view(&v);
 
-	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, &filters, &cfg);
+	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, filters, &cfg, rxman);
 	itemlist.set_feed(feed);
 	itemlist.process_op(OP_OPENBROWSER_AND_MARK);
 	std::ifstream browserFileStream(browserfile.get_path());
@@ -135,7 +144,8 @@ TEST_CASE(
 	"OP_OPENBROWSER_AND_MARK does not mark read when browser fails",
 	"[ItemListFormAction]")
 {
-	Controller c;
+	ConfigPaths paths;
+	Controller c(paths);
 	newsboat::View v(&c);
 
 	const std::string test_url = "http://test_url";
@@ -145,6 +155,7 @@ TEST_CASE(
 
 	Cache rsscache(":memory:", &cfg);
 	FilterContainer filters;
+	RegexManager rxman;
 
 	std::shared_ptr<RssFeed> feed = std::make_shared<RssFeed>(&rsscache);
 	std::shared_ptr<RssItem> item = std::make_shared<RssItem>(&rsscache);
@@ -155,7 +166,7 @@ TEST_CASE(
 	v.set_config_container(&cfg);
 	c.set_view(&v);
 
-	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, &filters, &cfg);
+	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, filters, &cfg, rxman);
 	itemlist.set_feed(feed);
 	itemlist.process_op(OP_OPENBROWSER_AND_MARK);
 
@@ -165,7 +176,8 @@ TEST_CASE(
 TEST_CASE("OP_OPENINBROWSER passes the url to the browser",
 	"[ItemListFormAction]")
 {
-	Controller c;
+	ConfigPaths paths;
+	Controller c(paths);
 	newsboat::View v(&c);
 	TestHelpers::TempFile browserfile;
 	const std::string test_url = "http://test_url";
@@ -176,6 +188,7 @@ TEST_CASE("OP_OPENINBROWSER passes the url to the browser",
 
 	Cache rsscache(":memory:", &cfg);
 	FilterContainer filters;
+	RegexManager rxman;
 
 	std::shared_ptr<RssFeed> feed = std::make_shared<RssFeed>(&rsscache);
 	std::shared_ptr<RssItem> item = std::make_shared<RssItem>(&rsscache);
@@ -185,7 +198,7 @@ TEST_CASE("OP_OPENINBROWSER passes the url to the browser",
 	v.set_config_container(&cfg);
 	c.set_view(&v);
 
-	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, &filters, &cfg);
+	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, filters, &cfg, rxman);
 	itemlist.set_feed(feed);
 	itemlist.process_op(OP_OPENINBROWSER);
 	std::ifstream browserFileStream(browserfile.get_path());
@@ -197,7 +210,8 @@ TEST_CASE("OP_OPENINBROWSER passes the url to the browser",
 TEST_CASE("OP_OPENALLUNREADINBROWSER passes the url list to the browser",
 	"[ItemListFormAction]")
 {
-	Controller c;
+	ConfigPaths paths;
+	Controller c(paths);
 	newsboat::View v(&c);
 	TestHelpers::TempFile browserfile;
 	std::unordered_set<std::string> url_set;
@@ -210,6 +224,7 @@ TEST_CASE("OP_OPENALLUNREADINBROWSER passes the url list to the browser",
 
 	Cache rsscache(":memory:", &cfg);
 	FilterContainer filters;
+	RegexManager rxman;
 
 	std::shared_ptr<RssFeed> feed = std::make_shared<RssFeed>(&rsscache);
 
@@ -225,7 +240,7 @@ TEST_CASE("OP_OPENALLUNREADINBROWSER passes the url list to the browser",
 	v.set_config_container(&cfg);
 	c.set_view(&v);
 
-	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, &filters, &cfg);
+	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, filters, &cfg, rxman);
 	itemlist.set_feed(feed);
 
 	SECTION("unread >= max-browser-tabs") {
@@ -279,23 +294,25 @@ TEST_CASE(
 	"and marks them read",
 	"[ItemListFormAction]")
 {
-	Controller c;
+	ConfigPaths paths;
+	Controller c(paths);
 	newsboat::View v(&c);
 	TestHelpers::TempFile browserfile;
 	std::unordered_set<std::string> url_set;
 	const std::string test_url = "http://test_url";
 	std::string line;
-	int itemCount = 6;
+	const unsigned int itemCount = 6;
 
 	ConfigContainer cfg;
 	cfg.set_configvalue("browser", "echo %u >> " + browserfile.get_path());
 
 	Cache rsscache(":memory:", &cfg);
 	FilterContainer filters;
+	RegexManager rxman;
 
 	std::shared_ptr<RssFeed> feed = std::make_shared<RssFeed>(&rsscache);
 
-	for (int i = 0; i < itemCount; i++) {
+	for (unsigned int i = 0; i < itemCount; i++) {
 		std::shared_ptr<RssItem> item =
 			std::make_shared<RssItem>(&rsscache);
 		item->set_link(test_url + std::to_string(i));
@@ -307,12 +324,12 @@ TEST_CASE(
 	v.set_config_container(&cfg);
 	c.set_view(&v);
 
-	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, &filters, &cfg);
+	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, filters, &cfg, rxman);
 	itemlist.set_feed(feed);
 
 	SECTION("unread >= max-browser-tabs") {
-		int maxItemsToOpen = 4;
-		int openedItemsCount = 0;
+		const unsigned int maxItemsToOpen = 4;
+		unsigned int openedItemsCount = 0;
 		cfg.set_configvalue(
 			"max-browser-tabs", std::to_string(maxItemsToOpen));
 
@@ -360,11 +377,13 @@ TEST_CASE(
 
 TEST_CASE("OP_SHOWURLS shows the article's properties", "[ItemListFormAction]")
 {
-	Controller c;
+	ConfigPaths paths;
+	Controller c(paths);
 	newsboat::View v(&c);
 	ConfigContainer cfg;
 	Cache rsscache(":memory:", &cfg);
 	FilterContainer filters;
+	RegexManager rxman;
 	TestHelpers::TempFile urlFile;
 
 	const std::string test_url = "http://test_url";
@@ -389,7 +408,7 @@ TEST_CASE("OP_SHOWURLS shows the article's properties", "[ItemListFormAction]")
 	item->set_author(test_author);
 	item->set_description(test_description);
 	item->set_pubDate(test_pubDate);
-	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, &filters, &cfg);
+	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, filters, &cfg, rxman);
 
 	SECTION("with external-url-viewer") {
 		feed->add_item(item);
@@ -399,7 +418,7 @@ TEST_CASE("OP_SHOWURLS shows the article's properties", "[ItemListFormAction]")
 
 		REQUIRE_NOTHROW(itemlist.process_op(OP_SHOWURLS));
 
-		TestHelpers::AssertArticleFileContent(urlFile.get_path(),
+		TestHelpers::assert_article_file_content(urlFile.get_path(),
 			test_title,
 			test_author,
 			test_pubDate_str,
@@ -421,11 +440,13 @@ TEST_CASE("OP_SHOWURLS shows the article's properties", "[ItemListFormAction]")
 TEST_CASE("OP_BOOKMARK pipes articles url and title to bookmark-command",
 	"[ItemListFormAction]")
 {
-	Controller c;
+	ConfigPaths paths;
+	Controller c(paths);
 	newsboat::View v(&c);
 	ConfigContainer cfg;
 	Cache rsscache(":memory:", &cfg);
 	FilterContainer filters;
+	RegexManager rxman;
 	TestHelpers::TempFile bookmarkFile;
 	std::string line;
 	std::vector<std::string> bookmark_args;
@@ -446,7 +467,7 @@ TEST_CASE("OP_BOOKMARK pipes articles url and title to bookmark-command",
 	item->set_link(test_url);
 	item->set_title(test_title);
 
-	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, &filters, &cfg);
+	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, filters, &cfg, rxman);
 
 	feed->add_item(item);
 	itemlist.set_feed(feed);
@@ -468,11 +489,13 @@ TEST_CASE("OP_BOOKMARK pipes articles url and title to bookmark-command",
 TEST_CASE("OP_EDITFLAGS arguments are added to an item's flags",
 	"[ItemListFormAction]")
 {
-	Controller c;
+	ConfigPaths paths;
+	Controller c(paths);
 	newsboat::View v(&c);
 	ConfigContainer cfg;
 	Cache rsscache(":memory:", &cfg);
 	FilterContainer filters;
+	RegexManager rxman;
 
 	std::vector<std::string> op_args;
 
@@ -482,7 +505,7 @@ TEST_CASE("OP_EDITFLAGS arguments are added to an item's flags",
 	std::shared_ptr<RssFeed> feed = std::make_shared<RssFeed>(&rsscache);
 	std::shared_ptr<RssItem> item = std::make_shared<RssItem>(&rsscache);
 
-	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, &filters, &cfg);
+	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, filters, &cfg, rxman);
 
 	feed->add_item(item);
 	itemlist.set_feed(feed);
@@ -555,12 +578,14 @@ TEST_CASE("OP_EDITFLAGS arguments are added to an item's flags",
 TEST_CASE("OP_SAVE writes an article's attributes to the specified file",
 	"[ItemListFormAction]")
 {
-	Controller c;
+	ConfigPaths paths;
+	Controller c(paths);
 	newsboat::View v(&c);
 	TestHelpers::TempFile saveFile;
 	ConfigContainer cfg;
 	Cache rsscache(":memory:", &cfg);
 	FilterContainer filters;
+	RegexManager rxman;
 
 	std::vector<std::string> op_args;
 	op_args.push_back(saveFile.get_path());
@@ -588,14 +613,14 @@ TEST_CASE("OP_SAVE writes an article's attributes to the specified file",
 	item->set_pubDate(test_pubDate);
 	item->set_description(test_description);
 
-	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, &filters, &cfg);
+	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, filters, &cfg, rxman);
 
 	feed->add_item(item);
 	itemlist.set_feed(feed);
 
 	REQUIRE_NOTHROW(itemlist.process_op(OP_SAVE, true, &op_args));
 
-	TestHelpers::AssertArticleFileContent(saveFile.get_path(),
+	TestHelpers::assert_article_file_content(saveFile.get_path(),
 		test_title,
 		test_author,
 		test_pubDate_str,
@@ -605,24 +630,24 @@ TEST_CASE("OP_SAVE writes an article's attributes to the specified file",
 
 TEST_CASE("OP_HELP command is processed", "[ItemListFormAction]")
 {
-	Controller c;
-	RegexManager regman;
+	ConfigPaths paths;
+	Controller c(paths);
 	newsboat::View v(&c);
 	ConfigContainer cfg;
 	Cache rsscache(":memory:", &cfg);
 	FilterContainer filters;
+	RegexManager rxman;
 
 	KeyMap k(KM_NEWSBOAT);
 	v.set_keymap(&k);
 
-	v.set_regexmanager(&regman);
 	v.set_config_container(&cfg);
 	c.set_view(&v);
 
 	std::shared_ptr<RssFeed> feed = std::make_shared<RssFeed>(&rsscache);
 	std::shared_ptr<RssItem> item = std::make_shared<RssItem>(&rsscache);
 
-	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, &filters, &cfg);
+	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, filters, &cfg, rxman);
 	feed->add_item(item);
 	itemlist.set_feed(feed);
 
@@ -633,23 +658,23 @@ TEST_CASE("OP_HELP command is processed", "[ItemListFormAction]")
 
 TEST_CASE("OP_HARDQUIT command is processed", "[ItemListFormAction]")
 {
-	Controller c;
-	RegexManager regman;
+	ConfigPaths paths;
+	Controller c(paths);
 	newsboat::View v(&c);
 	ConfigContainer cfg;
 	Cache rsscache(":memory:", &cfg);
 	FilterContainer filters;
+	RegexManager rxman;
 
 	KeyMap k(KM_NEWSBOAT);
 	v.set_keymap(&k);
 
-	v.set_regexmanager(&regman);
 	v.set_config_container(&cfg);
 	c.set_view(&v);
 
 	std::shared_ptr<RssFeed> feed = std::make_shared<RssFeed>(&rsscache);
 
-	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, &filters, &cfg);
+	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, filters, &cfg, rxman);
 	itemlist.set_feed(feed);
 
 	v.push_itemlist(feed);
@@ -663,15 +688,16 @@ TEST_CASE("Navigate back and forth using OP_NEXT and OP_PREVIOUS",
 	// We are using the OP_SHOWURLS command to print the current
 	// article'attibutes to a file, and assert the position was indeed
 	// updated.
-	Controller c;
+	ConfigPaths paths;
+	Controller c(paths);
 	TestHelpers::TempFile articleFile;
-	RegexManager regman;
 	newsboat::View v(&c);
 	ConfigContainer cfg;
 	cfg.set_configvalue(
 		"external-url-viewer", "tee > " + articleFile.get_path());
 	Cache rsscache(":memory:", &cfg);
 	FilterContainer filters;
+	RegexManager rxman;
 	std::string line;
 
 	std::string first_article_title = "First_Article";
@@ -681,7 +707,6 @@ TEST_CASE("Navigate back and forth using OP_NEXT and OP_PREVIOUS",
 	KeyMap k(KM_NEWSBOAT);
 	v.set_keymap(&k);
 
-	v.set_regexmanager(&regman);
 	v.set_config_container(&cfg);
 	c.set_view(&v);
 
@@ -695,7 +720,7 @@ TEST_CASE("Navigate back and forth using OP_NEXT and OP_PREVIOUS",
 	item2->set_title(second_article_title);
 	feed->add_item(item2);
 
-	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, &filters, &cfg);
+	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, filters, &cfg, rxman);
 	itemlist.set_feed(feed);
 
 	v.push_itemlist(feed);
@@ -718,17 +743,17 @@ TEST_CASE("Navigate back and forth using OP_NEXT and OP_PREVIOUS",
 TEST_CASE("OP_TOGGLESHOWREAD switches the value of show-read-articles",
 	"[ItemListFormAction]")
 {
-	Controller c;
-	RegexManager regman;
+	ConfigPaths paths;
+	Controller c(paths);
 	newsboat::View v(&c);
 	ConfigContainer cfg;
 	Cache rsscache(":memory:", &cfg);
 	FilterContainer filters;
+	RegexManager rxman;
 
 	KeyMap k(KM_NEWSBOAT);
 	v.set_keymap(&k);
 
-	v.set_regexmanager(&regman);
 	v.set_config_container(&cfg);
 	c.set_view(&v);
 
@@ -737,7 +762,7 @@ TEST_CASE("OP_TOGGLESHOWREAD switches the value of show-read-articles",
 	std::shared_ptr<RssItem> item = std::make_shared<RssItem>(&rsscache);
 	feed->add_item(item);
 
-	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, &filters, &cfg);
+	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, filters, &cfg, rxman);
 	itemlist.set_feed(feed);
 	v.push_itemlist(feed);
 
@@ -758,12 +783,14 @@ TEST_CASE("OP_TOGGLESHOWREAD switches the value of show-read-articles",
 TEST_CASE("OP_PIPE_TO pipes an article's content to an external command",
 	"[ItemListFormAction]")
 {
-	Controller c;
+	ConfigPaths paths;
+	Controller c(paths);
 	newsboat::View v(&c);
 	TestHelpers::TempFile articleFile;
 	ConfigContainer cfg;
 	Cache rsscache(":memory:", &cfg);
 	FilterContainer filters;
+	RegexManager rxman;
 
 	std::vector<std::string> op_args;
 	op_args.push_back("tee > " + articleFile.get_path());
@@ -791,14 +818,14 @@ TEST_CASE("OP_PIPE_TO pipes an article's content to an external command",
 	item->set_pubDate(test_pubDate);
 	item->set_description(test_description);
 
-	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, &filters, &cfg);
+	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, filters, &cfg, rxman);
 
 	feed->add_item(item);
 	itemlist.set_feed(feed);
 
 	REQUIRE_NOTHROW(itemlist.process_op(OP_PIPE_TO, true, &op_args));
 
-	TestHelpers::AssertArticleFileContent(articleFile.get_path(),
+	TestHelpers::assert_article_file_content(articleFile.get_path(),
 		test_title,
 		test_author,
 		test_pubDate_str,
