@@ -6,7 +6,13 @@
 #include <unistd.h>
 
 #include "3rd-party/catch.hpp"
-#include "test-helpers.h"
+#include "htmlrenderer.h"
+#include "rs_utils.h"
+#include "test-helpers/chdir.h"
+#include "test-helpers/envvar.h"
+#include "test-helpers/stringmaker/optional.h"
+#include "test-helpers/tempdir.h"
+#include "test-helpers/tempfile.h"
 
 using namespace newsboat;
 
@@ -92,6 +98,16 @@ TEST_CASE(
 		REQUIRE(tokens[0] == "foo \\xxx");
 		REQUIRE(tokens[1] == " ");
 	}
+
+	SECTION("Closing double quote marks the end of a token") {
+		tokens = utils::tokenize_quoted(R"(set browser "mpv %u";)");
+
+		REQUIRE(tokens.size() == 4);
+		REQUIRE(tokens[0] == "set");
+		REQUIRE(tokens[1] == "browser");
+		REQUIRE(tokens[2] == "mpv %u");
+		REQUIRE(tokens[3] == ";");
+	}
 }
 
 TEST_CASE("tokenize_quoted() implicitly closes quotes at the end of the string",
@@ -108,6 +124,10 @@ TEST_CASE("tokenize_quoted() implicitly closes quotes at the end of the string",
 	REQUIRE(tokens[0] == "\\");
 	REQUIRE(tokens[1] == "and");
 	REQUIRE(tokens[2] == "some other stuff");
+
+	tokens = utils::tokenize_quoted(R"("abc\)");
+	REQUIRE(tokens.size() == 1);
+	REQUIRE(tokens[0] == "abc");
 }
 
 TEST_CASE(
@@ -117,6 +137,26 @@ TEST_CASE(
 {
 	std::vector<std::string> tokens;
 
+	tokens = utils::tokenize_quoted(R"_("")_");
+	REQUIRE(tokens.size() == 1);
+	REQUIRE(tokens[0] == "");
+
+	tokens = utils::tokenize_quoted(R"_("\\")_");
+	REQUIRE(tokens.size() == 1);
+	REQUIRE(tokens[0] == R"_(\)_");
+
+	tokens = utils::tokenize_quoted(R"_("#\\")_");
+	REQUIRE(tokens.size() == 1);
+	REQUIRE(tokens[0] == R"_(#\)_");
+
+	tokens = utils::tokenize_quoted(R"_("'#\\'")_");
+	REQUIRE(tokens.size() == 1);
+	REQUIRE(tokens[0] == R"_('#\')_");
+
+	tokens = utils::tokenize_quoted(R"_("'#\\ \\'")_");
+	REQUIRE(tokens.size() == 1);
+	REQUIRE(tokens[0] == R"_('#\ \')_");
+
 	tokens = utils::tokenize_quoted("\"\\\\\\\\");
 	REQUIRE(tokens.size() == 1);
 	REQUIRE(tokens[0] == "\\\\");
@@ -132,6 +172,19 @@ TEST_CASE(
 	tokens = utils::tokenize_quoted("\"\\\\\\\\\\\\\"");
 	REQUIRE(tokens.size() == 1);
 	REQUIRE(tokens[0] == "\\\\\\");
+
+	// https://github.com/newsboat/newsboat/issues/642
+	tokens = utils::tokenize_quoted(R"("\\bgit\\b")");
+	REQUIRE(tokens.size() == 1);
+	REQUIRE(tokens[0] == R"(\bgit\b)");
+
+	// https://github.com/newsboat/newsboat/issues/536
+	tokens = utils::tokenize_quoted(
+			R"(browser "/Applications/Google\\ Chrome.app/Contents/MacOS/Google\\ Chrome --app %u")");
+	REQUIRE(tokens.size() == 2);
+	REQUIRE(tokens[0] == "browser");
+	REQUIRE(tokens[1] ==
+		R"(/Applications/Google\ Chrome.app/Contents/MacOS/Google\ Chrome --app %u)");
 }
 
 TEST_CASE("tokenize_quoted() doesn't un-escape escaped backticks", "[utils]")
@@ -233,6 +286,24 @@ TEST_CASE("tokenize_nl() split a string into delimiters and fields", "[utils]")
 		REQUIRE(tokens[2] == "rst\nsecond\nth");
 		REQUIRE(tokens[4] == "rd");
 	}
+
+	SECTION("no non-delimiter text") {
+		SECTION("single newline") {
+			tokens = utils::tokenize_nl("\n");
+
+			REQUIRE(tokens.size() == 1);
+			REQUIRE(tokens[0] == "\n");
+		}
+
+		SECTION("multiple newlines") {
+			tokens = utils::tokenize_nl("\n\n\n");
+
+			REQUIRE(tokens.size() == 3);
+			REQUIRE(tokens[0] == "\n");
+			REQUIRE(tokens[1] == "\n");
+			REQUIRE(tokens[2] == "\n");
+		}
+	}
 }
 
 TEST_CASE(
@@ -373,24 +444,40 @@ TEST_CASE("extract_filter()", "[utils]")
 
 	REQUIRE(filter == "~/bin/script.sh");
 	REQUIRE(url == "https://newsboat.org");
+
+	utils::extract_filter("filter::https://newsboat.org", filter, url);
+
+	REQUIRE(filter == "");
+	REQUIRE(url == "https://newsboat.org");
+
+	utils::extract_filter("filter:https://newsboat.org", filter, url);
+
+	REQUIRE(filter == "https");
+	REQUIRE(url == "//newsboat.org");
+
+	utils::extract_filter("filter:foo:", filter, url);
+
+	REQUIRE(filter == "foo");
+	REQUIRE(url == "");
+
+	utils::extract_filter("filter:", filter, url);
+
+	REQUIRE(filter == "");
+	REQUIRE(url == "");
 }
 
 TEST_CASE("run_program()", "[utils]")
 {
-	char* argv[4];
-	char cat[] = "cat";
-	argv[0] = cat;
+	const char* argv[4];
+	argv[0] = "cat";
 	argv[1] = nullptr;
 	REQUIRE(utils::run_program(
 			argv, "this is a multine-line\ntest string") ==
 		"this is a multine-line\ntest string");
 
-	char echo[] = "echo";
-	char dashn[] = "-n";
-	char helloworld[] = "hello world";
-	argv[0] = echo;
-	argv[1] = dashn;
-	argv[2] = helloworld;
+	argv[0] = "echo";
+	argv[1] = "-n";
+	argv[2] = "hello world";
 	argv[3] = nullptr;
 	REQUIRE(utils::run_program(argv, "") == "hello world");
 }
@@ -417,11 +504,11 @@ TEST_CASE("run_command() executes the given command with a given argument",
 	struct stat sb;
 	int result = 0;
 
-	// Busy-wait for 10 tries of 10 milliseconds each, waiting for `touch` to
+	// Busy-wait for 100 tries of 10 milliseconds each, waiting for `touch` to
 	// create the file. Usually it happens quickly, and the loop exists on the
 	// first try; but sometimes on CI it takes longer for `touch` to finish, so
 	// we need a slightly longer wait.
-	int tries = 10;
+	int tries = 100;
 	while (tries-- > 0) {
 		::usleep(10 * 1000);
 
@@ -458,10 +545,15 @@ TEST_CASE("resolve_tilde() replaces ~ with the path to the $HOME directory",
 {
 	TestHelpers::EnvVar envVar("HOME");
 	envVar.set("test");
-	REQUIRE(utils::resolve_tilde("~") == "test");
+	REQUIRE(utils::resolve_tilde("~") == "test/");
 	REQUIRE(utils::resolve_tilde("~/") == "test/");
 	REQUIRE(utils::resolve_tilde("~/dir") == "test/dir");
 	REQUIRE(utils::resolve_tilde("/home/~") == "/home/~");
+	REQUIRE(
+		utils::resolve_tilde("~/foo/bar") ==
+		"test/foo/bar"
+	);
+	REQUIRE(utils::resolve_tilde("/foo/bar") == "/foo/bar");
 }
 
 TEST_CASE("resolve_relative() returns an absolute file path relative to another",
@@ -632,9 +724,15 @@ TEST_CASE("strwidth_stfl()", "[utils]")
 {
 	REQUIRE(utils::strwidth_stfl("") == 0);
 
-	REQUIRE(utils::strwidth_stfl("x<hi>x") == 3);
+	REQUIRE(utils::strwidth_stfl("x<hi>x") == 2);
 
-	REQUIRE(utils::strwidth_stfl("x<>x") == 4);
+	REQUIRE(utils::strwidth_stfl("x<longtag>x</>") == 2);
+
+	REQUIRE(utils::strwidth_stfl("x<>x") == 3);
+
+	REQUIRE(utils::strwidth_stfl("x<>hi>x") == 6);
+
+	REQUIRE(utils::strwidth_stfl("x<>y<>z") == 5);
 
 	REQUIRE(utils::strwidth_stfl(utils::wstr2str(L"\uF91F")) == 2);
 	REQUIRE(utils::strwidth_stfl("\07") == 0);
@@ -801,6 +899,18 @@ TEST_CASE("utils::make_title extracts possible title from URL", "[utils]")
 	}
 }
 
+TEST_CASE("run_interactively", "[utils]")
+{
+	SECTION("echo hello should return 0") {
+		int32_t result = utils::run_interactively("echo hello", "test");
+		REQUIRE(result == 0);
+	}
+	SECTION("exit 1 should return 1") {
+		int32_t result = utils::run_interactively("exit 1", "test");
+		REQUIRE(result == 1);
+	}
+}
+
 TEST_CASE("remove_soft_hyphens remove all U+00AD characters from a string",
 	"[utils]")
 {
@@ -849,13 +959,13 @@ TEST_CASE(
 		"A\u3042B");
 
 	SECTION("returns an empty string if the given string is empty") {
-		REQUIRE(utils::substr_with_width("", 0).empty());
-		REQUIRE(utils::substr_with_width("", 1).empty());
+		REQUIRE(utils::substr_with_width("", 0) == "");
+		REQUIRE(utils::substr_with_width("", 1) == "");
 	}
 
 	SECTION("returns an empty string if the given width is zero") {
-		REQUIRE(utils::substr_with_width("world", 0).empty());
-		REQUIRE(utils::substr_with_width("", 0).empty());
+		REQUIRE(utils::substr_with_width("world", 0) == "");
+		REQUIRE(utils::substr_with_width("", 0) == "");
 	}
 
 	SECTION("doesn't split single codepoint in two") {
@@ -865,22 +975,19 @@ TEST_CASE(
 		REQUIRE(utils::substr_with_width(data, 5) == "\u3042\u3044");
 	}
 
-	SECTION("doesn't count a width of STFL tag") {
+	SECTION("handles angular brackets as regular characters") {
 		REQUIRE(utils::substr_with_width("ＡＢＣ<b>ＤＥ</b>Ｆ", 9) ==
-			"ＡＢＣ<b>Ｄ");
+			"ＡＢＣ<b>");
 		REQUIRE(utils::substr_with_width("<foobar>ＡＢＣ", 4) ==
-			"<foobar>ＡＢ");
+			"<foo");
 		REQUIRE(utils::substr_with_width("a<<xyz>>bcd", 3) ==
-			"a<<xyz>>b"); // tag: "<<xyz>"
+			"a<<");
 		REQUIRE(utils::substr_with_width("ＡＢＣ<b>ＤＥ", 10) ==
-			"ＡＢＣ<b>ＤＥ");
+			"ＡＢＣ<b>");
 		REQUIRE(utils::substr_with_width("a</>b</>c</>", 2) ==
-			"a</>b</>");
-	}
-
-	SECTION("count a width of escaped less-than mark") {
-		REQUIRE(utils::substr_with_width("<><><>", 2) == "<><>");
-		REQUIRE(utils::substr_with_width("a<>b<>c", 3) == "a<>b");
+			"a<");
+		REQUIRE(utils::substr_with_width("<><><>", 2) == "<>");
+		REQUIRE(utils::substr_with_width("a<>b<>c", 3) == "a<>");
 	}
 
 	SECTION("treat non-printable has zero width") {
@@ -892,6 +999,62 @@ TEST_CASE(
 	}
 }
 
+TEST_CASE(
+	"substr_with_width_stfl() returns a longest substring fits to the given "
+	"width",
+	"[utils]")
+{
+	REQUIRE(utils::substr_with_width_stfl("a", 1) == "a");
+	REQUIRE(utils::substr_with_width_stfl("a", 2) == "a");
+	REQUIRE(utils::substr_with_width_stfl("ab", 1) == "a");
+	REQUIRE(utils::substr_with_width_stfl("abc", 1) == "a");
+	REQUIRE(utils::substr_with_width_stfl("A\u3042B\u3044C\u3046", 5) ==
+		"A\u3042B");
+
+	SECTION("returns an empty string if the given string is empty") {
+		REQUIRE(utils::substr_with_width_stfl("", 0) == "");
+		REQUIRE(utils::substr_with_width_stfl("", 1) == "");
+	}
+
+	SECTION("returns an empty string if the given width is zero") {
+		REQUIRE(utils::substr_with_width_stfl("world", 0) == "");
+		REQUIRE(utils::substr_with_width_stfl("", 0) == "");
+	}
+
+	SECTION("doesn't split single codepoint in two") {
+		std::string data = "\u3042\u3044\u3046";
+		REQUIRE(utils::substr_with_width_stfl(data, 1) == "");
+		REQUIRE(utils::substr_with_width_stfl(data, 3) == "\u3042");
+		REQUIRE(utils::substr_with_width_stfl(data, 5) == "\u3042\u3044");
+	}
+
+	SECTION("doesn't count a width of STFL tag") {
+		REQUIRE(utils::substr_with_width_stfl("ＡＢＣ<b>ＤＥ</b>Ｆ", 9) ==
+			"ＡＢＣ<b>Ｄ");
+		REQUIRE(utils::substr_with_width_stfl("<foobar>ＡＢＣ", 4) ==
+			"<foobar>ＡＢ");
+		REQUIRE(utils::substr_with_width_stfl("a<<xyz>>bcd", 3) ==
+			"a<<xyz>>b"); // tag: "<<xyz>"
+		REQUIRE(utils::substr_with_width_stfl("ＡＢＣ<b>ＤＥ", 10) ==
+			"ＡＢＣ<b>ＤＥ");
+		REQUIRE(utils::substr_with_width_stfl("a</>b</>c</>", 2) ==
+			"a</>b</>");
+	}
+
+	SECTION("count a width of escaped less-than mark") {
+		REQUIRE(utils::substr_with_width_stfl("<><><>", 2) == "<><>");
+		REQUIRE(utils::substr_with_width_stfl("a<>b<>c", 3) == "a<>b");
+	}
+
+	SECTION("treat non-printable has zero width") {
+		REQUIRE(utils::substr_with_width_stfl("\x01\x02"
+				"abc",
+				1) ==
+			"\x01\x02"
+			"a");
+	}
+}
+
 TEST_CASE("getcwd() returns current directory of the process", "[utils]")
 {
 	SECTION("Returns non-empty string") {
@@ -971,13 +1134,15 @@ TEST_CASE("strnaturalcmp() compares strings using natural numeric ordering",
 
 TEST_CASE(
 	"is_valid_podcast_type() returns true if supplied MIME type "
-	"is audio or a container",
+	"is audio, video, or a container",
 	"[utils]")
 {
 	REQUIRE(utils::is_valid_podcast_type("audio/mpeg"));
 	REQUIRE(utils::is_valid_podcast_type("audio/mp3"));
 	REQUIRE(utils::is_valid_podcast_type("audio/x-mp3"));
 	REQUIRE(utils::is_valid_podcast_type("audio/ogg"));
+	REQUIRE(utils::is_valid_podcast_type("video/x-matroska"));
+	REQUIRE(utils::is_valid_podcast_type("video/webm"));
 	REQUIRE(utils::is_valid_podcast_type("application/ogg"));
 
 	REQUIRE_FALSE(utils::is_valid_podcast_type("image/jpeg"));
@@ -986,6 +1151,36 @@ TEST_CASE(
 	REQUIRE_FALSE(utils::is_valid_podcast_type("application/zip"));
 }
 
+TEST_CASE("podcast_mime_to_link_type() returns HtmlRenderer's LinkType that "
+	"corresponds to the given podcast MIME type",
+	"[utils]")
+{
+	SECTION("Valid podcast MIME types") {
+		const auto check = [](std::string mime, LinkType expected) {
+			REQUIRE(utils::podcast_mime_to_link_type(mime) == expected);
+		};
+
+		check("audio/mpeg", LinkType::AUDIO);
+		check("audio/mp3", LinkType::AUDIO);
+		check("audio/x-mp3", LinkType::AUDIO);
+		check("audio/ogg", LinkType::AUDIO);
+		check("video/x-matroska", LinkType::VIDEO);
+		check("video/webm", LinkType::VIDEO);
+		check("application/ogg", LinkType::AUDIO);
+	}
+
+	SECTION("Sets `ok` to `false` if given MIME type is not a podcast type") {
+		const auto check = [](std::string mime) {
+			REQUIRE(utils::podcast_mime_to_link_type(mime) == nonstd::nullopt);
+		};
+
+		check("image/jpeg");
+		check("image/png");
+		check("text/plain");
+		check("application/zip");
+	}
+}
+
 TEST_CASE(
 	"is_valid_color() returns false for things that aren't valid STFL "
 	"colors",
